@@ -12,16 +12,16 @@
 FLOCK=$(command -v flock)
 
 if [ -d "$1" ]; then
-  ABSDIR=$(cd "$1"; pwd -P)
+  ABSDIR=$(cd "$1" && pwd -P)
 elif [ -f "$1" ]; then
-  ABSDIR=$(cd "$(dirname "$1")"; pwd -P)
+  ABSDIR=$(cd "$(dirname "$1")" && pwd -P)
 else
-  ABSDIR=$(cd "$(dirname "$0")"; pwd -P)
+  ABSDIR=$(cd "$(dirname "$0")" && pwd -P)
 fi
 
 shift
-cd "${ABSDIR}"
-if [ "" != "${FLOCK}" ]; then
+cd "${ABSDIR}" || true
+if [ "${FLOCK}" ]; then
   ${FLOCK} "${ABSDIR}" -c "$@"
 else
   eval "$@"
