@@ -32,8 +32,10 @@
 #define LIBXSMM_X86_AVX512_CPX   1022
 #define LIBXSMM_X86_ALLFEAT      1999 /* all features supported which are used anywhere in LIBXSMM, this value should never be used to set arch, only for compares */
 
+/** A zero-initialized structure assumes conservative properties. */
 LIBXSMM_EXTERN_C typedef struct LIBXSMM_RETARGETABLE libxsmm_cpuid_x86_info {
-  int constant_tsc;
+  int constant_tsc; /** Timer stamp counter is monotonic. */
+  int has_context;  /** Context switches are permitted. */
 } libxsmm_cpuid_x86_info;
 
 /** Returns the target architecture and instruction set extensions. */
