@@ -1,4 +1,4 @@
-#!/usr/bin/env sh
+#!/usr/bin/env bash
 ###############################################################################
 # Copyright (c) Intel Corporation - All rights reserved.                      #
 # This file is part of the LIBXSMM library.                                   #
@@ -10,7 +10,7 @@
 # Hans Pabst (Intel Corp.)
 ###############################################################################
 
-HERE=$(cd "$(dirname "$0")"; pwd -P)
+HERE=$(cd "$(dirname "$0")" && pwd -P)
 SCRT=${HERE}/../../scripts/libxsmm_utilities.py
 FILE=cp2k-perf.txt
 
@@ -32,17 +32,17 @@ case "$1" in
   ;;
 esac
 
-if [ "" = "${!RUNS}" ]; then
+if [ -z "${RUNS}" ]; then
   RUNS=RUNS11
 fi
 
-if [ "" != "$1" ]; then
+if [ "$1" ]; then
   SIZE=$1
   shift
 else
   SIZE=0
 fi
-if [ "" != "$1" ]; then
+if [ "$1" ]; then
   FILE=$1
   shift
 fi
