@@ -1,30 +1,10 @@
 /******************************************************************************
-** Copyright (c) 2016-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Alexander Heinecke (Intel Corp.)
 ******************************************************************************/
@@ -55,6 +35,7 @@ int main(int argc, char* argv[])
   float *naive_beta, *naive_gamma, *naive_delbeta, *naive_delgamma, *naive_expectval, *naive_rcpstddev, *naive_variance;
   libxsmm_bfloat16 *input_libxsmm, *output_libxsmm, *input_add_libxsmm, *delinput_libxsmm, *deloutput_libxsmm, *delinput_add_libxsmm;
   float *beta_libxsmm, *gamma_libxsmm, *delbeta_libxsmm, *delgamma_libxsmm, *expectval_libxsmm, *rcpstddev_libxsmm, *variance_libxsmm;
+  unsigned char* relumask_libxsmm;
 
   int ifhp, ifwp, ofhp, ofwp, ofh, ofw;
   int stride_h, stride_w;
@@ -110,6 +91,7 @@ int main(int argc, char* argv[])
   libxsmm_dnn_tensor*  libxsmm_expectval;
   libxsmm_dnn_tensor*  libxsmm_rcpstddev;
   libxsmm_dnn_tensor*  libxsmm_variance;
+  libxsmm_dnn_tensor*  libxsmm_relumask;
   libxsmm_dnn_tensor_datalayout* libxsmm_layout;
   libxsmm_dnn_err_t status;
   libxsmm_dnn_err_t global_status = LIBXSMM_DNN_SUCCESS;
@@ -149,8 +131,8 @@ int main(int argc, char* argv[])
     printf("norm type needs to be 0 or 1\n");
     return -1;
   }
-  if ((fuse_type < 0) && (fuse_type > 3)) {
-    printf("fuse type needs to be 0, 1, 2, or 3\n");
+  if ((fuse_type < 0) || (fuse_type > 5)) {
+    printf("fuse type needs to be 0, 1, 2, 3, 4 or 5\n");
     return -1;
   }
 
@@ -249,7 +231,9 @@ int main(int argc, char* argv[])
   rcpstddev_libxsmm              = (float*)libxsmm_aligned_malloc( nFm*               sizeof(float), 2097152);
   variance_libxsmm               = (float*)libxsmm_aligned_malloc( nFm*               sizeof(float), 2097152);
 
-  /* initialize data */
+  relumask_libxsmm               = (unsigned char*)libxsmm_aligned_malloc( nImg*nFm*ofhp*ofwp*sizeof(unsigned char), 2097152);
+
+ /* initialize data */
   init_buf( naive_input, nImg*nFm*ifh*ifw, 0, 0 );
   copy_internal_nchw( naive_input_pad , naive_input, nImg, nFm, ifh, ifw, pad_h_in, pad_w_in );
   init_buf( naive_delinput, nImg*nFm*ifh*ifw, 0, 0 );
@@ -270,12 +254,12 @@ int main(int argc, char* argv[])
   set_zeropad_nchw(naive_output_pad,       nImg, nFm, ofhp, ofwp, pad_h_out, pad_w_out);
   set_zeropad_nchw(naive_deloutput_pad,    nImg, nFm, ofhp, ofwp, pad_h_out, pad_w_out);
 
-  libxsmm_rne_convert_fp32_bfp16( naive_input_pad,        naive_input_pad_bf16,        nImg*nFm*ifhp*ifwp );
-  libxsmm_rne_convert_fp32_bfp16( naive_delinput_pad,     naive_delinput_pad_bf16,     nImg*nFm*ifhp*ifwp );
-  libxsmm_rne_convert_fp32_bfp16( naive_input_add_pad,    naive_input_add_pad_bf16,    nImg*nFm*ifhp*ifwp );
-  libxsmm_rne_convert_fp32_bfp16( naive_delinput_add_pad, naive_delinput_add_pad_bf16, nImg*nFm*ifhp*ifwp );
-  libxsmm_rne_convert_fp32_bfp16( naive_output_pad,       naive_output_pad_bf16,       nImg*nFm*ofhp*ofwp );
-  libxsmm_rne_convert_fp32_bfp16( naive_deloutput_pad,    naive_deloutput_pad_bf16,    nImg*nFm*ofhp*ofwp );
+  libxsmm_rne_convert_fp32_bf16( naive_input_pad,        naive_input_pad_bf16,        nImg*nFm*ifhp*ifwp );
+  libxsmm_rne_convert_fp32_bf16( naive_delinput_pad,     naive_delinput_pad_bf16,     nImg*nFm*ifhp*ifwp );
+  libxsmm_rne_convert_fp32_bf16( naive_input_add_pad,    naive_input_add_pad_bf16,    nImg*nFm*ifhp*ifwp );
+  libxsmm_rne_convert_fp32_bf16( naive_delinput_add_pad, naive_delinput_add_pad_bf16, nImg*nFm*ifhp*ifwp );
+  libxsmm_rne_convert_fp32_bf16( naive_output_pad,       naive_output_pad_bf16,       nImg*nFm*ofhp*ofwp );
+  libxsmm_rne_convert_fp32_bf16( naive_deloutput_pad,    naive_deloutput_pad_bf16,    nImg*nFm*ofhp*ofwp );
 
   init_buf(naive_beta,      nFm, 0, 0);
   init_buf(naive_gamma,     nFm, 0, 0);
@@ -315,7 +299,8 @@ int main(int argc, char* argv[])
     printf("##########################################\n");
 
     /* setup LIBXSMM handle */
-    fusedbatchnorm_desc.N = nImg;
+    fusedbatchnorm_desc.partN = nImg;
+    fusedbatchnorm_desc.fullN = nImg;
     fusedbatchnorm_desc.C = nFm;
     fusedbatchnorm_desc.H = ifh;
     fusedbatchnorm_desc.W = ifw;
@@ -340,6 +325,10 @@ int main(int argc, char* argv[])
         fusedbatchnorm_desc.fuse_ops = LIBXSMM_DNN_FUSEDBN_OPS_BN_ELTWISE;
       } else if ( fuse_type == 3 ) {
         fusedbatchnorm_desc.fuse_ops = LIBXSMM_DNN_FUSEDBN_OPS_BN_ELTWISE_RELU;
+      } else if ( fuse_type == 4 ) {
+        fusedbatchnorm_desc.fuse_ops = LIBXSMM_DNN_FUSEDBN_OPS_BN_RELU_WITH_MASK;
+      } else if ( fuse_type == 5 ) {
+        fusedbatchnorm_desc.fuse_ops = LIBXSMM_DNN_FUSEDBN_OPS_BN_ELTWISE_RELU_WITH_MASK;
       } else {
         /* shouldn't happen */
         return -1;
@@ -353,6 +342,10 @@ int main(int argc, char* argv[])
         fusedbatchnorm_desc.fuse_ops = LIBXSMM_DNN_FUSEDBN_OPS_BNSCALE_ELTWISE;
       } else if ( fuse_type == 3 ) {
         fusedbatchnorm_desc.fuse_ops = LIBXSMM_DNN_FUSEDBN_OPS_BNSCALE_ELTWISE_RELU;
+      } else if ( fuse_type == 4 ) {
+        fusedbatchnorm_desc.fuse_ops = LIBXSMM_DNN_FUSEDBN_OPS_BNSCALE_RELU_WITH_MASK;
+      } else if ( fuse_type == 5 ) {
+        fusedbatchnorm_desc.fuse_ops = LIBXSMM_DNN_FUSEDBN_OPS_BNSCALE_ELTWISE_RELU_WITH_MASK;
       } else {
         /* shouldn't happen */
         return -1;
@@ -415,6 +408,10 @@ int main(int argc, char* argv[])
     libxsmm_variance  = libxsmm_dnn_link_tensor( libxsmm_layout, variance_libxsmm, &status ); CHKERR_LIBXSMM_DNN( status );
     libxsmm_dnn_destroy_tensor_datalayout( libxsmm_layout );
 
+    libxsmm_layout = libxsmm_dnn_fusedbatchnorm_create_tensor_datalayout( libxsmm_handle, LIBXSMM_DNN_RELU_MASK, &status ); CHKERR_LIBXSMM_DNN( status );
+    libxsmm_relumask  = libxsmm_dnn_link_tensor( libxsmm_layout, relumask_libxsmm, &status ); CHKERR_LIBXSMM_DNN( status );
+    libxsmm_dnn_destroy_tensor_datalayout( libxsmm_layout );
+
     /* copy in data to LIBXSMM format */
     /* we can also use the layout functions and set the data on our
        own external to the library */
@@ -439,6 +436,7 @@ int main(int argc, char* argv[])
     CHKERR_LIBXSMM_DNN( libxsmm_dnn_fusedbatchnorm_bind_tensor( libxsmm_handle, libxsmm_expectval,    LIBXSMM_DNN_CHANNEL_EXPECTVAL ) );
     CHKERR_LIBXSMM_DNN( libxsmm_dnn_fusedbatchnorm_bind_tensor( libxsmm_handle, libxsmm_rcpstddev,    LIBXSMM_DNN_CHANNEL_RCPSTDDEV ) );
     CHKERR_LIBXSMM_DNN( libxsmm_dnn_fusedbatchnorm_bind_tensor( libxsmm_handle, libxsmm_variance,     LIBXSMM_DNN_CHANNEL_VARIANCE ) );
+    CHKERR_LIBXSMM_DNN( libxsmm_dnn_fusedbatchnorm_bind_tensor( libxsmm_handle, libxsmm_relumask,     LIBXSMM_DNN_RELU_MASK ) );
 
     /* let's allocate and bind scratch */
     scratch_size = libxsmm_dnn_fusedbatchnorm_get_scratch_size( libxsmm_handle, &status );
@@ -680,6 +678,7 @@ int main(int argc, char* argv[])
     CHKERR_LIBXSMM_DNN( libxsmm_dnn_fusedbatchnorm_release_tensor( libxsmm_handle, LIBXSMM_DNN_CHANNEL_EXPECTVAL ) );
     CHKERR_LIBXSMM_DNN( libxsmm_dnn_fusedbatchnorm_release_tensor( libxsmm_handle, LIBXSMM_DNN_CHANNEL_RCPSTDDEV ) );
     CHKERR_LIBXSMM_DNN( libxsmm_dnn_fusedbatchnorm_release_tensor( libxsmm_handle, LIBXSMM_DNN_CHANNEL_VARIANCE ) );
+    CHKERR_LIBXSMM_DNN( libxsmm_dnn_fusedbatchnorm_release_tensor( libxsmm_handle, LIBXSMM_DNN_RELU_MASK) );
 
     CHKERR_LIBXSMM_DNN( libxsmm_dnn_destroy_tensor( libxsmm_input ) );
     CHKERR_LIBXSMM_DNN( libxsmm_dnn_destroy_tensor( libxsmm_delinput ) );
@@ -694,6 +693,7 @@ int main(int argc, char* argv[])
     CHKERR_LIBXSMM_DNN( libxsmm_dnn_destroy_tensor( libxsmm_expectval ) );
     CHKERR_LIBXSMM_DNN( libxsmm_dnn_destroy_tensor( libxsmm_rcpstddev ) );
     CHKERR_LIBXSMM_DNN( libxsmm_dnn_destroy_tensor( libxsmm_variance ) );
+    CHKERR_LIBXSMM_DNN( libxsmm_dnn_destroy_tensor( libxsmm_relumask ) );
     CHKERR_LIBXSMM_DNN( libxsmm_dnn_destroy_fusedbatchnorm( libxsmm_handle ) );
   }
 
@@ -742,6 +742,7 @@ int main(int argc, char* argv[])
   libxsmm_free(expectval_libxsmm);
   libxsmm_free(rcpstddev_libxsmm);
   libxsmm_free(variance_libxsmm);
+  libxsmm_free(relumask_libxsmm);
 
   { const char *const env_check_scale = getenv("CHECK_SCALE");
     const double check_scale = LIBXSMM_ABS(0 == env_check_scale ? 1.0 : atof(env_check_scale));
@@ -754,6 +755,6 @@ int main(int argc, char* argv[])
   /* some empty lines at the end */
   printf("\n\n\n");
 
-  return EXIT_SUCCESS;
+  return global_status;
 }
 
