@@ -1,30 +1,10 @@
 /******************************************************************************
-** Copyright (c) 2017-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Sasikanth Avancha, Dhiraj Kalamkar, Alexander Heinecke (Intel Corp.)
 ******************************************************************************/
@@ -159,6 +139,7 @@ ConvNode::ConvNode(ConvParams* p, MLEngine* e): NNNode(p, e)
   // size of master weights -- FP32
   wsize = welem*sizeof(float);
 
+  gparams_.num_numa_nodes = NUM_NUMA_NODES;
   tenWeightData_->setBufferSize(wsize);
 
   wfiller_type_ = p->get_weight_filler_type();
@@ -236,7 +217,11 @@ ConvNode::ConvNode(ConvParams* p, MLEngine* e): NNNode(p, e)
       else if(in_dtype == DT_BF16)
       {
         tenWeightDiff_->setDataType(DT_BF16);
+#ifdef BF16_MLSL
         tenWeightDiff_->setBufferSize(welem*sizeof(libxsmm_bfloat16));
+#else
+        tenWeightDiff_->setBufferSize(welem*sizeof(float));
+#endif
       }
 
       if(bias_term)
@@ -362,6 +347,7 @@ ConvNode::ConvNode(ConvParams* p, MLEngine* e): NNNode(p, e)
   size_t opIdx = s->AddOperation(myRegInfo, e->get_distribution());
   this->op_ = s->GetOperation(opIdx);
   s->DeleteOperationRegInfo(myRegInfo);
+  e->get_wtgrad_comms_vec().push_back(op_);
 #endif
 
   configure(p->get_compute_engine());
@@ -369,7 +355,6 @@ ConvNode::ConvNode(ConvParams* p, MLEngine* e): NNNode(p, e)
 
 void ConvNode::fillWeightBuffers(TensorBuf* tBuf, int buftype, long long int size)
 {
-  int dtype = DT_FLOAT;
   void *ptr = tBuf->getBuffer();
 
 #ifdef USE_MLSL
@@ -378,26 +363,25 @@ void ConvNode::fillWeightBuffers(TensorBuf* tBuf, int buftype, long long int siz
   unsigned int node_id = 0;
 #endif
 
+  int ic = gparams_.nInput;
+  int oc = gparams_.nOutput;
+  int kh = gparams_.kh;
+  int kw = gparams_.kw;
+  int g = gparams_.group;
+  int fanin = (ic * kh * kw)/g;
+  int fanout = (oc * kh * kw)/g;
+  int welem = ic * oc * kh * kw;
+
   if(buftype == DATA)
   {
-    int n = gparams_.batch_size;
-    int ic = gparams_.nInput;
-    int oc = gparams_.nOutput;
-    int kh = gparams_.kh;
-    int kw = gparams_.kw;
-    int g = gparams_.group;
-    int fanin = (ic * kh * kw)/g;
-    int fanout = (oc * kh * kw)/g;
-    int welem = ic * oc * kh * kw;
-
-    initBuffer(ptr, dtype, variance_norm_, fanin, fanout, welem*sizeof(float), wfiller_type_, (unsigned int)(node_id+PRIME_SEED), std_);
+    if(node_id == 0)
+      initBuffer(ptr, variance_norm_, fanin, fanout, welem*sizeof(float), wfiller_type_, std_);
 
 #ifdef USE_MLSL
-    if(dtype == DT_FLOAT)
-      MPI_Bcast(ptr, welem, MPI_FLOAT, 0, MPI_COMM_WORLD);
+    MPI_Bcast(ptr, welem, MPI_FLOAT, 0, MPI_COMM_WORLD);
 #endif
   }
-  else
+  else if(buftype == HISTORY || buftype == DIFF)
     memset(ptr, 0, size);
 }
 
@@ -412,12 +396,11 @@ void ConvNode::fillWeightMultipliers(float* lr, float* decay, long long int size
 
 void ConvNode::fillBiasBuffers(TensorBuf* tBuf, int buftype, long long int size)
 {
-  int dtype = DT_FLOAT;
   void *ptr = tBuf->getBuffer();
 
   if(buftype == DATA)
   {
-    initConstantBuffer(ptr, dtype, size, "CONSTANT", value_);
+    initConstantBuffer(ptr, size, "CONSTANT", value_);
   }
   else
     memset(ptr, 0, size);
@@ -468,6 +451,7 @@ void ConvNode::Checkpoint(TensorBuf *tBuf, string name, string format)
       f = fopen(name.c_str(), "wb");
       if(f != NULL)
       {
+#if 0
         if(name.find("wt") != name.npos)
         {
           ptr = _mm_malloc(bytes, 64);
@@ -475,13 +459,16 @@ void ConvNode::Checkpoint(TensorBuf *tBuf, string name, string format)
           impl->dumpBuffer(tBuf, ptr);
         }
         else
+#endif
           ptr = tBuf->getBuffer();
 
         size_t b = fwrite(ptr, 1, bytes, f);
         assert((long long int)b == bytes);
 
+#if 0
         if(name.find("wt") != name.npos)
           _mm_free(ptr);
+#endif
       }
       else
         printf("Warning: could not checkpoint to file %s\n",name.c_str());
@@ -491,6 +478,7 @@ void ConvNode::Checkpoint(TensorBuf *tBuf, string name, string format)
       f = fopen(name.c_str(), "w");
       if(f != NULL)
       {
+#if 0
         if(name.find("wt") != name.npos)
         {
           ptr = _mm_malloc(bytes, 64);
@@ -498,13 +486,16 @@ void ConvNode::Checkpoint(TensorBuf *tBuf, string name, string format)
           impl->dumpBuffer(tBuf, ptr);
         }
         else
+#endif
           ptr = tBuf->getBuffer();
 
         for(int i=0; i<bytes/sizeof(float); i++)
           fprintf(f, "%f\n", *((float*)ptr + i));
 
+#if 0
         if(name.find("wt") != name.npos)
           _mm_free(ptr);
+#endif
       }
       else
         printf("Warning: could not checkpoint to file %s\n",name.c_str());
@@ -542,16 +533,71 @@ void ConvNode::convert_f32_bf16(float* in, libxsmm_bfloat16* out, int len)
 
 void ConvNode::convert_bf16_f32(libxsmm_bfloat16* in, float* out, int len)
 {
-  int i;
+#if 1
 
 #ifdef _OPENMP
-#pragma omp parallel for private(i)
+#pragma omp parallel
 #endif
-  for ( i = 0; i < len; i+=16 ) {
-    __m256i vbfp16    = _mm256_loadu_si256( (const __m256i*)(in+i) );
-    __m512  vfp32     = gxm_bfp16_to_fp32_avx512f( vbfp16 );
-    _mm512_storeu_ps( out+i, vfp32 );
+  {
+    int tid = omp_get_thread_num();
+    int ntps = gparams_.num_threads/gparams_.num_numa_nodes;
+    int n = tid/ntps;
+    if(n == 0)
+    {
+      int lenv = len/16;
+      int rem = lenv % ntps;
+      int jobs = (rem == 0) ? (lenv/ntps)*16 : ((lenv-rem)/ntps)*16;
+      int tb = (tid*jobs < len) ? tid*jobs : len;
+      int te = ((tid+1)*jobs < len) ? (tid+1)*jobs : len;
+
+      for (int i = tb; i < te; i+=16 ) {
+        __m256i vbfp16    = _mm256_loadu_si256( (const __m256i*)(in+i) );
+        __m512  vfp32     = gxm_bfp16_to_fp32_avx512f( vbfp16 );
+        _mm512_storeu_ps( out+i, vfp32 );
+      }
+
+      //Remainder processing
+      if(tid == 0)
+      {
+        if(rem > 0)
+        {
+          for(int i=ntps*jobs; i<len; i+=16)
+          {
+            __m256i vbfp16    = _mm256_loadu_si256( (const __m256i*)(in+i) );
+            __m512  vfp32     = gxm_bfp16_to_fp32_avx512f( vbfp16 );
+            _mm512_storeu_ps( out+i, vfp32 );
+          }
+        }
+      }
+    }
   }
+#else
+
+#ifdef _OPENMP
+#pragma omp parallel
+#endif
+  {
+    int tid = omp_get_thread_num();
+    int ntps = gparams_.num_threads/gparams_.num_numa_nodes;
+    int n = tid/ntps;
+
+    if(n == 0)
+    {
+      union libxsmm_bfloat16_hp delwt_32_0;
+      delwt_32_0.i[0] = 0;
+
+      int jobs = (len % ntps == 0) ? len/ntps : len/ntps + 1;
+      int tb = (tid*jobs < len) ? tid*jobs : len;
+      int te = ((tid+1)*jobs < len) ? (tid+1)*jobs : len;
+
+      for(int j=tb; j<te; j++)
+      {
+        delwt_32_0.i[1] = in[j];
+        out[j] = delwt_32_0.f;
+      }
+    }
+  }
+#endif
 }
 
 void ConvNode::forwardPropagate()
@@ -611,7 +657,9 @@ void ConvNode::forwardPropagate()
         ptr[i] = 0;
     }
 
+#ifdef CHECK_BLOWUP_FP32
     cbptr = (float*)_mm_malloc(10240*4, 64);
+#endif
 
     first_fp = false;
   }
@@ -750,7 +798,8 @@ void ConvNode::forwardPropagate()
 
         s = nname_ + "_Wt";
         float *fptr = (float*)tenWeightData_->getBuffer();
-        MeanOfLayer((char*)s.c_str(), fptr, ifm*ofm*kh*kw);
+        int w = ifm*ofm*kh*kw;
+        MeanOfLayer((char*)s.c_str(), fptr, w);
 
         if(gparams_.bias_term)
         {
@@ -841,7 +890,7 @@ void ConvNode::backPropagate()
 #ifdef CHECK_BLOWUP_FP32
   if(out_dtype == DT_FLOAT)
   {
-    for(int i=0; i<16; i++)
+    for(int i=0; i<10240; i++)
     {
       float v = ((float*)tenBotDiff_->getBuffer())[i];
       if(isnan(v) || isinf(v))
@@ -853,13 +902,27 @@ void ConvNode::backPropagate()
   }
   else if(out_dtype == DT_BF16)
   {
-    convert_bf16_f32((libxsmm_bfloat16*)tenBotDiff_->getBuffer(), cbptr, 16);
-    for(int i=0; i<16; i++)
+    convert_bf16_f32((libxsmm_bfloat16*)tenBotDiff_->getBuffer(), cbptr, 10240);
+#ifdef USE_MLSL
+    int node_id = MLSL::Environment::GetEnv().GetProcessIdx();
+#else
+    int node_id = 0;
+#endif
+    if(node_id == 0)
     {
-      if(isnan(cbptr[i]) || isinf(cbptr[i]))
+      for(int i=0; i<10240; i++)
       {
-        printf("Warning! %s layer BP activations are NaN or Inf\n", nname_.c_str());
-        exit(-1);
+        if(isnan(cbptr[i]) || isinf(cbptr[i]))
+        {
+          printf("Warning! %s layer BP activations are NaN or Inf\n", nname_.c_str());
+          MeanOfLayer((char*)((nname_+"_delin").c_str()), (libxsmm_bfloat16*)tenBotDiff_->getBuffer(), nImg*ifm*ifhp*ifwp);
+          MeanOfLayer((char*)((nname_+"_delout").c_str()), (libxsmm_bfloat16*)tenTopDiff_->getBuffer(), nImg*ofm*ofhp*ofwp);
+          MeanOfLayer((char*)((nname_+"_weight").c_str()), (libxsmm_bfloat16*)tenWeightData_->getLPBuffer(), ofm*ifm*kh*kw);
+#ifdef USE_MLSL
+          MPI_Finalize();
+#endif
+          exit(-1);
+        }
       }
     }
   }
@@ -938,8 +1001,9 @@ void ConvNode::weightUpdate()
 #endif
 
 #ifdef GETSTATS
+  int node_id = 0;
 #ifdef USE_MLSL
-  unsigned int node_id = MLSL::Environment::GetEnv().GetProcessIdx();
+  node_id = MLSL::Environment::GetEnv().GetProcessIdx();
   if(node_id == 0 && eptr_->get_current_batch() % STATFREQ == 0)
 #else
   if(eptr_->get_current_batch() % STATFREQ == 0)
@@ -976,25 +1040,42 @@ void ConvNode::weightUpdate()
 #ifdef CHECK_BLOWUP_FP32
   if(out_dtype == DT_FLOAT)
   {
-    for(int i=0; i<16; i++)
+    for(int i=0; i<10240; i++)
     {
       float v = ((float*)tenWeightDiff_->getBuffer())[i];
       if(isnan(v) || isinf(v))
       {
-        printf("Warning! %s layer BP activations are NaN or Inf\n", nname_.c_str());
+        printf("Warning! %s layer weight-gradients are NaN or Inf\n", nname_.c_str());
         exit(-1);
       }
     }
   }
   else if(out_dtype == DT_BF16)
   {
-    convert_bf16_f32((libxsmm_bfloat16*)tenWeightDiff_->getBuffer(), cbptr, 16);
-    for(int i=0; i<16; i++)
+#ifdef BF16_MLSL
+    void **wptrptr = tenWeightDiff_->getBufferPtr();
+#else
+    void **wptrptr = tenWeightDiff_->getLPBufferPtr();
+#endif
+    int offset = tenWeightDiff_->getOffset();
+    void* bf16_wtdiff = wptrptr[0] + offset*sizeof(libxsmm_bfloat16);
+
+    convert_bf16_f32((libxsmm_bfloat16*)bf16_wtdiff, cbptr, 10240);
+#ifdef USE_MLSL
+    int node_id = MLSL::Environment::GetEnv().GetProcessIdx();
+#else
+    int node_id = 0;
+#endif
+    if(node_id == 0)
     {
-      if(isnan(cbptr[i]) || isinf(cbptr[i]))
+      for(int i=0; i<10240; i++)
       {
-        printf("Warning! %s layer BP activations are NaN or Inf\n", nname_.c_str());
-        exit(-1);
+        if(isnan(cbptr[i]) || isinf(cbptr[i]))
+        {
+          printf("Warning! %s layer weight-gradients are NaN or Inf\n", nname_.c_str());
+          MeanOfLayer((char*)nname_.c_str(), (libxsmm_bfloat16*)bf16_wtdiff, ofm*ifm*kw*kw);
+          exit(-1);
+        }
       }
     }
   }
@@ -1002,27 +1083,31 @@ void ConvNode::weightUpdate()
 
 #ifdef USE_MLSL
   void *mptr = tenWeightDiff_->getBuffer();
-  void *mpptr = tenWeightDiff_->getPrivBuffer();
-  void *mp = (mpptr == NULL) ? mptr : mpptr;
+
+#ifndef BF16_MLSL
+  void *lmptr = tenWeightDiff_->getLPBuffer();
 
   if(in_dtype == DT_BF16)
   {
-    if(dwptr == NULL)
-    {
-      int wsize = ALIGN_SIZE(ifm*ofm*kh*kw*sizeof(float), 2097152);
-      dwptr = (float*)MLSL::Environment::GetEnv().Alloc(wsize, 2097152);
-    }
-    convert_bf16_f32((libxsmm_bfloat16*)mp, dwptr, ifm*ofm*kh*kw);
-    op_->GetParameterSet(0)->StartGradientComm(dwptr);
+    convert_bf16_f32((libxsmm_bfloat16*)lmptr, (float*)mptr, ifm*ofm*kh*kw);
+    op_->GetParameterSet(0)->StartGradientComm(mptr);
   }
   else if(in_dtype == DT_FLOAT)
-    op_->GetParameterSet(0)->StartGradientComm(mp);
+    op_->GetParameterSet(0)->StartGradientComm(mptr);
+#else
+  op_->GetParameterSet(0)->StartGradientComm(mptr);
+#endif
 
   if(gparams_.bias_term)
     op_->GetParameterSet(1)->StartGradientComm(tenBiasDiff_->getBuffer());
 #endif
 
 #ifdef GETSTATS
+#ifdef USE_MLSL
+  node_id = MLSL::Environment::GetEnv().GetProcessIdx();
+#else
+  node_id = 0;
+#endif
   if(node_id == 0)
   {
     if(in_dtype == DT_FLOAT)
@@ -1078,10 +1163,7 @@ void ConvNode::weightUpdate()
 
 void ConvNode::solverStep()
 {
-#ifdef RETURNALL
-  return;
-#endif
-
+#ifdef USE_MLSL
   int ifm = gparams_.nInput;
   int ofm = gparams_.nOutput;
   int kh = gparams_.kh;
@@ -1093,7 +1175,6 @@ void ConvNode::solverStep()
   if(gparams_.bias_term)
     gbias = (float*)(tenBiasDiff_->getBuffer());
 
-#ifdef USE_MLSL
   int wsize = ifm*ofm*kh*kw;
   void *mptr = op_->GetParameterSet(0)->WaitGradientComm();
   if(in_dtype == DT_FLOAT)
