@@ -1,30 +1,10 @@
 /******************************************************************************
-** Copyright (c) 2017-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Sasikanth Avancha, Dhiraj Kalamkar (Intel Corp.)
 ******************************************************************************/
@@ -91,17 +71,16 @@ FusedBNormNode::FusedBNormNode(FusedBNormParams* p, MLEngine* e): NNNode(p, e)
   Shape* bs = tenBot_[0]->getShape();
   assert(bs->ndims <= MAX_DIMS);
 
-  Shape ts;
-  shape_setzero(&ts);
-  ts.ndims = bs->ndims;
-  ts.dims[0] = bs->dims[0];
-  ts.dims[1] = bs->dims[1];
-  ts.dims[2] = bs->dims[2]/vs[0];
-  ts.dims[3] = bs->dims[3]/vs[1];
+  shape_setzero(&ts_);
+  ts_.ndims = bs->ndims;
+  ts_.dims[0] = bs->dims[0];
+  ts_.dims[1] = bs->dims[1];
+  ts_.dims[2] = bs->dims[2]/vs[0];
+  ts_.dims[3] = bs->dims[3]/vs[1];
 
-  tenTop_->setShape(&ts);
+  tenTop_->setShape(&ts_);
 
-  int telem = ts.dims[0] * ts.dims[1] * (ts.dims[2] + 2*vp[0]) * (ts.dims[3] + 2*vp[1]);
+  int telem = ts_.dims[0] * ts_.dims[1] * (ts_.dims[2] + 2*vp[0]) * (ts_.dims[3] + 2*vp[1]);
   long long int tsize;
 
   if(in_dtype == DT_FLOAT && out_dtype == DT_FLOAT)
@@ -262,8 +241,10 @@ FusedBNormNode::FusedBNormNode(FusedBNormParams* p, MLEngine* e): NNNode(p, e)
   gparams_.batch_size = bs->dims[0];
   gparams_.node_name = nname_;
   gparams_.nOutput = bs->dims[1];
-  gparams_.iHeight = gparams_.oHeight = bs->dims[2];
-  gparams_.iWidth = gparams_.oWidth = bs->dims[3];
+  gparams_.iHeight = bs->dims[2];
+  gparams_.iWidth = bs->dims[3];
+  gparams_.oHeight = ts_.dims[2];
+  gparams_.oWidth = ts_.dims[3];
   gparams_.pad_h = vp[0];
   gparams_.pad_w = vp[1];
   gparams_.ipad_h = ivp[0];
@@ -281,6 +262,7 @@ FusedBNormNode::FusedBNormNode(FusedBNormParams* p, MLEngine* e): NNNode(p, e)
   gparams_.out_data_type = out_dtype;
   gparams_.algType = p->get_algo_type();
   gparams_.num_threads = e->get_num_threads();
+  gparams_.num_numa_nodes = NUM_NUMA_NODES;
   gparams_.use_global_stats = false;
 
   lr_mult_ = p->get_lr_mult();
@@ -306,6 +288,7 @@ FusedBNormNode::FusedBNormNode(FusedBNormParams* p, MLEngine* e): NNNode(p, e)
   size_t opIdx = s->AddOperation(myRegInfo, e->get_distribution());
   this->op_ = s->GetOperation(opIdx);
   s->DeleteOperationRegInfo(myRegInfo);
+  e->get_bias_grad_comms_vec().push_back(op_);
 #endif
 };
 
@@ -342,12 +325,12 @@ void FusedBNormNode::fillBuffer(TensorBuf *tBuf, int buftype, long long int byte
   if(ttype==BNORMSCALE && buftype == DATA)
   {
     if(nname_.find("bn3") == nname_.npos)
-      initConstantBuffer(ptr, dtype, bytes, "CONSTANT", 1.0f);
+      initConstantBuffer(ptr, bytes, "CONSTANT", 1.0f);
     else
-      initConstantBuffer(ptr, dtype, bytes, "CONSTANT", 0.0f);
+      initConstantBuffer(ptr, bytes, "CONSTANT", 0.0f);
   }
   else
-    initConstantBuffer(ptr, dtype, bytes, "CONSTANT", 0.0f);
+    initConstantBuffer(ptr, bytes, "CONSTANT", 0.0f);
 }
 
 void FusedBNormNode::fillBiasMultipliers(float* lr, float* decay, long long int size)
@@ -450,21 +433,37 @@ void FusedBNormNode::forwardPropagate()
   }
 #endif
 
+  if(eptr_->get_execution_mode() == TRAIN) // || eptr_->get_execution_mode() == VAL)
+  {
+    impl->set_global_stats(false);
+    gparams_.exec_mode = "TRAIN";
+  }
+  else if(eptr_->get_execution_mode() == TEST || eptr_->get_execution_mode() == VAL)
+  {
+    impl->set_global_stats(true);
+    gparams_.exec_mode = "TEST";
+  }
+
   if(first_fp)
   {
     impl->set_bot_compute_engine(bot_cengine_[0]);
     impl->set_top_compute_engine(top_compute_engine_);
     impl->set_node_name(nname_);
     impl->set_scratch_buffer(tenScratchData_);
-    if(eptr_->get_execution_mode() == TRAIN || eptr_->get_execution_mode() == VAL)
+#if 0
+    if(eptr_->get_execution_mode() == TRAIN) // || eptr_->get_execution_mode() == VAL)
     {
       impl->set_global_stats(false);
       gparams_.exec_mode = "TRAIN";
     }
-    else if(eptr_->get_execution_mode() == TEST)
+    else if(eptr_->get_execution_mode() == TEST || eptr_->get_execution_mode() == VAL)
+    {
       impl->set_global_stats(true);
+      gparams_.exec_mode = "TEST";
+    }
+#endif
 
-    int size = nImg * ofm * (ofh/sh + 2*oph) * (ofw/sw + 2*opw);
+    int size = nImg * ofm * (ofh + 2*oph) * (ofw + 2*opw);
 
     if((gparams_.in_data_type == DT_FLOAT && gparams_.out_data_type == DT_FLOAT)
         || (gparams_.in_data_type == DT_FLOAT && gparams_.out_data_type == DT_BF16))
@@ -488,11 +487,26 @@ void FusedBNormNode::forwardPropagate()
         ptr[i] = 0;
     }
 
+#ifdef CHECK_BLOWUP_FP32
     cbptr = (float*)_mm_malloc(10240*4, 64);
+#endif
 
     scf_ = eptr_->get_scaling_factor();
     impl->set_scaling_factor(scf_);
 
+    void** meanp = tenMeanData_->getBufferPtr();
+    void** varp = tenVarData_->getBufferPtr();
+
+    for(int n=0; n<gparams_.num_numa_nodes; n++)
+    {
+      float *mean = (float*)meanp[n];
+      float *var = (float*)varp[n];
+      for(int i=0; i<ifm0; i++)
+      {
+        mean[i] = 0.0;
+        var[i] = 0.0;
+      }
+    }
     first_fp = false;
   }
 
@@ -580,26 +594,60 @@ void FusedBNormNode::forwardPropagate()
       }
     }
 
-    string s = nname_ + "_gammap";
+    string s = nname_ + "_gammap0";
     float* gamma = (float*)tenScaleData_->getBuffer();
-    MeanOfLayer((char*)s.c_str(), gamma, gparams_.nInput[0]);
+    MeanOfLayer((char*)s.c_str(), gamma, gparams_.nOutput);
+#if 0
+    void **g = tenScaleData_->getBufferPtr();
+    float *g1 = (float*)g[1] + tenScaleData_->getOffset();
+    s = nname_ + "_gammap1";
+    MeanOfLayer((char*)s.c_str(), g1, gparams_.nOutput);
+#endif
 
-    s = nname_ + "_betap";
+    s = nname_ + "_betap0";
     float* beta = (float*)tenShiftData_->getBuffer();
-    MeanOfLayer((char*)s.c_str(), beta, gparams_.nInput[0]);
+    MeanOfLayer((char*)s.c_str(), beta, gparams_.nOutput);
+#if 0
+    void **b = tenShiftData_->getBufferPtr();
+    float *b1 = (float*)b[1] + tenShiftData_->getOffset();
+    s = nname_ + "_betap1";
+    MeanOfLayer((char*)s.c_str(), b1, gparams_.nOutput);
+#endif
+
+    if(gparams_.exec_mode == "TEST")
+    {
+      float meanp[2048], varp[2048], stdevp[2048];
+
+      s = nname_ + "_meanp";
+      float *mean = (float*)tenMeanData_->getBuffer();
+      for(int i=0; i<gparams_.nOutput; i++)
+        meanp[i] = mean[i]/scf_;
+      MeanOfLayer((char*)s.c_str(), meanp, gparams_.nOutput);
+
+      s = nname_ + "_varp";
+      float *var = (float*)tenVarData_->getBuffer();
+      for(int i=0; i<gparams_.nOutput; i++)
+        varp[i] = var[i]/scf_;
+      MeanOfLayer((char*)s.c_str(), varp, gparams_.nOutput);
+
+      s = nname_ + "_stdevp";
+      for(int i=0; i<gparams_.nOutput; i++)
+        stdevp[i] = 1/sqrt(varp[i] + 1e-7);
+      MeanOfLayer((char*)s.c_str(), stdevp, gparams_.nOutput);
+    }
 
     if(out_dtype == DT_FLOAT)
     {
-      float *ptr = (float*)tenTopData_->getBuffer();
+      void *ptr = tenTopData_->getBuffer();
       string s = nname_ + "_Outp";
-      int size = nImg*ofm*(ofh/gparams_.stride_h + 2*gparams_.pad_h)*(ofw/gparams_.stride_w + 2*gparams_.pad_w);
-      MeanOfLayer((char*)s.c_str(), ptr, size);
+      int size = nImg*ofm*(ofh + 2*gparams_.pad_h)*(ofw + 2*gparams_.pad_w);
+      MeanOfLayer((char*)s.c_str(), (float*)ptr, size);
     }
     else if(out_dtype == DT_BF16)
     {
       libxsmm_bfloat16 *ptr = (libxsmm_bfloat16*)tenTopData_->getBuffer();
       s = nname_ + "_Outp";
-      int size = nImg*ofm*(ofh/gparams_.stride_h + 2*gparams_.pad_h)*(ofw/gparams_.stride_w + 2*gparams_.pad_w);
+      int size = nImg*ofm*(ofh + 2*gparams_.pad_h)*(ofw + 2*gparams_.pad_w);
       convert_bf16_f32(ptr, stptr, size);
       MeanOfLayer((char*)s.c_str(), stptr, size);
     }
@@ -619,8 +667,12 @@ void FusedBNormNode::backPropagate()
   int ifwp = ifw + 2*gparams_.ipad_w;
   int ofh = gparams_.oHeight;
   int ofw = gparams_.oWidth;
-  int ofhp = ofh + 2*gparams_.pad_h;
-  int ofwp = ofw + 2*gparams_.pad_w;
+  int oph = gparams_.pad_h;
+  int opw = gparams_.pad_w;
+  int ofhp = ofh + 2*oph;
+  int ofwp = ofw + 2*opw;
+  int sh = gparams_.stride_h;
+  int sw = gparams_.stride_w;
 
   tenTopDiff_ = tenTop_->getBuf(DIFF);
 
@@ -642,10 +694,9 @@ void FusedBNormNode::backPropagate()
 
   if(first_bp)
   {
-    int size = nImg*ifm0*ifhp*ifwp;
-
     if(in_dtype == DT_FLOAT)
     {
+      int size = nImg*ifm0*ifhp*ifwp;
       float* gbot0 = (float*)(tenBotDiff_[0]->getBuffer());
       float* gbot1 = gparams_.eltwise ? (float*)(tenBotDiff_[1]->getBuffer()) : NULL;
 
@@ -657,6 +708,7 @@ void FusedBNormNode::backPropagate()
 
       if(gbot1)
       {
+        size = nImg*gparams_.nInput[1]*ifhp*ifwp;
 #ifdef _OPENMP
 #pragma omp parallel for
 #endif
@@ -669,6 +721,7 @@ void FusedBNormNode::backPropagate()
       libxsmm_bfloat16* gbot0 = (libxsmm_bfloat16*)(tenBotDiff_[0]->getBuffer());
       libxsmm_bfloat16* gbot1 = gparams_.eltwise ? (libxsmm_bfloat16*)(tenBotDiff_[1]->getBuffer()) : NULL;
 
+      int size = nImg*ifm0*ifhp*ifwp;
 #ifdef _OPENMP
 #pragma omp parallel for
 #endif
@@ -677,6 +730,7 @@ void FusedBNormNode::backPropagate()
 
       if(gbot1)
       {
+        size = nImg*gparams_.nInput[1]*ifhp*ifwp;
 #ifdef _OPENMP
 #pragma omp parallel for
 #endif
@@ -688,30 +742,44 @@ void FusedBNormNode::backPropagate()
     first_bp = false;
   }
 
-  impl->backPropagate(tenBotData_, tenTopData_, tenScaleData_, tenTopDiff_, tenScaleDiff_, tenShiftDiff_, tenBotDiff_, 0);
+  impl->backPropagate(tenTopDiff_, tenScaleDiff_, tenShiftDiff_, tenBotDiff_, 0);
 
 #ifdef CHECK_BLOWUP_FP32
   if(out_dtype == DT_FLOAT)
   {
-    for(int i=0; i<16; i++)
+    for(int i=0; i<10240; i++)
     {
       float v = ((float*)tenBotDiff_[0]->getBuffer())[i];
       if(isnan(v) || isinf(v))
       {
-        printf("Warning! %s layer FP activations are NaN or Inf\n", nname_.c_str());
+        printf("Warning! %s layer BP activations are NaN or Inf\n", nname_.c_str());
         exit(-1);
       }
     }
   }
   else if(out_dtype == DT_BF16)
   {
-    convert_bf16_f32((libxsmm_bfloat16*)tenBotDiff_[0]->getBuffer(), cbptr, 16);
-    for(int i=0; i<16; i++)
+    convert_bf16_f32((libxsmm_bfloat16*)tenBotDiff_[0]->getBuffer(), cbptr, 10240);
+#ifdef USE_MLSL
+    int node_id = MLSL::Environment::GetEnv().GetProcessIdx();
+#else
+    int node_id = 0;
+#endif
+    if(node_id == 0)
     {
-      if(isnan(cbptr[i]) || isinf(cbptr[i]))
+      for(int i=0; i<10240; i++)
       {
-        printf("Warning! %s layer FP activations are NaN or Inf\n", nname_.c_str());
-        exit(-1);
+        if(isnan(cbptr[i]) || isinf(cbptr[i]))
+        {
+          printf("Warning! %s layer BP activations are NaN or Inf\n", nname_.c_str());
+          MeanOfLayer((char*)nname_.c_str(), (libxsmm_bfloat16*)tenBotDiff_[0]->getBuffer(), nImg*ifm0*ifhp*ifwp);
+          if(gparams_.eltwise)
+            MeanOfLayer((char*)nname_.c_str(), (libxsmm_bfloat16*)tenBotDiff_[1]->getBuffer(), nImg*ifm1*ifhp*ifwp);
+#ifdef USE_MLSL
+          MPI_Finalize();
+#endif
+          exit(-1);
+        }
       }
     }
   }
@@ -741,13 +809,25 @@ void FusedBNormNode::backPropagate()
       MeanOfLayer((char*)s.c_str(), stptr, size);
     }
 
-    string s = nname_ + "_delgammap";
+    string s = nname_ + "_delgammap0";
     float* delgamma = (float*)tenScaleDiff_->getBuffer();
     MeanOfLayer((char*)s.c_str(), delgamma, gparams_.nOutput);
+#if 0
+    void **g = tenScaleDiff_->getBufferPtr();
+    float *g1 = (float*)g[1] + tenScaleDiff_->getOffset();
+    s = nname_ + "_delgammap1";
+    MeanOfLayer((char*)s.c_str(), g1, gparams_.nOutput);
+#endif
 
-    s = nname_ + "_delbetap";
+    s = nname_ + "_delbetap0";
     float* delbeta = (float*)tenShiftDiff_->getBuffer();
     MeanOfLayer((char*)s.c_str(), delbeta, gparams_.nOutput);
+#if 0
+    void **b = tenShiftDiff_->getBufferPtr();
+    float *b1 = (float*)b[1] + tenShiftDiff_->getOffset();
+    s = nname_ + "_delbetap1";
+    MeanOfLayer((char*)s.c_str(), b1, gparams_.nOutput);
+#endif
 
     if(in_dtype == DT_FLOAT)
     {
@@ -770,31 +850,35 @@ void FusedBNormNode::backPropagate()
 
 void FusedBNormNode::weightUpdate()
 {
-  void* gexp = tenMeanData_->getBuffer();
-  void* gvar = tenVarData_->getBuffer();
+#ifdef USE_MLSL
   void* gexp_test = tenMeanData_->getPrivBuffer();
   void* gvar_test = tenVarData_->getPrivBuffer();
 
-#ifdef USE_MLSL
-  this->op_->GetParameterSet(0)->StartGradientComm(tenScaleDiff_->getBuffer());
-  this->op_->GetParameterSet(1)->StartGradientComm(tenShiftDiff_->getBuffer());
+  float *gmean = (float*)tenMeanData_->getBuffer();
+  float *gvar = (float*)tenVarData_->getBuffer();
 
+  int num_nodes = eptr_->get_num_machines();
   for(int i=0; i<gparams_.nOutput; i++)
   {
-    ((float*)gexp_test)[i] = ((float*)gexp)[i]/eptr_->get_num_machines();
-    ((float*)gvar_test)[i] = ((float*)gvar)[i]/eptr_->get_num_machines();
+    ((float*)gexp_test)[i] = gmean[i]/num_nodes;
+    ((float*)gvar_test)[i] = gvar[i]/num_nodes;
   }
-  this->op_->GetParameterSet(2)->StartGradientComm(gexp_test);
-  this->op_->GetParameterSet(3)->StartGradientComm(gvar_test);
+
+  op_->GetParameterSet(0)->StartGradientComm(tenScaleDiff_->getBuffer());
+  op_->GetParameterSet(1)->StartGradientComm(tenShiftDiff_->getBuffer());
+  op_->GetParameterSet(2)->StartGradientComm(gexp_test);
+  op_->GetParameterSet(3)->StartGradientComm(gvar_test);
 #endif
 }
 
 void FusedBNormNode::solverStep()
 {
+#if defined(USE_MLSL) || defined(CHECK_BLOWUP_FP32)
   float *delgamma = (float*)tenScaleDiff_->getBuffer();
   float *delbeta = (float*)tenShiftDiff_->getBuffer();
   void* gexp_test = tenMeanData_->getPrivBuffer();
   void* gvar_test = tenVarData_->getPrivBuffer();
+#endif
 
 #ifdef USE_MLSL
   void *mptr = op_->GetParameterSet(0)->WaitGradientComm();
