@@ -1,30 +1,10 @@
 /******************************************************************************
-** Copyright (c) 2017-2018, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Sasikanth Avancha, Dhiraj Kalamkar, Alexander Heinecke (Intel Corp.)
 ******************************************************************************/
@@ -163,6 +143,7 @@ FusedConvBNNode::FusedConvBNNode(FusedConvBNParams* p, MLEngine* e): NNNode(p, e
   // size of master weights -- FP32.
   wsize = welem*sizeof(float);
 
+  gparams_.num_numa_nodes = NUM_NUMA_NODES;
   tenWeightData_->setBufferSize(wsize);
 
   wfiller_type_ = p->get_weight_filler_type();
@@ -236,6 +217,7 @@ FusedConvBNNode::FusedConvBNNode(FusedConvBNParams* p, MLEngine* e): NNNode(p, e
         tenBotDiff_[i]->setBufferType(DIFF);
 
         // Set the size of the input-gradient buffer
+        Shape *bs = tenBot_[i]->getShape();
         int botelem = bs->dims[0] * bs->dims[1] * (bs->dims[2] + 2*ivp[0]) * (bs->dims[3] + 2*ivp[1]);
         if(in_dtype == DT_FLOAT)
           tenBotDiff_[i]->setBufferSize((botelem + convelem)*sizeof(float));
@@ -249,6 +231,17 @@ FusedConvBNNode::FusedConvBNNode(FusedConvBNParams* p, MLEngine* e): NNNode(p, e
 
     if(has_weights_)
     {
+      if(tenMidDiff_ == NULL)
+      {
+        tenMidDiff_ = tenMid_->addBuf(); // DIFF type and index
+        tenMidDiff_->setDataType(in_dtype);
+        tenMidDiff_->setBufferType(DIFF);
+        if(in_dtype == DT_FLOAT)
+          tenMidDiff_->setBufferSize(convelem*sizeof(float));
+        else if(in_dtype == DT_BF16)
+          tenMidDiff_->setBufferSize(convelem*sizeof(libxsmm_bfloat16));
+      }
+
       tenWeightDiff_ = tenWeight_->addBuf(); // DIFF type and index
       tenWeightDiff_->setBufferType(DIFF);
 
@@ -305,6 +298,11 @@ FusedConvBNNode::FusedConvBNNode(FusedConvBNParams* p, MLEngine* e): NNNode(p, e
   if(!inserted)
     printf("Warning: Tensor %s already registered\n",top_[0].c_str());
 
+  string m = "mid_"+top_[0];
+  inserted = e->register_tensor(m, ACT, tenMid_);
+  if(!inserted)
+    printf("Warning: Tensor %s already registered\n",m.c_str());
+
   // Register weight tensor in weight tensor map
   inserted = e->register_tensor(weight_, CONVWEIGHT, tenWeight_);
   if(!inserted)
@@ -371,6 +369,7 @@ FusedConvBNNode::FusedConvBNNode(FusedConvBNParams* p, MLEngine* e): NNNode(p, e
   gparams_.eps = p->get_eps();
   gparams_.use_global_stats = p->get_global_stats_flag();
   gparams_.eltwise = p->get_eltwise();
+  gparams_.bprop = bp_flag_;
 
   gparams_.in_data_type = in_dtype;
   gparams_.out_data_type = out_dtype;
@@ -402,6 +401,7 @@ FusedConvBNNode::FusedConvBNNode(FusedConvBNParams* p, MLEngine* e): NNNode(p, e
   size_t opIdx = s->AddOperation(myRegInfo, e->get_distribution());
   this->op_ = s->GetOperation(opIdx);
   s->DeleteOperationRegInfo(myRegInfo);
+  e->get_combo_grad_comms_vec().push_back(op_);
 #endif
 
   configure(p->get_compute_engine());
@@ -423,31 +423,31 @@ void FusedConvBNNode::fillWeightBuffers(TensorBuf* tBuf, int buftype, long long
   void *ptr = tBuf->getBuffer();
 
 #ifdef USE_MLSL
-    unsigned int node_id = MLSL::Environment::GetEnv().GetProcessIdx();
+  unsigned int node_id = MLSL::Environment::GetEnv().GetProcessIdx();
 #else
-    unsigned int node_id = 0;
+  unsigned int node_id = 0;
 #endif
 
+  int ic = gparams_.nInput[0];
+  int oc = gparams_.nOutput;
+  int kh = gparams_.kh;
+  int kw = gparams_.kw;
+  int g = gparams_.group;
+  int fanin = (ic * kh * kw)/g;
+  int fanout = (oc * kh * kw)/g;
+  int welem = ic * oc * kh * kw;
+
   if(buftype == DATA)
   {
-    int n = gparams_.batch_size;
-    int ic = gparams_.nInput[0];
-    int oc = gparams_.nOutput;
-    int kh = gparams_.kh;
-    int kw = gparams_.kw;
-    int g = gparams_.group;
-    int fanin = (ic * kh * kw)/g;
-    int fanout = (oc * kh * kw)/g;
-    int welem = ic * oc * kh * kw;
-
-    initBuffer(ptr, dtype, variance_norm_, fanin, fanout, welem*sizeof(float), wfiller_type_, (unsigned int)(node_id+PRIME_SEED), std_);
+    if(node_id == 0)
+      initBuffer(ptr, variance_norm_, fanin, fanout, welem*sizeof(float), wfiller_type_, std_);
 
 #ifdef USE_MLSL
     if(dtype == DT_FLOAT)
       MPI_Bcast(ptr, welem, MPI_FLOAT, 0, MPI_COMM_WORLD);
 #endif
   }
-  else
+  else if(buftype == HISTORY || buftype == DIFF)
     memset(ptr, 0, size);
 }
 
@@ -478,12 +478,12 @@ void FusedConvBNNode::fillBuffer(TensorBuf* tBuf, int buftype, long long int siz
   if(ttype==BNORMSCALE && buftype == DATA)
   {
     if(nname_.find("bn3") == nname_.npos)
-      initConstantBuffer(ptr, dtype, size, "CONSTANT", 1.0f);
+      initConstantBuffer(ptr, size, "CONSTANT", 1.0f);
     else
-      initConstantBuffer(ptr, dtype, size, "CONSTANT", 0.0f);
+      initConstantBuffer(ptr, size, "CONSTANT", 0.0f);
   }
   else
-      initConstantBuffer(ptr, dtype, size, "CONSTANT", 0.0f);
+      initConstantBuffer(ptr, size, "CONSTANT", 0.0f);
 }
 
 void FusedConvBNNode::Checkpoint(TensorBuf *tBuf, string name, string format)
@@ -577,8 +577,8 @@ void FusedConvBNNode::convert_f32_bf16(float* in, libxsmm_bfloat16* out, int len
 #pragma omp parallel for private(i)
 #endif
   for ( i = 0; i < len; i+=16 ) {
-    __m512  vfp32  = gxm_fp32_to_bfp16_rne_adjustment_avx512f( _mm512_loadu_ps( in+i ) );
-    __m256i vbfp16 = gxm_fp32_to_bfp16_truncate_avx512f( vfp32 );
+    __m512  vfp32  = gxm_fp32_to_bfp16_rne_adjustment_avx512f(_mm512_loadu_ps(in + i));
+    __m256i vbfp16 = gxm_fp32_to_bfp16_truncate_avx512f(vfp32);
     _mm256_storeu_si256( (__m256i*)(out+i), vbfp16 );
   }
 }
@@ -708,6 +708,14 @@ void FusedConvBNNode::forwardPropagate()
 
   impl->forwardPropagate(tenBotData_, tenWeightData_, tenWeightInc_, tenMidData_, tenScaleData_, tenShiftData_, tenMeanData_, tenVarData_, tenTopData_, 0);
 
+  if(eptr_->get_execution_mode() != TEST && eptr_->get_execution_mode() != VAL)
+  {
+    scf_ *= gparams_.mmf;
+    scf_ += 1.;
+
+    eptr_->set_scaling_factor(scf_);
+  }
+
 #ifdef CHECK_BLOWUP_FP32
   if(out_dtype == DT_FLOAT)
   {
@@ -893,6 +901,8 @@ void FusedConvBNNode::backPropagate()
       tenMidDiff_->setBuffer(tenBotDiff_[0]->getBuffer() + bsize0*sizeof(float));
       tenMidDiff_->setBufferSize(msize*sizeof(float));
       tenBotDiff_[0]->setBufferSize(bsize0*sizeof(float));
+      if(gparams_.eltwise)
+        tenBotDiff_[1]->setBufferSize(bsize1*sizeof(float));
 
       // NUMA initialize Conv delinp
 #ifdef _OPENMP
@@ -926,6 +936,8 @@ void FusedConvBNNode::backPropagate()
       tenMidDiff_->setBuffer(tenBotDiff_[0]->getBuffer() + bsize0*sizeof(libxsmm_bfloat16));
       tenMidDiff_->setBufferSize(msize*sizeof(libxsmm_bfloat16));
       tenBotDiff_[0]->setBufferSize(bsize0*sizeof(libxsmm_bfloat16));
+      if(gparams_.eltwise)
+        tenBotDiff_[1]->setBufferSize(bsize1*sizeof(libxsmm_bfloat16));
 
       // NUMA initialize Conv delinp
 #ifdef _OPENMP
@@ -1078,7 +1090,39 @@ void FusedConvBNNode::weightUpdate()
   }
 #endif
 
-  impl->weightUpdate(tenBotData_[0], tenMidDiff_, tenWeightDiff_, 0);
+  if(!bp_flag_ && first_upd)
+  {
+    int msize = nImg*ofm*mfhp*mfwp;
+
+    if(in_dtype == DT_FLOAT)
+    {
+      float *ptr = (float*)tenMidDiff_->getBuffer();
+
+      // NUMA initialize Conv delmidp
+#ifdef _OPENMP
+#pragma omp parallel for
+#endif
+      for(int i=0; i<msize; i++)
+        ptr[i] = 0;
+    }
+    else if(in_dtype == DT_BF16)
+    {
+      libxsmm_bfloat16* ptr = (libxsmm_bfloat16*)tenMidDiff_->getBuffer();
+
+      // NUMA initialize = Conv delmidp
+      ptr = (libxsmm_bfloat16*)tenMidDiff_->getBuffer();
+
+#ifdef _OPENMP
+#pragma omp parallel for
+#endif
+      for(int i=0; i<msize; i++)
+        ptr[i] = 0;
+    }
+    first_upd = false;
+  }
+
+  tenTopDiff_ = tenTop_->getBuf(DIFF);
+  impl->weightUpdate(tenBotData_[0], tenTopDiff_, tenMidDiff_, tenWeightDiff_, tenScaleDiff_, tenShiftDiff_, 0);
 
 #ifdef CHECK_BLOWUP_FP32
   if(out_dtype == DT_FLOAT)
@@ -1107,11 +1151,22 @@ void FusedConvBNNode::weightUpdate()
   }
 #endif
 
-  void* gexp = tenMeanData_->getBuffer();
-  void* gvar = tenVarData_->getBuffer();
+  void* gexp[NUM_NUMA_NODES];
+  void* gvar[NUM_NUMA_NODES];
   void* gexp_test = tenMeanData_->getPrivBuffer();
   void* gvar_test = tenVarData_->getPrivBuffer();
 
+  void **mptrptr = tenMeanData_->getBufferPtr();
+  void **vptrptr = tenVarData_->getBufferPtr();
+  int offset = tenMeanData_->getOffset();
+
+  for(int n=0; n<NUM_NUMA_NODES; n++)
+    gexp[n] = mptrptr[n] + offset*sizeof(float);
+
+  offset = tenVarData_->getOffset();
+  for(int n=0; n<NUM_NUMA_NODES; n++)
+    gvar[n] = vptrptr[n] + offset*sizeof(float);
+
 #ifdef USE_MLSL
   void *mptr = tenWeightDiff_->getBuffer();
 
@@ -1119,7 +1174,7 @@ void FusedConvBNNode::weightUpdate()
   {
     if(dwptr == NULL)
     {
-      int wsize = ALIGN_SIZE(ifm0*ofm*kh*kw*sizeof(float), 2097152);
+      int wsize = ifm0*ofm*kh*kw*sizeof(float);
       dwptr = (float*)MLSL::Environment::GetEnv().Alloc(wsize, 2097152);
     }
     convert_bf16_f32((libxsmm_bfloat16*)mptr, dwptr, ifm0*ofm*kh*kw);
@@ -1131,13 +1186,23 @@ void FusedConvBNNode::weightUpdate()
   op_->GetParameterSet(1)->StartGradientComm(tenScaleDiff_->getBuffer());
   op_->GetParameterSet(2)->StartGradientComm(tenShiftDiff_->getBuffer());
 
-#ifdef _OPENMP
-#pragma omp parallel for
-#endif
+  int num_nodes = eptr_->get_num_machines();
   for(int i=0; i<ofm; i++)
   {
-    ((float*)gexp_test)[i] = ((float*)gexp)[i]/eptr_->get_num_machines();
-    ((float*)gvar_test)[i] = ((float*)gvar)[i]/eptr_->get_num_machines();
+    float mtmp = 0.0;
+    float vtmp = 0.0;
+
+    for(int n=0; n<NUM_NUMA_NODES; n++)
+    {
+      mtmp += ((float*)gexp[n])[i];
+      vtmp += ((float*)gvar[n])[i];
+    }
+
+    mtmp = mtmp/NUM_NUMA_NODES;
+    vtmp = vtmp/NUM_NUMA_NODES;
+
+    ((float*)gexp_test)[i] = mtmp/num_nodes;
+    ((float*)gvar_test)[i] = vtmp/num_nodes;
   }
   this->op_->GetParameterSet(3)->StartGradientComm(gexp_test);
   this->op_->GetParameterSet(4)->StartGradientComm(gvar_test);
@@ -1182,7 +1247,14 @@ void FusedConvBNNode::weightUpdate()
       MeanOfLayer((char*)s.c_str(), stptr, nImg*ofm*mfhp*mfwp);
 
       s = nname_ + "_delWt_Aft";
+#ifdef USE_MLSL
       MeanOfLayer((char*)s.c_str(), dwptr, ifm0*ofm*kh*kw);
+#else
+      ptr = (libxsmm_bfloat16*)tenWeightDiff_->getBuffer();
+      memset(stptr, 0, ifm0*ofm*kh*kw);
+      convert_bf16_f32(ptr, stptr, ifm0*ofm*kh*kw);
+      MeanOfLayer((char*)s.c_str(), stptr, ifm0*ofm*kh*kw);
+#endif
     }
   }
 #endif
@@ -1190,6 +1262,7 @@ void FusedConvBNNode::weightUpdate()
 
 void FusedConvBNNode::solverStep()
 {
+#ifdef USE_MLSL
   int ifm = gparams_.nInput[0];
   int ofm = gparams_.nOutput;
   int kh = gparams_.kh;
@@ -1203,7 +1276,6 @@ void FusedConvBNNode::solverStep()
 
   int wsize = ifm*ofm*kh*kw;
 
-#ifdef USE_MLSL
   void *mptr = op_->GetParameterSet(0)->WaitGradientComm();
   if(in_dtype == DT_FLOAT)
   {
@@ -1232,7 +1304,6 @@ void FusedConvBNNode::solverStep()
   mptr = op_->GetParameterSet(4)->WaitGradientComm();
   if(mptr != NULL && mptr != gvar_test)
     memcpy((void*)gvar_test, mptr, ofm*sizeof(float));
-#endif
 
 #ifdef CHECK_BLOWUP_FP32
   float* ptr = (float*)tenWeightDiff_->getBuffer();
@@ -1261,4 +1332,5 @@ void FusedConvBNNode::solverStep()
     }
   }
 #endif
+#endif
 }
