@@ -1,30 +1,10 @@
 /******************************************************************************
-** Copyright (c) 2017-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Sasikanth Avancha, Dhiraj Kalamkar (Intel Corp.)
 ******************************************************************************/
@@ -495,14 +475,19 @@ void JitterDataNode::imageTransform(vector<cv::Mat>& vcrop, float* outp)
       for(int w = 0; w < ofw; w++) {
         for(int ofm = 0; ofm < nOfm; ofm++) {
 
-          assert(vcrop[img].channels() == nOfm);
+          //assert(vcrop[img].channels() == nOfm);
           int out_idx;
 
           int oh = h+padh;
           int ow = w+padw;
 
-          float inp = static_cast<float>(ptr[img_index++]);
-          int fm = (gparams_.scale_values.size() == 1) ? 0 : ofm;
+          float inp;
+          int fm;
+          if(ofm < 3)
+          {
+            inp = static_cast<float>(ptr[img_index++]);
+            fm = (gparams_.scale_values.size() == 1) ? 0 : ofm;
+          }
 
           if(gparams_.exec_mode == TRAIN)
           {
@@ -514,7 +499,10 @@ void JitterDataNode::imageTransform(vector<cv::Mat>& vcrop, float* outp)
           else
             out_idx = img * ofhp * ofwp * nOfm + oh * ofwp * nOfm + ow * nOfm + ofm;
 
-          outp[out_idx] = (inp - mean[ofm]) * rscale[fm];
+          if(ofm == 3)
+            outp[out_idx] = 0.0;
+          else
+            outp[out_idx] = (inp - mean[ofm]) * rscale[fm];
         }
       }
     }
@@ -525,12 +513,13 @@ int myrandom (int i) { return std::rand()%i;}
 
 /* it's fine to alias in and out */
 void JitterDataNode::convert_f32_bf16(float* in, libxsmm_bfloat16* out, unsigned int len) {
+
   unsigned int i = 0;
 
 #pragma omp parallel for private(i)
   for ( i = 0; i < len; i+=16 ) {
-    __m512  vfp32  = gxm_fp32_to_bfp16_rne_adjustment_avx512f( _mm512_loadu_ps( in+i ) );
-    __m256i vbfp16 = gxm_fp32_to_bfp16_truncate_avx512f( vfp32 );
+    __m512  vfp32  = gxm_fp32_to_bfp16_rne_adjustment_avx512f(_mm512_loadu_ps(in + i));
+    __m256i vbfp16 = gxm_fp32_to_bfp16_truncate_avx512f(vfp32);
     _mm256_storeu_si256( (__m256i*)(out+i), vbfp16 );
   }
 }
@@ -545,6 +534,7 @@ void JitterDataNode::forwardPropagate()
   int padw = gparams_.pad_w;
   int ofhp = ofh + 2*padh;
   int ofwp = ofw + 2*padw;
+  int out_dtype = tenTopData_[0]->getDataType();
 
   float *topdata = (float*)(tenTopData_[0]->getBuffer());
   int* toplabel = (int*)(tenTopData_[1]->getBuffer());
@@ -552,6 +542,7 @@ void JitterDataNode::forwardPropagate()
   if(first_fp)
   {
     int size = nImg * nOfm * ofhp *ofwp;
+
 #ifdef _OPENMP
 #pragma omp parallel for
 #endif
@@ -656,7 +647,6 @@ void JitterDataNode::forwardPropagate()
       check_physical_pad(nname_.c_str(), topdata, nImg, 1, ofh, ofw, nOfm, gparams_.pad_h, gparams_.pad_w);
 #endif
 
-    int out_dtype = tenTopData_[0]->getDataType();
     int crop_img_size = nImg * ofhp * ofwp * nOfm;
 
     if(out_dtype == DT_BF16)
@@ -749,7 +739,6 @@ void JitterDataNode::forwardPropagate()
 
     imageTransform(cropbuf_[mbslot], topdata);
 
-    int out_dtype = tenTopData_[0]->getDataType();
     int crop_img_size = nImg * ofhp * ofwp * nOfm;
 
     if(out_dtype == DT_BF16)
