@@ -1,30 +1,10 @@
 /******************************************************************************
-** Copyright (c) 2017-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Alexander Heinecke (Intel Corp.)
 ******************************************************************************/
@@ -83,7 +63,7 @@ static void libxsmm_sparse_csr_reader( const char*    i_csr_file_in,
         {
           /* allocate CSC datastructure matching mtx file */
           *o_column_idx = (unsigned int*) malloc(sizeof(unsigned int) * (*o_element_count));
-          *o_row_idx = (unsigned int*) malloc(sizeof(unsigned int) * (*o_row_count + 1));
+          *o_row_idx = (unsigned int*) malloc(sizeof(unsigned int) * (*o_row_count + (size_t)1));
           *o_values = (REALTYPE*) malloc(sizeof(double) * (*o_element_count));
           l_row_idx_id = (unsigned int*) malloc(sizeof(unsigned int) * (*o_row_count));
 
@@ -97,7 +77,7 @@ static void libxsmm_sparse_csr_reader( const char*    i_csr_file_in,
           }
 
           /* set everything to zero for init */
-          memset(*o_row_idx, 0, sizeof(unsigned int)*(*o_row_count + 1));
+          memset(*o_row_idx, 0, sizeof(unsigned int)*(*o_row_count + (size_t)1));
           memset(*o_column_idx, 0, sizeof(unsigned int)*(*o_element_count));
           memset(*o_values, 0, sizeof(double)*(*o_element_count));
           memset(l_row_idx_id, 0, sizeof(unsigned int)*(*o_row_count));
@@ -153,6 +133,11 @@ static void libxsmm_sparse_csr_reader( const char*    i_csr_file_in,
     return;
   }
 
+  if ( NULL == l_row_idx_id ) {
+    fprintf( stderr, "allocating memory for row indexes failed!\n" );
+    return;
+  }
+
   /* let's handle empty rows */
   for ( l_i = 0; l_i < (*o_row_count); l_i++) {
     if ( l_row_idx_id[l_i] == 0 ) {
@@ -200,7 +185,7 @@ static void libxsmm_sparse_csc_reader( const char*    i_csc_file_in,
         if ( sscanf(l_line, "%u %u %u", o_row_count, o_column_count, o_element_count) == 3 ) {
           /* allocate CSC datastructure matching mtx file */
           *o_row_idx = (unsigned int*) malloc(sizeof(unsigned int) * (*o_element_count));
-          *o_column_idx = (unsigned int*) malloc(sizeof(unsigned int) * (*o_column_count + 1));
+          *o_column_idx = (unsigned int*) malloc(sizeof(unsigned int) * (*o_column_count + (size_t)1));
           *o_values = (REALTYPE*) malloc(sizeof(double) * (*o_element_count));
           l_column_idx_id = (unsigned int*) malloc(sizeof(unsigned int) * (*o_column_count));
 
@@ -214,15 +199,15 @@ static void libxsmm_sparse_csc_reader( const char*    i_csc_file_in,
           }
 
           /* set everything to zero for init */
-          memset(*o_column_idx, 0, sizeof(unsigned int)*(*o_column_count + 1));
+          memset(*o_column_idx, 0, sizeof(unsigned int)*(*o_column_count + (size_t)1));
           memset(*o_row_idx, 0, sizeof(unsigned int)*(*o_element_count));
           memset(*o_values, 0, sizeof(double)*(*o_element_count));
           memset(l_column_idx_id, 0, sizeof(unsigned int)*(*o_column_count));
 
           /* init column idx */
-          for ( l_i = 0; l_i < (*o_column_count + 1); l_i++)
+          for ( l_i = 0; l_i < (*o_column_count + 1); l_i++ ) {
             (*o_column_idx)[l_i] = (*o_element_count);
-
+          }
           /* init */
           (*o_column_idx)[0] = 0;
           l_i = 0;
@@ -270,6 +255,11 @@ static void libxsmm_sparse_csc_reader( const char*    i_csc_file_in,
     return;
   }
 
+  if ( NULL == l_column_idx_id ) {
+    fprintf( stderr, "allocating memory for column indexes failed!\n" );
+    return;
+  }
+
   /* let's handle empty rows */
   for ( l_i = 0; l_i < (*o_column_count); l_i++) {
     if ( l_column_idx_id[l_i] == 0 ) {
