@@ -37,10 +37,14 @@
 #endif
 #if 0
 # define STREAM_C(EXPR) (EXPR)
+# define SYNC(IDX, INC, END) ((IDX) * (INC))
+#elif defined(_OPENMP)
+# define STREAM_C(EXPR) (EXPR)
+# define SYNC(IDX, INC, END) (((1048573 * omp_get_thread_num()) % (END)) * (INC))
 #else
 # define STREAM_C(EXPR) 0
 /* synchronization among C matrices */
-# define SYNC
+# define SYNC(IDX, INC, END) 0
 #endif
 
 /**
@@ -64,12 +68,11 @@
 
 static void init(int seed, TYPE* dst, int nrows, int ncols, int ld, double scale) {
   const double seed1 = scale * seed + scale;
-  int i;
+  int i, j;
   for (i = 0; i < ncols; ++i) {
-    int j = 0;
-    for (; j < nrows; ++j) {
+    for (j = 0; j < nrows; ++j) {
       const int k = i * ld + j;
-      dst[k] = (TYPE)(seed1 / (1.0 + k));
+      dst[k] = (TYPE)(seed1 * (1.0 + i * nrows + j));
     }
     for (; j < ld; ++j) {
       const int k = i * ld + j;
