@@ -95,7 +95,7 @@ int main(int argc, char* argv[])
 #endif
 
   /* initialize data according to touch-first policy */
-#if defined(_OPENMP) && !defined(SYNC)
+#if defined(_OPENMP)
 # pragma omp parallel for
 #endif
   for (int i = 0; i < size; ++i) {
@@ -111,11 +111,11 @@ int main(int argc, char* argv[])
     }
   }
 
-#if defined(_OPENMP) && !defined(SYNC)
+#if defined(_OPENMP)
 # pragma omp parallel
 #endif
   {
-#if defined(_OPENMP) && !defined(SYNC)
+#if defined(_OPENMP)
 #   pragma omp single
 #endif
 #if defined(__EIGEN_TIMER)
@@ -123,7 +123,7 @@ int main(int argc, char* argv[])
 #else
     duration = seconds();
 #endif
-#if defined(_OPENMP) && !defined(SYNC)
+#if defined(_OPENMP)
 #   pragma omp for
 #endif
     for (int i = 0; i < size; ++i) {
@@ -140,10 +140,10 @@ int main(int argc, char* argv[])
        * Expression templates attempt to delay evaluation until the sequence point
        * is reached, or an "expression object" goes out of scope and hence must
        * materialize the effect. Ideally, a complex expression is mapped to the
-       * best possible implementation e.g., c = alpha * a * b + beta * c may be
+       * best possible implementation, e.g., c = alpha * a * b + beta * c may be
        * mapped to GEMM or definitely omits alpha*a in case of alpha=1, or similar
        * for special cases for beta=0 and beta=1. However, to not rely on an ideal
-       * transformation a *manually specialized* expression is written for e.g.,
+       * transformation a *manually specialized* expression is written for, e.g.,
        * alpha=1 and beta=1 (c += a * b) or tweaked manually ("noalias").
        * NOTE: changing alpha or beta from above may not have an effect
        *       depending on what is selected below (expression).
