@@ -1,30 +1,10 @@
 /******************************************************************************
-** Copyright (c) 2017-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Greg Henry, Hans Pabst, Alexander Heinecke (Intel Corp.)
 ******************************************************************************/
@@ -77,17 +57,17 @@ void dcopy_to_temp ( int layout, double *A, int lda, int m, int n, double *Atemp
     if ( layout == 102 )
     {
        /* printf("Column major\n"); */
-       for ( j = 0 ; j < n ; j++ )
+       for ( j = 0; j < n; j++ )
        {
-          for ( i = 0 ; i < m ; i++ )
+          for ( i = 0; i < m; i++ )
           {
              Atemp[i+j*m] = A[i*VLEN+j*lda*VLEN];
           }
        }
 #if EVENTUALLY_USE_THIS_LOOP_IT_SHOULD_BE_FASTER
-       for ( j = 0 ; j < n ; j++ )
+       for ( j = 0; j < n; j++ )
        {
-          for ( i = 0, ia = 0 ; i < m ; i++, ia+=VLEN )
+          for ( i = 0, ia = 0; i < m; i++, ia+=VLEN )
           {
              Atemp[i+j*m] = A[ ia+j*lda*VLEN ];
           }
@@ -95,9 +75,9 @@ void dcopy_to_temp ( int layout, double *A, int lda, int m, int n, double *Atemp
 #endif
     } else {
        /* printf("Row major\n"); */
-       for ( j = 0 ; j < n ; j++ )
+       for ( j = 0; j < n; j++ )
        {
-          for ( i = 0 ; i < m ; i++ )
+          for ( i = 0; i < m; i++ )
           {
              /* Transpose the data */
              Atemp[i+j*m] = A[j*VLEN+i*lda*VLEN];
@@ -120,18 +100,18 @@ void scopy_to_temp ( int layout, float *A, int lda, int m, int n, float *Atemp,
     if ( layout == 102 )
     {
        /* printf("Column major\n"); */
-       for ( j = 0 ; j < n ; j++ )
+       for ( j = 0; j < n; j++ )
        {
-          for ( i = 0 ; i < m ; i++ )
+          for ( i = 0; i < m; i++ )
           {
              Atemp[i+j*m] = A[i*VLEN+j*lda*VLEN];
           }
        }
     } else {
        /* printf("Row major\n"); */
-       for ( j = 0 ; j < n ; j++ )
+       for ( j = 0; j < n; j++ )
        {
-          for ( i = 0 ; i < m ; i++ )
+          for ( i = 0; i < m; i++ )
           {
              /* Transpose the data */
              Atemp[i+j*m] = A[j*VLEN+i*lda*VLEN];
@@ -152,17 +132,17 @@ void dcopy_from_temp ( int layout, double *A, int lda, int m, int n, double *Ate
     }
     if ( layout == 102 )
     {
-       for ( j = 0 ; j < n ; j++ )
+       for ( j = 0; j < n; j++ )
        {
-          for ( i = 0, ia = 0 ; i < m ; i++, ia+=VLEN )
+          for ( i = 0, ia = 0; i < m; i++, ia+=VLEN )
           {
              A[ia+j*lda*VLEN] = Atemp[i+j*m];
           }
        }
     } else {
-       for ( j = 0 ; j < n ; j++ )
+       for ( j = 0; j < n; j++ )
        {
-          for ( i = 0 ; i < m ; i++ )
+          for ( i = 0; i < m; i++ )
           {
              /* Transpose the data */
              A[j*VLEN+i*lda*VLEN] = Atemp[i+j*m];
@@ -183,17 +163,17 @@ void scopy_from_temp ( int layout, float *A, int lda, int m, int n, float *Atemp
     }
     if ( layout == 102 )
     {
-       for ( j = 0 ; j < n ; j++ )
+       for ( j = 0; j < n; j++ )
        {
-          for ( i = 0, ia = 0 ; i < m ; i++, ia+=VLEN )
+          for ( i = 0, ia = 0; i < m; i++, ia+=VLEN )
           {
              A[ia+j*lda*VLEN] = Atemp[i+j*m];
           }
        }
     } else {
-       for ( j = 0 ; j < n ; j++ )
+       for ( j = 0; j < n; j++ )
        {
-          for ( i = 0 ; i < m ; i++ )
+          for ( i = 0; i < m; i++ )
           {
              /* Transpose the data */
              A[j*VLEN+i*lda*VLEN] = Atemp[i+j*m];
@@ -234,9 +214,9 @@ void compact_dtrsm_ ( unsigned int *layout, char *side, char *uplo,
     }
     offseta = (*lda)*asize*(*VLEN);
     if ( ++ntimes < 3 ) printf("m/n=%u,%u layout=%u asize=%i VLEN=%u nmat=%u offseta=%i offsetb=%i\n",*m,*n,*layout, asize, *VLEN, *nmat, offseta, offsetb );
-    for ( i = 0, num = 0 ; i < (int)(*nmat) ; i+= *VLEN, num++ )
+    for ( i = 0, num = 0; i < (int)(*nmat); i+= *VLEN, num++ )
     {
-       for ( j = 0 ; j < (int)*VLEN ; j++ )
+       for ( j = 0; j < (int)*VLEN; j++ )
        {
            /* Unpack the data, call a reference DTRSM, repack the data */
            Ap = &A[j+num*offseta];
@@ -269,9 +249,9 @@ void compact_strsm_ ( unsigned int *layout, char *side, char *uplo,
 
     if ( (*side == 'L') || (*side == 'l') ) asize = *m;
     else asize = *n;
-    for ( i = 0, num = 0 ; i < (int)(*nmat) ; i+= *VLEN, num++ )
+    for ( i = 0, num = 0; i < (int)(*nmat); i+= *VLEN, num++ )
     {
-       for ( j = 0 ; j < (int)*VLEN ; j++ )
+       for ( j = 0; j < (int)*VLEN; j++ )
        {
            /* Unpack the data, call a reference DTRSM, repack the data */
            Ap = &A[j+num*(*lda)*asize*(*VLEN)];
@@ -297,7 +277,7 @@ void dfill_matrix ( double *matrix, unsigned int ld, unsigned int m, unsigned in
      fprintf(stderr,"Error in dfill_matrix: ld=%u m=%u mismatched!\n",ld,m);
      exit(-1);
   }
-  for ( j = 1 ; j <= n ; j++ )
+  for ( j = 1; j <= n; j++ )
   {
      /* Fill through the leading dimension */
      for ( i = 1; i <= ld; i++ )
@@ -318,12 +298,12 @@ void dfill_identity ( double *matrix, unsigned int ld, unsigned int m, unsigned
      fprintf(stderr,"Error in dfill_identity: ld=%u m=%u mismatched!\n",ld,m);
      exit(-1);
   }
-  for ( h = 0; h < (unsigned int)number_of_cases ; h++ ) {
+  for ( h = 0; h < (unsigned int)number_of_cases; h++ ) {
      ia = h*ld*n*VLEN;
-     for ( j = 1 ; j <= n ; j++ ) {
-        for ( i = 1 ; i <= ld; i++ ) {
+     for ( j = 1; j <= n; j++ ) {
+        for ( i = 1; i <= ld; i++ ) {
            if ( i == j ) dtmp = 1.0; else dtmp = 0.0;
-           for ( k = 0 ; k < (unsigned int)VLEN ; k++ ) matrix[ia++] = dtmp;
+           for ( k = 0; k < (unsigned int)VLEN; k++ ) matrix[ia++] = dtmp;
         }
      }
   }
@@ -340,7 +320,7 @@ void sfill_matrix ( float *matrix, unsigned int ld, unsigned int m, unsigned int
      fprintf(stderr,"Error is sfill_matrix: ld=%u m=%u mismatched!\n",ld,m);
      exit(-1);
   }
-  for ( j = 1 ; j <= n ; j++ )
+  for ( j = 1; j <= n; j++ )
   {
      /* Fill through the leading dimension */
      for ( i = 1; i <= ld; i++ )
@@ -363,7 +343,7 @@ double residual_d ( double *A, unsigned int lda, unsigned int m, unsigned int n,
    *nerrs = 0;
    *ncorr = 0;
    derror = 0.0;
-   for ( j = 1 ; j<= n; j++ )
+   for ( j = 1; j<= n; j++ )
    {
       for ( i = 1; i <= m; i++ )
       {
@@ -414,7 +394,7 @@ double residual_s ( float *A, unsigned int lda, unsigned int m, unsigned int n,
    *nerrs = 0;
    *ncorr = 0;
    derror = 0.0;
-   for ( j = 1 ; j<= n; j++ )
+   for ( j = 1; j<= n; j++ )
    {
       for ( i = 1; i <= m; i++ )
       {
@@ -502,6 +482,7 @@ int main(int argc, char* argv[])
   io_generated_code.code_size = 0;
   io_generated_code.code_type = 2;
   io_generated_code.last_error = 0;
+  io_generated_code.sf_size = 0;
 #endif
 
   if ( argc <= 3 )
@@ -607,10 +588,10 @@ int main(int argc, char* argv[])
   dfill_matrix ( db, bsize, bsize, nmatd );
 
 #ifndef NO_ACCURACY_CHECK
-  for ( i = 0 ; i < (int)(bsize*nmats) ; i++ ) sc[i]=sb[i];
-  for ( i = 0 ; i < (int)(bsize*nmatd) ; i++ ) dc[i]=db[i];
-  for ( i = 0 ; i < (int)(bsize*nmats) ; i++ ) sd[i]=sb[i];
-  for ( i = 0 ; i < (int)(bsize*nmatd) ; i++ ) dd[i]=db[i];
+  for ( i = 0; i < (int)(bsize*nmats); i++ ) sc[i]=sb[i];
+  for ( i = 0; i < (int)(bsize*nmatd); i++ ) dc[i]=db[i];
+  for ( i = 0; i < (int)(bsize*nmats); i++ ) sd[i]=sb[i];
+  for ( i = 0; i < (int)(bsize*nmatd); i++ ) dd[i]=db[i];
   printf("Pointing at the kernel now\n");
 #endif
 
@@ -637,7 +618,7 @@ int main(int argc, char* argv[])
   fputs("\t.align 256\n",fp);
   fputs("\t.globl trsm_xct_\n",fp);
   fputs("trsm_xct_:\n",fp);
-  for (i = 0 ; i < 4000; i+=4 )
+  for (i = 0; i < 4000; i+=4 )
   {
      sprintf(buffer,".byte 0x%02x, 0x%02x, 0x%02x, 0x%02x\n",cptr[i],cptr[i+1],cptr[i+2],cptr[i+3]);
      fputs(buffer,fp);
@@ -676,12 +657,12 @@ int main(int argc, char* argv[])
 #endif
 
   timer = 0.0;
-  for ( j = 0 ; j < (int)ntest ; j++ )
+  for ( j = 0; j < (int)ntest; j++ )
   {
 #ifndef TRIANGLE_IS_IDENTITY
-  for ( i = 0 ; i < (int)(bsize*nmatd) ; i++ ) db[i]=dd[i];
+  for ( i = 0; i < (int)(bsize*nmatd); i++ ) db[i]=dd[i];
 #endif
-  for ( i = 0 , num = 0; i < (int)nmatd ; i+= (int)VLEND, num++ )
+  for ( i = 0 , num = 0; i < (int)nmatd; i+= (int)VLEND, num++ )
   {
      double *Ap = &da[num*lda*asize*VLEND];
      double *Bp = &db[num*bsize*VLEND];
@@ -722,7 +703,7 @@ int main(int argc, char* argv[])
 
 #ifdef TEST_SINGLE
   printf("Before r4 routine, initial B(1,1)=%g B[256]=%g\n",sb[0],sb[256]);
-  for ( i = 0 , num = 0; i < nmats ; i+= VLENS, num++ )
+  for ( i = 0 , num = 0; i < nmats; i+= VLENS, num++ )
   {
      float *Ap = &sa[num*lda*asize*VLENS];
      float *Bp = &sb[num*bsize*VLENS];
@@ -736,10 +717,10 @@ int main(int argc, char* argv[])
 #ifndef NO_ACCURACY_CHECK
   /* Call some reference code now on a copy of the B matrix (C) */
   double timer2 = 0.0;
-  for ( j = 0 ; j < (int)ntest ; j++ )
+  for ( j = 0; j < (int)ntest; j++ )
   {
 #ifndef TRIANGLE_IS_IDENTITY
-  for ( i = 0 ; i < (int)(bsize*nmatd) ; i++ ) dc[i]=dd[i];
+  for ( i = 0; i < (int)(bsize*nmatd); i++ ) dc[i]=dd[i];
 #endif
 
 #ifdef MKL_TIMER
@@ -791,7 +772,7 @@ int main(int argc, char* argv[])
 #endif
 
 #else
-  for ( j = 0, nerrs = 0 ; j < bsize*nmatd ; j++ )
+  for ( j = 0, nerrs = 0; j < bsize*nmatd; j++ )
   {
      if ( isnan(db[j]) || isinf(db[j]) )
      {
