@@ -10,23 +10,34 @@
 # Hans Pabst (Intel Corp.)
 ###############################################################################
 
+HERE=$(cd "$(dirname "$0")" && pwd -P)
+MKTEMP=${HERE}/../.mktmp.sh
 MAKE=$(command -v make)
 GREP=$(command -v grep)
 SORT=$(command -v sort)
 CXX=$(command -v clang++)
 CC=$(command -v clang)
+CP=$(command -v cp)
+MV=$(command -v mv)
 
-if [ "" != "${MAKE}" ] && [ "" != "${CXX}" ] && [ "" != "${CC}" ] && \
-   [ "" != "${GREP}" ] && [ "" != "${SORT}" ];
+if [ "${MKTEMP}" ] && [ "${MAKE}" ] && \
+   [ "${GREP}" ] && [ "${SORT}" ] && \
+   [ "${CXX}" ] && [ "${CC}" ] && \
+   [ "${CP}" ] && [ "${MV}" ];
 then
-  HERE=$(cd "$(dirname "$0")"; pwd -P)
-  cd "${HERE}/.."
+  cd "${HERE}/.." || exit 1
   ARG=$*
   if [ "" = "${ARG}" ]; then
     ARG=lib
   fi
-  ${MAKE} -e CXX=${CXX} CC=${CC} FC= FORCE_CXX=1 DBG=1 ILP64=1 EFLAGS="--analyze" ${ARG} 2> .analyze.log
-  ISSUES=$(${GREP} -e "error:" -e "warning:" .analyze.log | ${GREP} -v "is never read" | ${SORT} -u)
+  TMPF=$("${MKTEMP}" .tool_analyze.XXXXXX)
+  ${CP} "${HERE}/../include/libxsmm_config.h" "${TMPF}"
+  ${MAKE} -e CXX="${CXX}" CC="${CC}" FC= FORCE_CXX=1 DBG=1 ILP64=1 EFLAGS="--analyze" ${ARG} 2> .analyze.log
+  ${MV} "${TMPF}" "${HERE}/../include/libxsmm_config.h"
+  ISSUES=$(${GREP} -e "error:" -e "warning:" .analyze.log \
+    | ${GREP} -v "make:" \
+    | ${GREP} -v "is never read" \
+    | ${SORT} -u)
   echo
   echo   "================================================================================"
   if [ "" = "${ISSUES}" ]; then
@@ -39,7 +50,6 @@ then
     exit 1
   fi
 else
-  echo "Error: missing prerequisites!"
+  >&2 echo "Error: missing prerequisites!"
   exit 1
 fi
-
