@@ -102,7 +102,7 @@ void libxsmm_close_function( libxsmm_generated_code* io_generated_code ) {
 }
 
 LIBXSMM_API_INTERN
-unsigned int libxsmm_check_x86_gp_reg_name_callee_save( const unsigned int i_gp_reg_number ) {
+unsigned int libxsmm_check_x86_gp_reg_callee_save( const unsigned int i_gp_reg_number ) {
   if ( (i_gp_reg_number == LIBXSMM_X86_GP_REG_RBX) ||
        (i_gp_reg_number == LIBXSMM_X86_GP_REG_RBP) ||
        (i_gp_reg_number == LIBXSMM_X86_GP_REG_R12) ||
@@ -950,26 +950,6 @@ const char* libxsmm_strerror(unsigned int i_error_code) {
       LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
         "there is no QFMA instruction set extension available (error #%u)!", i_error_code );
       break;
-    case LIBXSMM_ERR_CALLEE_SAVE_A:
-      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
-        "reg_a cannot be callee save, since input, please use either rdi, rsi, rdx, rcx, r8, r9 for this value (error #%u)!", i_error_code );
-      break;
-    case LIBXSMM_ERR_CALLEE_SAVE_B:
-      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
-        "reg_b cannot be callee save, since input, please use either rdi, rsi, rdx, rcx, r8, r9 for this value (error #%u)!", i_error_code );
-      break;
-    case LIBXSMM_ERR_CALLEE_SAVE_C:
-      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
-        "reg_c cannot be callee save, since input, please use either rdi, rsi, rdx, rcx, r8, r9 for this value (error #%u)!", i_error_code );
-      break;
-    case LIBXSMM_ERR_CALLEE_SAVE_A_PREF:
-      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
-        "reg_a_prefetch cannot be callee save, since input, please use either rdi, rsi, rdx, rcx, r8, r9 for this value (error #%u)!", i_error_code );
-      break;
-    case LIBXSMM_ERR_CALLEE_SAVE_B_PREF:
-      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
-        "reg_b_prefetch cannot be callee save, since input, please use either rdi, rsi, rdx, rcx, r8, r9 for this value (error #%u)!", i_error_code );
-      break;
     case LIBXSMM_ERR_NO_INDEX_SCALE_ADDR:
       LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
         "Index + Scale addressing mode is currently not implemented (error #%u)!", i_error_code );
@@ -1054,6 +1034,10 @@ const char* libxsmm_strerror(unsigned int i_error_code) {
       LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
         "B is not provided in supported VNNI format (error #%u)!", i_error_code );
       break;
+    case LIBXSMM_ERR_NO_AVX512VL:
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
+        "the AVX512VL instruction set extension is currently not available (error #%u)!", i_error_code );
+      break;
     default: /* we do not know what happened */
       LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
         "an unknown error occurred (error #%u)!", i_error_code );
@@ -1116,3 +1100,301 @@ LIBXSMM_API_INTERN unsigned int libxsmm_compute_equalized_blocking(
   return l_ret;
 }
 
+
+LIBXSMM_API_INTERN libxsmm_meltw_comp_redu_flags libxsmm_get_meltw_comp_redu_flags( libxsmm_meltw_redu_flags flags )
+{
+  switch ( flags ) {
+    case LIBXSMM_MELTW_FLAG_REDUCE_NONE:
+      return LIBXSMM_MELTW_COMP_FLAG_REDUCE_NONE;
+    case LIBXSMM_MELTW_FLAG_REDUCE_OP_ADD:
+      return LIBXSMM_MELTW_COMP_FLAG_REDUCE_OP_ADD;
+    case LIBXSMM_MELTW_FLAG_REDUCE_OP_MAX:
+      return LIBXSMM_MELTW_COMP_FLAG_REDUCE_OP_MAX;
+    case LIBXSMM_MELTW_FLAG_REDUCE_OP_MUL:
+      return LIBXSMM_MELTW_COMP_FLAG_REDUCE_OP_MUL;
+    case LIBXSMM_MELTW_FLAG_REDUCE_ROWS:
+      return LIBXSMM_MELTW_COMP_FLAG_REDUCE_ROWS;
+    case LIBXSMM_MELTW_FLAG_REDUCE_COLS:
+      return LIBXSMM_MELTW_COMP_FLAG_REDUCE_COLS;
+    case LIBXSMM_MELTW_FLAG_REDUCE_ELTS:
+      return LIBXSMM_MELTW_COMP_FLAG_REDUCE_ELTS;
+    case LIBXSMM_MELTW_FLAG_REDUCE_ELTS_SQUARED:
+      return LIBXSMM_MELTW_COMP_FLAG_REDUCE_ELTS_SQUARED;
+    case LIBXSMM_MELTW_FLAG_REDUCE_OP_ADD_ROWS:
+      return LIBXSMM_MELTW_COMP_FLAG_REDUCE_OP_ADD_ROWS;
+    case LIBXSMM_MELTW_FLAG_REDUCE_OP_ADD_COLS:
+      return LIBXSMM_MELTW_COMP_FLAG_REDUCE_OP_ADD_COLS;
+    default:
+      return LIBXSMM_MELTW_COMP_FLAG_REDUCE_NONE;
+  }
+}
+
+
+LIBXSMM_API_INTERN libxsmm_meltw_redu_flags libxsmm_get_meltw_redu_flags( libxsmm_meltw_comp_redu_flags flags )
+{
+  switch ( flags ) {
+    case LIBXSMM_MELTW_COMP_FLAG_REDUCE_NONE:
+      return LIBXSMM_MELTW_FLAG_REDUCE_NONE;
+    case LIBXSMM_MELTW_COMP_FLAG_REDUCE_OP_ADD:
+      return LIBXSMM_MELTW_FLAG_REDUCE_OP_ADD;
+    case LIBXSMM_MELTW_COMP_FLAG_REDUCE_OP_MAX:
+      return LIBXSMM_MELTW_FLAG_REDUCE_OP_MAX;
+    case LIBXSMM_MELTW_COMP_FLAG_REDUCE_OP_MUL:
+      return LIBXSMM_MELTW_FLAG_REDUCE_OP_MUL;
+    case LIBXSMM_MELTW_COMP_FLAG_REDUCE_ROWS:
+      return LIBXSMM_MELTW_FLAG_REDUCE_ROWS;
+    case LIBXSMM_MELTW_COMP_FLAG_REDUCE_COLS:
+      return LIBXSMM_MELTW_FLAG_REDUCE_COLS;
+    case LIBXSMM_MELTW_COMP_FLAG_REDUCE_ELTS:
+      return LIBXSMM_MELTW_FLAG_REDUCE_ELTS;
+    case LIBXSMM_MELTW_COMP_FLAG_REDUCE_ELTS_SQUARED:
+      return LIBXSMM_MELTW_FLAG_REDUCE_ELTS_SQUARED;
+    case LIBXSMM_MELTW_COMP_FLAG_REDUCE_OP_ADD_ROWS:
+      return LIBXSMM_MELTW_FLAG_REDUCE_OP_ADD_ROWS;
+    case LIBXSMM_MELTW_COMP_FLAG_REDUCE_OP_ADD_COLS:
+      return LIBXSMM_MELTW_FLAG_REDUCE_OP_ADD_COLS;
+    default:
+      return LIBXSMM_MELTW_FLAG_REDUCE_NONE;
+  }
+}
+
+
+LIBXSMM_API_INTERN libxsmm_meltw_comp_scal_flags libxsmm_get_meltw_comp_scal_flags( libxsmm_meltw_scal_flags flags )
+{
+  switch( flags ) {
+    case LIBXSMM_MELTW_FLAG_SCALE_NONE:
+      return LIBXSMM_MELTW_COMP_FLAG_SCALE_NONE;
+    case LIBXSMM_MELTW_FLAG_SCALE_MULT:
+      return LIBXSMM_MELTW_COMP_FLAG_SCALE_MULT;
+    case LIBXSMM_MELTW_FLAG_SCALE_SHIFT:
+      return LIBXSMM_MELTW_COMP_FLAG_SCALE_SHIFT;
+    case LIBXSMM_MELTW_FLAG_SCALE_ADD_BIAS:
+      return LIBXSMM_MELTW_COMP_FLAG_SCALE_ADD_BIAS;
+    case LIBXSMM_MELTW_FLAG_SCALE_ROWS:
+      return LIBXSMM_MELTW_COMP_FLAG_SCALE_ROWS;
+    case LIBXSMM_MELTW_FLAG_SCALE_COLS:
+      return LIBXSMM_MELTW_COMP_FLAG_SCALE_COLS;
+    case LIBXSMM_MELTW_FLAG_SCALE_MULT_ROWS:
+      return LIBXSMM_MELTW_COMP_FLAG_SCALE_MULT_ROWS;
+    case LIBXSMM_MELTW_FLAG_SCALE_SHIFT_ROWS:
+      return LIBXSMM_MELTW_COMP_FLAG_SCALE_SHIFT_ROWS;
+    case LIBXSMM_MELTW_FLAG_SCALE_ADD_BIAS_ROWS:
+      return LIBXSMM_MELTW_COMP_FLAG_SCALE_ADD_BIAS_ROWS;
+    case LIBXSMM_MELTW_FLAG_SCALE_MULT_SHIFT_ROWS:
+      return LIBXSMM_MELTW_COMP_FLAG_SCALE_MULT_SHIFT_ROWS;
+    case LIBXSMM_MELTW_FLAG_SCALE_ADD_BIAS_SHIFT_ROWS:
+      return LIBXSMM_MELTW_COMP_FLAG_SCALE_ADD_BIAS_SHIFT_ROWS;
+    case LIBXSMM_MELTW_FLAG_SCALE_MULT_ADD_BIAS_ROWS:
+      return LIBXSMM_MELTW_COMP_FLAG_SCALE_MULT_ADD_BIAS_ROWS;
+    case LIBXSMM_MELTW_FLAG_SCALE_MULT_SHIFT_ADD_BIAS_ROWS:
+      return LIBXSMM_MELTW_COMP_FLAG_SCALE_MULT_SHIFT_ADD_BIAS_ROWS;
+    case LIBXSMM_MELTW_FLAG_SCALE_MULT_COLS:
+      return LIBXSMM_MELTW_COMP_FLAG_SCALE_MULT_COLS;
+    case LIBXSMM_MELTW_FLAG_SCALE_SHIFT_COLS:
+      return LIBXSMM_MELTW_COMP_FLAG_SCALE_SHIFT_COLS;
+    case LIBXSMM_MELTW_FLAG_SCALE_ADD_BIAS_COLS:
+      return LIBXSMM_MELTW_COMP_FLAG_SCALE_ADD_BIAS_COLS;
+    case LIBXSMM_MELTW_FLAG_SCALE_MULT_SHIFT_COLS:
+      return LIBXSMM_MELTW_COMP_FLAG_SCALE_MULT_SHIFT_COLS;
+    case LIBXSMM_MELTW_FLAG_SCALE_ADD_BIAS_SHIFT_COLS:
+      return LIBXSMM_MELTW_COMP_FLAG_SCALE_ADD_BIAS_SHIFT_COLS;
+    case LIBXSMM_MELTW_FLAG_SCALE_MULT_ADD_BIAS_COLS:
+      return LIBXSMM_MELTW_COMP_FLAG_SCALE_MULT_ADD_BIAS_COLS;
+    case LIBXSMM_MELTW_FLAG_SCALE_MULT_SHIFT_ADD_BIAS_COLS:
+      return LIBXSMM_MELTW_COMP_FLAG_SCALE_MULT_SHIFT_ADD_BIAS_COLS;
+    default:
+      return LIBXSMM_MELTW_COMP_FLAG_SCALE_NONE;
+  }
+}
+
+
+LIBXSMM_API_INTERN libxsmm_meltw_scal_flags libxsmm_get_meltw_scal_flags( libxsmm_meltw_comp_scal_flags flags )
+{
+  switch( flags ) {
+    case LIBXSMM_MELTW_COMP_FLAG_SCALE_NONE:
+      return LIBXSMM_MELTW_FLAG_SCALE_NONE;
+    case LIBXSMM_MELTW_COMP_FLAG_SCALE_MULT:
+      return LIBXSMM_MELTW_FLAG_SCALE_MULT;
+    case LIBXSMM_MELTW_COMP_FLAG_SCALE_SHIFT:
+      return LIBXSMM_MELTW_FLAG_SCALE_SHIFT;
+    case LIBXSMM_MELTW_COMP_FLAG_SCALE_ADD_BIAS:
+      return LIBXSMM_MELTW_FLAG_SCALE_ADD_BIAS;
+    case LIBXSMM_MELTW_COMP_FLAG_SCALE_ROWS:
+      return LIBXSMM_MELTW_FLAG_SCALE_ROWS;
+    case LIBXSMM_MELTW_COMP_FLAG_SCALE_COLS:
+      return LIBXSMM_MELTW_FLAG_SCALE_COLS;
+    case LIBXSMM_MELTW_COMP_FLAG_SCALE_MULT_ROWS:
+      return LIBXSMM_MELTW_FLAG_SCALE_MULT_ROWS;
+    case LIBXSMM_MELTW_COMP_FLAG_SCALE_SHIFT_ROWS:
+      return LIBXSMM_MELTW_FLAG_SCALE_SHIFT_ROWS;
+    case LIBXSMM_MELTW_COMP_FLAG_SCALE_ADD_BIAS_ROWS:
+      return LIBXSMM_MELTW_FLAG_SCALE_ADD_BIAS_ROWS;
+    case LIBXSMM_MELTW_COMP_FLAG_SCALE_MULT_SHIFT_ROWS:
+      return LIBXSMM_MELTW_FLAG_SCALE_MULT_SHIFT_ROWS;
+    case LIBXSMM_MELTW_COMP_FLAG_SCALE_ADD_BIAS_SHIFT_ROWS:
+      return LIBXSMM_MELTW_FLAG_SCALE_ADD_BIAS_SHIFT_ROWS;
+    case LIBXSMM_MELTW_COMP_FLAG_SCALE_MULT_ADD_BIAS_ROWS:
+      return LIBXSMM_MELTW_FLAG_SCALE_MULT_ADD_BIAS_ROWS;
+    case LIBXSMM_MELTW_COMP_FLAG_SCALE_MULT_SHIFT_ADD_BIAS_ROWS:
+      return LIBXSMM_MELTW_FLAG_SCALE_MULT_SHIFT_ADD_BIAS_ROWS;
+    case LIBXSMM_MELTW_COMP_FLAG_SCALE_MULT_COLS:
+      return LIBXSMM_MELTW_FLAG_SCALE_MULT_COLS;
+    case LIBXSMM_MELTW_COMP_FLAG_SCALE_SHIFT_COLS:
+      return LIBXSMM_MELTW_FLAG_SCALE_SHIFT_COLS;
+    case LIBXSMM_MELTW_COMP_FLAG_SCALE_ADD_BIAS_COLS:
+      return LIBXSMM_MELTW_FLAG_SCALE_ADD_BIAS_COLS;
+    case LIBXSMM_MELTW_COMP_FLAG_SCALE_MULT_SHIFT_COLS:
+      return LIBXSMM_MELTW_FLAG_SCALE_MULT_SHIFT_COLS;
+    case LIBXSMM_MELTW_COMP_FLAG_SCALE_ADD_BIAS_SHIFT_COLS:
+      return LIBXSMM_MELTW_FLAG_SCALE_ADD_BIAS_SHIFT_COLS;
+    case LIBXSMM_MELTW_COMP_FLAG_SCALE_MULT_ADD_BIAS_COLS:
+      return LIBXSMM_MELTW_FLAG_SCALE_MULT_ADD_BIAS_COLS;
+    case LIBXSMM_MELTW_COMP_FLAG_SCALE_MULT_SHIFT_ADD_BIAS_COLS:
+      return LIBXSMM_MELTW_FLAG_SCALE_MULT_SHIFT_ADD_BIAS_COLS;
+    default:
+      return LIBXSMM_MELTW_FLAG_SCALE_NONE;
+  }
+}
+
+
+LIBXSMM_API_INTERN libxsmm_meltw_comp_cvta_flags libxsmm_get_meltw_comp_cvta_flags( libxsmm_meltw_cvta_flags flags )
+{
+  switch ( flags ) {
+    case LIBXSMM_MELTW_FLAG_CVTA_NONE:
+      return LIBXSMM_MELTW_COMP_FLAG_CVTA_NONE;
+    case LIBXSMM_MELTW_FLAG_CVTA_FUSE_RELU:
+      return LIBXSMM_MELTW_COMP_FLAG_CVTA_FUSE_RELU;
+    case LIBXSMM_MELTW_FLAG_CVTA_FUSE_TANH:
+      return LIBXSMM_MELTW_COMP_FLAG_CVTA_FUSE_TANH;
+    case LIBXSMM_MELTW_FLAG_CVTA_FUSE_SIGM:
+      return LIBXSMM_MELTW_COMP_FLAG_CVTA_FUSE_SIGM;
+    default:
+      return LIBXSMM_MELTW_COMP_FLAG_CVTA_NONE;
+  }
+}
+
+
+LIBXSMM_API_INTERN libxsmm_meltw_cvta_flags libxsmm_get_meltw_cvta_flags( libxsmm_meltw_comp_cvta_flags flags )
+{
+  switch ( flags ) {
+    case LIBXSMM_MELTW_COMP_FLAG_CVTA_NONE:
+      return LIBXSMM_MELTW_FLAG_CVTA_NONE;
+    case LIBXSMM_MELTW_COMP_FLAG_CVTA_FUSE_RELU:
+      return LIBXSMM_MELTW_FLAG_CVTA_FUSE_RELU;
+    case LIBXSMM_MELTW_COMP_FLAG_CVTA_FUSE_TANH:
+      return LIBXSMM_MELTW_FLAG_CVTA_FUSE_TANH;
+    case LIBXSMM_MELTW_COMP_FLAG_CVTA_FUSE_SIGM:
+      return LIBXSMM_MELTW_FLAG_CVTA_FUSE_SIGM;
+    default:
+      return LIBXSMM_MELTW_FLAG_CVTA_NONE;
+  }
+}
+
+
+LIBXSMM_API_INTERN libxsmm_meltw_comp_acvt_flags libxsmm_get_meltw_comp_acvt_flags( libxsmm_meltw_acvt_flags flags )
+{
+  switch ( flags ) {
+    case LIBXSMM_MELTW_FLAG_ACVT_NONE:
+      return LIBXSMM_MELTW_COMP_FLAG_ACVT_NONE;
+    case LIBXSMM_MELTW_FLAG_ACVT_FUSE_TANH:
+      return LIBXSMM_MELTW_COMP_FLAG_ACVT_FUSE_TANH;
+    case LIBXSMM_MELTW_FLAG_ACVT_FUSE_SIGM:
+      return LIBXSMM_MELTW_COMP_FLAG_ACVT_FUSE_SIGM;
+    default:
+      return LIBXSMM_MELTW_COMP_FLAG_ACVT_NONE;
+  }
+}
+
+
+LIBXSMM_API_INTERN libxsmm_meltw_acvt_flags libxsmm_get_meltw_acvt_flags( libxsmm_meltw_comp_acvt_flags flags )
+{
+  switch ( flags ) {
+    case LIBXSMM_MELTW_COMP_FLAG_ACVT_NONE:
+      return LIBXSMM_MELTW_FLAG_ACVT_NONE;
+    case LIBXSMM_MELTW_COMP_FLAG_ACVT_FUSE_TANH:
+      return LIBXSMM_MELTW_FLAG_ACVT_FUSE_TANH;
+    case LIBXSMM_MELTW_COMP_FLAG_ACVT_FUSE_SIGM:
+      return LIBXSMM_MELTW_FLAG_ACVT_FUSE_SIGM;
+    default:
+      return LIBXSMM_MELTW_FLAG_ACVT_NONE;
+  }
+}
+
+
+LIBXSMM_API_INTERN libxsmm_meltw_comp_cbiasact_flags libxsmm_get_meltw_comp_cbiasact_flags( libxsmm_meltw_cbiasact_flags flags )
+{
+  switch ( flags ) {
+    case LIBXSMM_MELTW_FLAG_CBIASACT_NONE:
+      return LIBXSMM_MELTW_COMP_FLAG_CBIASACT_NONE;
+    case LIBXSMM_MELTW_FLAG_CBIASACT_COLBIAS:
+      return LIBXSMM_MELTW_COMP_FLAG_CBIASACT_COLBIAS;
+    case LIBXSMM_MELTW_FLAG_CBIASACT_ACT_RELU:
+      return LIBXSMM_MELTW_COMP_FLAG_CBIASACT_ACT_RELU;
+    case LIBXSMM_MELTW_FLAG_CBIASACT_ACT_TANH:
+      return LIBXSMM_MELTW_COMP_FLAG_CBIASACT_ACT_TANH;
+    case LIBXSMM_MELTW_FLAG_CBIASACT_ACT_SIGM:
+      return LIBXSMM_MELTW_COMP_FLAG_CBIASACT_ACT_SIGM;
+    case LIBXSMM_MELTW_FLAG_CBIASACT_ACT_GELU:
+      return LIBXSMM_MELTW_COMP_FLAG_CBIASACT_ACT_GELU;
+    case LIBXSMM_MELTW_FLAG_CBIASACT_OVERWRITE_C:
+      return LIBXSMM_MELTW_COMP_FLAG_CBIASACT_OVERWRITE_C;
+    case LIBXSMM_MELTW_FLAG_CBIASACT_COLBIAS_ACT_RELU:
+      return LIBXSMM_MELTW_COMP_FLAG_CBIASACT_COLBIAS_ACT_RELU;
+    case LIBXSMM_MELTW_FLAG_CBIASACT_COLBIAS_ACT_TANH:
+      return LIBXSMM_MELTW_COMP_FLAG_CBIASACT_COLBIAS_ACT_TANH;
+    case LIBXSMM_MELTW_FLAG_CBIASACT_COLBIAS_ACT_SIGM:
+      return LIBXSMM_MELTW_COMP_FLAG_CBIASACT_COLBIAS_ACT_SIGM;
+    case LIBXSMM_MELTW_FLAG_CBIASACT_COLBIAS_ACT_GELU:
+      return LIBXSMM_MELTW_COMP_FLAG_CBIASACT_COLBIAS_ACT_GELU;
+    case LIBXSMM_MELTW_FLAG_CBIASACT_COLBIAS_ACT_RELU_OVERWRITE_C:
+      return LIBXSMM_MELTW_COMP_FLAG_CBIASACT_COLBIAS_ACT_RELU_OVERWRITE_C;
+    case LIBXSMM_MELTW_FLAG_CBIASACT_COLBIAS_ACT_TANH_OVERWRITE_C:
+      return LIBXSMM_MELTW_COMP_FLAG_CBIASACT_COLBIAS_ACT_TANH_OVERWRITE_C;
+    case LIBXSMM_MELTW_FLAG_CBIASACT_COLBIAS_ACT_SIGM_OVERWRITE_C:
+      return LIBXSMM_MELTW_COMP_FLAG_CBIASACT_COLBIAS_ACT_SIGM_OVERWRITE_C;
+    case LIBXSMM_MELTW_FLAG_CBIASACT_COLBIAS_ACT_GELU_OVERWRITE_C:
+      return LIBXSMM_MELTW_COMP_FLAG_CBIASACT_COLBIAS_ACT_GELU_OVERWRITE_C;
+    default:
+      return LIBXSMM_MELTW_COMP_FLAG_CBIASACT_NONE;
+  }
+}
+
+
+LIBXSMM_API_INTERN libxsmm_meltw_cbiasact_flags libxsmm_get_meltw_cbiasact_flags( libxsmm_meltw_comp_cbiasact_flags flags )
+{
+  switch ( flags ) {
+    case LIBXSMM_MELTW_COMP_FLAG_CBIASACT_NONE:
+      return LIBXSMM_MELTW_FLAG_CBIASACT_NONE;
+    case LIBXSMM_MELTW_COMP_FLAG_CBIASACT_COLBIAS:
+      return LIBXSMM_MELTW_FLAG_CBIASACT_COLBIAS;
+    case LIBXSMM_MELTW_COMP_FLAG_CBIASACT_ACT_RELU:
+      return LIBXSMM_MELTW_FLAG_CBIASACT_ACT_RELU;
+    case LIBXSMM_MELTW_COMP_FLAG_CBIASACT_ACT_TANH:
+      return LIBXSMM_MELTW_FLAG_CBIASACT_ACT_TANH;
+    case LIBXSMM_MELTW_COMP_FLAG_CBIASACT_ACT_SIGM:
+      return LIBXSMM_MELTW_FLAG_CBIASACT_ACT_SIGM;
+    case LIBXSMM_MELTW_COMP_FLAG_CBIASACT_ACT_GELU:
+      return LIBXSMM_MELTW_FLAG_CBIASACT_ACT_GELU;
+    case LIBXSMM_MELTW_COMP_FLAG_CBIASACT_OVERWRITE_C:
+      return LIBXSMM_MELTW_FLAG_CBIASACT_OVERWRITE_C;
+    case LIBXSMM_MELTW_COMP_FLAG_CBIASACT_COLBIAS_ACT_RELU:
+      return LIBXSMM_MELTW_FLAG_CBIASACT_COLBIAS_ACT_RELU;
+    case LIBXSMM_MELTW_COMP_FLAG_CBIASACT_COLBIAS_ACT_TANH:
+      return LIBXSMM_MELTW_FLAG_CBIASACT_COLBIAS_ACT_TANH;
+    case LIBXSMM_MELTW_COMP_FLAG_CBIASACT_COLBIAS_ACT_SIGM:
+      return LIBXSMM_MELTW_FLAG_CBIASACT_COLBIAS_ACT_SIGM;
+    case LIBXSMM_MELTW_COMP_FLAG_CBIASACT_COLBIAS_ACT_GELU:
+      return LIBXSMM_MELTW_FLAG_CBIASACT_COLBIAS_ACT_GELU;
+    case LIBXSMM_MELTW_COMP_FLAG_CBIASACT_COLBIAS_ACT_RELU_OVERWRITE_C:
+      return LIBXSMM_MELTW_FLAG_CBIASACT_COLBIAS_ACT_RELU_OVERWRITE_C;
+    case LIBXSMM_MELTW_COMP_FLAG_CBIASACT_COLBIAS_ACT_TANH_OVERWRITE_C:
+      return LIBXSMM_MELTW_FLAG_CBIASACT_COLBIAS_ACT_TANH_OVERWRITE_C;
+    case LIBXSMM_MELTW_COMP_FLAG_CBIASACT_COLBIAS_ACT_SIGM_OVERWRITE_C:
+      return LIBXSMM_MELTW_FLAG_CBIASACT_COLBIAS_ACT_SIGM_OVERWRITE_C;
+    case LIBXSMM_MELTW_COMP_FLAG_CBIASACT_COLBIAS_ACT_GELU_OVERWRITE_C:
+      return LIBXSMM_MELTW_FLAG_CBIASACT_COLBIAS_ACT_GELU_OVERWRITE_C;
+    default:
+      return LIBXSMM_MELTW_FLAG_CBIASACT_NONE;
+  }
+}
+
