@@ -138,69 +138,110 @@
 #define LIBXSMM_X86_INSTR_VFNMADD231SS   20028
 #define LIBXSMM_X86_INSTR_VFNMSUB231SS   20029
 #define LIBXSMM_X86_INSTR_VPERMW         20030
+#define LIBXSMM_X86_INSTR_VFMADD132PD    20031
+#define LIBXSMM_X86_INSTR_VFMSUB132PD    20032
+#define LIBXSMM_X86_INSTR_VFNMADD132PD   20033
+#define LIBXSMM_X86_INSTR_VFNMSUB132PD   20034
+#define LIBXSMM_X86_INSTR_VFMADD132SD    20035
+#define LIBXSMM_X86_INSTR_VFMSUB132SD    20036
+#define LIBXSMM_X86_INSTR_VFNMADD132SD   20037
+#define LIBXSMM_X86_INSTR_VFNMSUB132SD   20038
+#define LIBXSMM_X86_INSTR_VFMADD132PS    20039
+#define LIBXSMM_X86_INSTR_VFMSUB132PS    20040
+#define LIBXSMM_X86_INSTR_VFNMADD132PS   20041
+#define LIBXSMM_X86_INSTR_VFNMSUB132PS   20042
+#define LIBXSMM_X86_INSTR_VFMADD132SS    20043
+#define LIBXSMM_X86_INSTR_VFMSUB132SS    20044
+#define LIBXSMM_X86_INSTR_VFNMADD132SS   20045
+#define LIBXSMM_X86_INSTR_VFNMSUB132SS   20046
+#define LIBXSMM_X86_INSTR_VFMADD213PD    20047
+#define LIBXSMM_X86_INSTR_VFMSUB213PD    20048
+#define LIBXSMM_X86_INSTR_VFNMADD213PD   20049
+#define LIBXSMM_X86_INSTR_VFNMSUB213PD   20050
+#define LIBXSMM_X86_INSTR_VFMADD213SD    20051
+#define LIBXSMM_X86_INSTR_VFMSUB213SD    20052
+#define LIBXSMM_X86_INSTR_VFNMADD213SD   20053
+#define LIBXSMM_X86_INSTR_VFNMSUB213SD   20054
+#define LIBXSMM_X86_INSTR_VFMADD213PS    20055
+#define LIBXSMM_X86_INSTR_VFMSUB213PS    20056
+#define LIBXSMM_X86_INSTR_VFNMADD213PS   20057
+#define LIBXSMM_X86_INSTR_VFNMSUB213PS   20058
+#define LIBXSMM_X86_INSTR_VFMADD213SS    20059
+#define LIBXSMM_X86_INSTR_VFMSUB213SS    20060
+#define LIBXSMM_X86_INSTR_VFNMADD213SS   20061
+#define LIBXSMM_X86_INSTR_VFNMSUB213SS   20062
 /* SSE */
-#define LIBXSMM_X86_INSTR_XORPD          20031
-#define LIBXSMM_X86_INSTR_MULPD          20032
-#define LIBXSMM_X86_INSTR_ADDPD          20033
-#define LIBXSMM_X86_INSTR_SUBPD          20034
-#define LIBXSMM_X86_INSTR_MULSD          20035
-#define LIBXSMM_X86_INSTR_ADDSD          20036
-#define LIBXSMM_X86_INSTR_SUBSD          20037
-#define LIBXSMM_X86_INSTR_XORPS          20038
-#define LIBXSMM_X86_INSTR_MULPS          20039
-#define LIBXSMM_X86_INSTR_ADDPS          20040
-#define LIBXSMM_X86_INSTR_SUBPS          20041
-#define LIBXSMM_X86_INSTR_MULSS          20042
-#define LIBXSMM_X86_INSTR_ADDSS          20043
-#define LIBXSMM_X86_INSTR_SUBSS          20044
+#define LIBXSMM_X86_INSTR_XORPD          20063
+#define LIBXSMM_X86_INSTR_MULPD          20064
+#define LIBXSMM_X86_INSTR_ADDPD          20065
+#define LIBXSMM_X86_INSTR_SUBPD          20066
+#define LIBXSMM_X86_INSTR_MULSD          20067
+#define LIBXSMM_X86_INSTR_ADDSD          20068
+#define LIBXSMM_X86_INSTR_SUBSD          20069
+#define LIBXSMM_X86_INSTR_XORPS          20070
+#define LIBXSMM_X86_INSTR_MULPS          20071
+#define LIBXSMM_X86_INSTR_ADDPS          20072
+#define LIBXSMM_X86_INSTR_SUBPS          20073
+#define LIBXSMM_X86_INSTR_MULSS          20074
+#define LIBXSMM_X86_INSTR_ADDSS          20075
+#define LIBXSMM_X86_INSTR_SUBSS          20076
 /* AVX512F: Integer XOR as there is no FP */
-#define LIBXSMM_X86_INSTR_VPXORD         20045
+#define LIBXSMM_X86_INSTR_VPXORD         20077
 /* additional integer stuff */
-#define LIBXSMM_X86_INSTR_VPADDQ         20046
-#define LIBXSMM_X86_INSTR_VPADDD         20047
-#define LIBXSMM_X86_INSTR_VPADDW         20048
-#define LIBXSMM_X86_INSTR_VPADDB         20049
-#define LIBXSMM_X86_INSTR_VPMADDWD       20050
-#define LIBXSMM_X86_INSTR_VPMADDUBSW     20051
-#define LIBXSMM_X86_INSTR_VPADDSW        20052
-#define LIBXSMM_X86_INSTR_VPADDSB        20053
-#define LIBXSMM_X86_INSTR_VPSUBD         20054
+#define LIBXSMM_X86_INSTR_VPADDQ         20078
+#define LIBXSMM_X86_INSTR_VPADDD         20079
+#define LIBXSMM_X86_INSTR_VPADDW         20080
+#define LIBXSMM_X86_INSTR_VPADDB         20081
+#define LIBXSMM_X86_INSTR_VPMADDWD       20082
+#define LIBXSMM_X86_INSTR_VPMADDUBSW     20083
+#define LIBXSMM_X86_INSTR_VPADDSW        20084
+#define LIBXSMM_X86_INSTR_VPADDSB        20085
+#define LIBXSMM_X86_INSTR_VPSUBD         20086
 /* Additional vector manipulations */
-#define LIBXSMM_X86_INSTR_VUNPCKLPD      20055
-#define LIBXSMM_X86_INSTR_VUNPCKLPS      20056
-#define LIBXSMM_X86_INSTR_VUNPCKHPD      20057
-#define LIBXSMM_X86_INSTR_VUNPCKHPS      20058
-#define LIBXSMM_X86_INSTR_VPSRAVD        20059
-#define LIBXSMM_X86_INSTR_VCVTDQ2PS      20060
-#define LIBXSMM_X86_INSTR_VDIVPS         20061
-#define LIBXSMM_X86_INSTR_VDIVPD         20062
-#define LIBXSMM_X86_INSTR_VFMADD213PS    20063
-#define LIBXSMM_X86_INSTR_VCVTPS2PD      20064
-#define LIBXSMM_X86_INSTR_VBLENDMPS      20065
-#define LIBXSMM_X86_INSTR_VCMPPS         20066
-#define LIBXSMM_X86_INSTR_VPANDD         20067
-#define LIBXSMM_X86_INSTR_VPANDQ         20068
-#define LIBXSMM_X86_INSTR_VMAXPD         20069
-#define LIBXSMM_X86_INSTR_VMAXPS         20070
-#define LIBXSMM_X86_INSTR_VCVTPS2PH      20071
-#define LIBXSMM_X86_INSTR_VCVTPH2PS      20072
-#define LIBXSMM_X86_INSTR_VPERMD         20073
-#define LIBXSMM_X86_INSTR_VPMOVDW        20074
-#define LIBXSMM_X86_INSTR_VPSRAD         20075
-#define LIBXSMM_X86_INSTR_VPSLLD         20076
-#define LIBXSMM_X86_INSTR_VPCMPD         20077
-#define LIBXSMM_X86_INSTR_VPORD          20078
-#define LIBXSMM_X86_INSTR_VPSRLD         20079
-#define LIBXSMM_X86_INSTR_VPERMT2W       20080
-#define LIBXSMM_X86_INSTR_VPMOVSXWD      20081
-#define LIBXSMM_X86_INSTR_VPMOVDB        20082
-#define LIBXSMM_X86_INSTR_VPMOVSDB       20083
-#define LIBXSMM_X86_INSTR_VPMOVUSDB      20084
-#define LIBXSMM_X86_INSTR_VCVTPS2DQ      20085
-#define LIBXSMM_X86_INSTR_VCVTPS2UDQ     20086
-#define LIBXSMM_X86_INSTR_VPMOVZXWD      20087
-#define LIBXSMM_X86_INSTR_VPMOVSXBD      20088
-#define LIBXSMM_X86_INSTR_VPMOVZXBD      20089
+#define LIBXSMM_X86_INSTR_VUNPCKLPD      20087
+#define LIBXSMM_X86_INSTR_VUNPCKLPS      20088
+#define LIBXSMM_X86_INSTR_VUNPCKHPD      20089
+#define LIBXSMM_X86_INSTR_VUNPCKHPS      20090
+#define LIBXSMM_X86_INSTR_VPSRAVD        20091
+#define LIBXSMM_X86_INSTR_VCVTDQ2PS      20092
+#define LIBXSMM_X86_INSTR_VDIVPS         20093
+#define LIBXSMM_X86_INSTR_VDIVPD         20094
+#define LIBXSMM_X86_INSTR_VCVTPS2PD      20095
+#define LIBXSMM_X86_INSTR_VBLENDMPS      20096
+#define LIBXSMM_X86_INSTR_VCMPPS         20097
+#define LIBXSMM_X86_INSTR_VPANDD         20098
+#define LIBXSMM_X86_INSTR_VPANDQ         20099
+#define LIBXSMM_X86_INSTR_VMAXPD         20100
+#define LIBXSMM_X86_INSTR_VMAXPS         20101
+#define LIBXSMM_X86_INSTR_VCVTPS2PH      20102
+#define LIBXSMM_X86_INSTR_VCVTPH2PS      20103
+#define LIBXSMM_X86_INSTR_VPERMD         20104
+#define LIBXSMM_X86_INSTR_VPMOVDW        20105
+#define LIBXSMM_X86_INSTR_VPSRAD         20106
+#define LIBXSMM_X86_INSTR_VPSLLD         20107
+#define LIBXSMM_X86_INSTR_VPCMPB         20108
+#define LIBXSMM_X86_INSTR_VPCMPD         20109
+#define LIBXSMM_X86_INSTR_VPCMPW         20110
+#define LIBXSMM_X86_INSTR_VPCMPUB        20111
+#define LIBXSMM_X86_INSTR_VPCMPUD        20112
+#define LIBXSMM_X86_INSTR_VPCMPUW        20113
+#define LIBXSMM_X86_INSTR_VPORD          20114
+#define LIBXSMM_X86_INSTR_VPSRLD         20115
+#define LIBXSMM_X86_INSTR_VPERMT2W       20116
+#define LIBXSMM_X86_INSTR_VPMOVSXWD      20117
+#define LIBXSMM_X86_INSTR_VPMOVDB        20118
+#define LIBXSMM_X86_INSTR_VPMOVSDB       20119
+#define LIBXSMM_X86_INSTR_VPMOVUSDB      20120
+#define LIBXSMM_X86_INSTR_VCVTPS2DQ      20121
+#define LIBXSMM_X86_INSTR_VCVTPS2UDQ     20122
+#define LIBXSMM_X86_INSTR_VPMOVZXWD      20123
+#define LIBXSMM_X86_INSTR_VPMOVSXBD      20124
+#define LIBXSMM_X86_INSTR_VPMOVZXBD      20125
+#define LIBXSMM_X86_INSTR_VPBLENDMB      20126
+#define LIBXSMM_X86_INSTR_VPBLENDMW      20127
+#define LIBXSMM_X86_INSTR_VRCP14PS       20128
+#define LIBXSMM_X86_INSTR_VPMAXSD        20129
+#define LIBXSMM_X86_INSTR_VPMINSD        20130
 
 /* AVX512, QUAD MADD, QUAD VNNI and VNNI */
 #define LIBXSMM_X86_INSTR_V4FMADDPS      26000
@@ -248,6 +289,7 @@
 #define LIBXSMM_X86_INSTR_POPCNT         30025
 #define LIBXSMM_X86_INSTR_TZCNT          30026
 #define LIBXSMM_X86_INSTR_LEAQ           30027
+#define LIBXSMM_X86_INSTR_ANDQ           30028
 
 /* Mask move instructions */
 #define LIBXSMM_X86_INSTR_KMOV           40000
@@ -260,54 +302,50 @@
 #define LIBXSMM_X86_INSTR_KXNORW         45000
 
 /* define error codes */
-#define LIBXSMM_ERR_GENERAL              90000
-#define LIBXSMM_ERR_ALLOC                90001
-#define LIBXSMM_ERR_BUFFER_TOO_SMALL     90002
-#define LIBXSMM_ERR_APPEND_STR           90003
-#define LIBXSMM_ERR_ARCH_PREC            90004
-#define LIBXSMM_ERR_ARCH                 90005
-#define LIBXSMM_ERR_UNSUP_ARCH           90006
-#define LIBXSMM_ERR_LDA                  90007
-#define LIBXSMM_ERR_LDB                  90008
-#define LIBXSMM_ERR_LDC                  90009
-#define LIBXSMM_ERR_SPGEMM_GEN           90010
-#define LIBXSMM_ERR_CSC_INPUT            90011
-#define LIBXSMM_ERR_CSC_READ_LEN         90012
-#define LIBXSMM_ERR_CSC_READ_DESC        90013
-#define LIBXSMM_ERR_CSC_READ_ELEMS       90014
-#define LIBXSMM_ERR_CSC_LEN              90015
-#define LIBXSMM_ERR_N_BLOCK              90016
-#define LIBXSMM_ERR_M_BLOCK              90017
-#define LIBXSMM_ERR_K_BLOCK              90018
-#define LIBXSMM_ERR_REG_BLOCK            90020
-#define LIBXSMM_ERR_NO_AVX512_BCAST      90022
-#define LIBXSMM_ERR_NO_AVX512_QFMA       90023
-#define LIBXSMM_ERR_CALLEE_SAVE_A        90024
-#define LIBXSMM_ERR_CALLEE_SAVE_B        90025
-#define LIBXSMM_ERR_CALLEE_SAVE_C        90026
-#define LIBXSMM_ERR_CALLEE_SAVE_A_PREF   90027
-#define LIBXSMM_ERR_CALLEE_SAVE_B_PREF   90028
-#define LIBXSMM_ERR_NO_INDEX_SCALE_ADDR  90029
-#define LIBXSMM_ERR_UNSUPPORTED_JUMP     90030
-#define LIBXSMM_ERR_NO_JMPLBL_AVAIL      90031
-#define LIBXSMM_ERR_EXCEED_JMPLBL        90032
-#define LIBXSMM_ERR_CSC_ALLOC_DATA       90033
-#define LIBXSMM_ERR_CSR_ALLOC_DATA       90034
-#define LIBXSMM_ERR_CSR_INPUT            90035
-#define LIBXSMM_ERR_CSR_READ_LEN         90036
-#define LIBXSMM_ERR_CSR_READ_DESC        90037
-#define LIBXSMM_ERR_CSR_READ_ELEMS       90038
-#define LIBXSMM_ERR_CSR_LEN              90039
-#define LIBXSMM_ERR_UNSUP_DATATYPE       90049
-#define LIBXSMM_ERR_UNSUP_DT_FORMAT      90050
-#define LIBXSMM_ERR_INVALID_GEMM_CONFIG  90051
-#define LIBXSMM_ERR_UNIQUE_VAL           90052
-#define LIBXSMM_ERR_VEC_REG_MUST_BE_UNDEF 90053
-#define LIBXSMM_ERR_JMPLBL_USED           90054
-#define LIBXSMM_ERR_TRANS_B               90055
-#define LIBXSMM_ERR_LDB_TRANS             90056
-#define LIBXSMM_ERR_VNNI_A                90057
-#define LIBXSMM_ERR_VNNI_B                90058
+#define LIBXSMM_ERR_GENERAL               90000
+#define LIBXSMM_ERR_ALLOC                 90001
+#define LIBXSMM_ERR_BUFFER_TOO_SMALL      90002
+#define LIBXSMM_ERR_APPEND_STR            90003
+#define LIBXSMM_ERR_ARCH_PREC             90004
+#define LIBXSMM_ERR_ARCH                  90005
+#define LIBXSMM_ERR_UNSUP_ARCH            90006
+#define LIBXSMM_ERR_LDA                   90007
+#define LIBXSMM_ERR_LDB                   90008
+#define LIBXSMM_ERR_LDC                   90009
+#define LIBXSMM_ERR_SPGEMM_GEN            90010
+#define LIBXSMM_ERR_CSC_INPUT             90011
+#define LIBXSMM_ERR_CSC_READ_LEN          90012
+#define LIBXSMM_ERR_CSC_READ_DESC         90013
+#define LIBXSMM_ERR_CSC_READ_ELEMS        90014
+#define LIBXSMM_ERR_CSC_LEN               90015
+#define LIBXSMM_ERR_N_BLOCK               90016
+#define LIBXSMM_ERR_M_BLOCK               90017
+#define LIBXSMM_ERR_K_BLOCK               90018
+#define LIBXSMM_ERR_REG_BLOCK             90019
+#define LIBXSMM_ERR_NO_AVX512_BCAST       90020
+#define LIBXSMM_ERR_NO_AVX512_QFMA        90021
+#define LIBXSMM_ERR_NO_INDEX_SCALE_ADDR   90022
+#define LIBXSMM_ERR_UNSUPPORTED_JUMP      90023
+#define LIBXSMM_ERR_NO_JMPLBL_AVAIL       90024
+#define LIBXSMM_ERR_EXCEED_JMPLBL         90025
+#define LIBXSMM_ERR_CSC_ALLOC_DATA        90026
+#define LIBXSMM_ERR_CSR_ALLOC_DATA        90027
+#define LIBXSMM_ERR_CSR_INPUT             90028
+#define LIBXSMM_ERR_CSR_READ_LEN          90029
+#define LIBXSMM_ERR_CSR_READ_DESC         90030
+#define LIBXSMM_ERR_CSR_READ_ELEMS        90031
+#define LIBXSMM_ERR_CSR_LEN               90032
+#define LIBXSMM_ERR_UNSUP_DATATYPE        90033
+#define LIBXSMM_ERR_UNSUP_DT_FORMAT       90034
+#define LIBXSMM_ERR_INVALID_GEMM_CONFIG   90035
+#define LIBXSMM_ERR_UNIQUE_VAL            90036
+#define LIBXSMM_ERR_VEC_REG_MUST_BE_UNDEF 90037
+#define LIBXSMM_ERR_JMPLBL_USED           90038
+#define LIBXSMM_ERR_TRANS_B               90039
+#define LIBXSMM_ERR_LDB_TRANS             90040
+#define LIBXSMM_ERR_VNNI_A                90041
+#define LIBXSMM_ERR_VNNI_B                90042
+#define LIBXSMM_ERR_NO_AVX512VL           90043
 
 #if defined(LIBXSMM_HANDLE_ERROR_QUIET)
 # define LIBXSMM_HANDLE_ERROR(GENERATED_CODE, ERROR_CODE)
@@ -396,6 +434,41 @@ LIBXSMM_EXTERN_C typedef struct libxsmm_matcopy_kernel_config_struct {
   char vector_name;
 } libxsmm_matcopy_kernel_config;
 
+/* structure for storing the current gp reg mapping for mateltwise */
+LIBXSMM_EXTERN_C typedef struct libxsmm_mateltwise_gp_reg_mapping_struct {
+  unsigned int gp_reg_param_struct;
+  unsigned int gp_reg_in;
+  unsigned int gp_reg_ldi;
+  unsigned int gp_reg_out;
+  unsigned int gp_reg_ldo;
+  unsigned int gp_reg_relumask;
+  unsigned int gp_reg_reduced_elts;
+  unsigned int gp_reg_reduced_elts_squared;
+  unsigned int gp_reg_scale_vals;
+  unsigned int gp_reg_shift_vals;
+  unsigned int gp_reg_bias_vals;
+  unsigned int gp_reg_m_loop;
+  unsigned int gp_reg_n_loop;
+} libxsmm_mateltwise_gp_reg_mapping;
+
+/* mateltwise kernel configuration */
+LIBXSMM_EXTERN_C typedef struct libxsmm_mateltwise_kernel_config_struct {
+  unsigned int instruction_set;
+  unsigned int vector_reg_count;
+  unsigned int vector_length_in;
+  unsigned int vector_length_out;
+  unsigned int datatype_size_in;
+  unsigned int datatype_size_out;
+  unsigned int vmove_instruction_in;
+  unsigned int vmove_instruction_out;
+  unsigned int alu_add_instruction;
+  unsigned int alu_cmp_instruction;
+  unsigned int alu_jmp_instruction;
+  unsigned int alu_mov_instruction;
+  unsigned int vxor_instruction;
+  char vector_name;
+} libxsmm_mateltwise_kernel_config;
+
 /* structure for storing the current gp reg mapping for transpose */
 LIBXSMM_EXTERN_C typedef struct libxsmm_transpose_gp_reg_mapping_struct {
   unsigned int gp_reg_a;
@@ -438,6 +511,78 @@ LIBXSMM_EXTERN_C typedef struct libxsmm_jump_label_tracker_struct {
   libxsmm_jump_source label_source[32];
 } libxsmm_jump_label_tracker;
 
+/* compressed meltw reduce structure */
+typedef enum libxsmm_meltw_comp_redu_flags {
+  LIBXSMM_MELTW_COMP_FLAG_REDUCE_NONE         = 0,
+  LIBXSMM_MELTW_COMP_FLAG_REDUCE_OP_ADD       = 1,
+  LIBXSMM_MELTW_COMP_FLAG_REDUCE_OP_MAX       = 2,
+  LIBXSMM_MELTW_COMP_FLAG_REDUCE_OP_MUL       = 3,
+  LIBXSMM_MELTW_COMP_FLAG_REDUCE_ROWS         = 4,
+  LIBXSMM_MELTW_COMP_FLAG_REDUCE_COLS         = 5,
+  LIBXSMM_MELTW_COMP_FLAG_REDUCE_ELTS         = 6,
+  LIBXSMM_MELTW_COMP_FLAG_REDUCE_ELTS_SQUARED = 7,
+  LIBXSMM_MELTW_COMP_FLAG_REDUCE_OP_ADD_ROWS  = 8,
+  LIBXSMM_MELTW_COMP_FLAG_REDUCE_OP_ADD_COLS  = 9
+} libxsmm_meltw_comp_redu_flags;
+
+/* compressed meltw scale structure */
+typedef enum libxsmm_meltw_comp_scal_flags {
+  LIBXSMM_MELTW_COMP_FLAG_SCALE_NONE                     = 0,
+  LIBXSMM_MELTW_COMP_FLAG_SCALE_MULT                     = 1,
+  LIBXSMM_MELTW_COMP_FLAG_SCALE_SHIFT                    = 2,
+  LIBXSMM_MELTW_COMP_FLAG_SCALE_ADD_BIAS                 = 3,
+  LIBXSMM_MELTW_COMP_FLAG_SCALE_ROWS                     = 4,
+  LIBXSMM_MELTW_COMP_FLAG_SCALE_COLS                     = 5,
+  LIBXSMM_MELTW_COMP_FLAG_SCALE_MULT_ROWS                = 6,
+  LIBXSMM_MELTW_COMP_FLAG_SCALE_SHIFT_ROWS               = 7,
+  LIBXSMM_MELTW_COMP_FLAG_SCALE_ADD_BIAS_ROWS            = 8,
+  LIBXSMM_MELTW_COMP_FLAG_SCALE_MULT_SHIFT_ROWS          = 9,
+  LIBXSMM_MELTW_COMP_FLAG_SCALE_ADD_BIAS_SHIFT_ROWS      = 10,
+  LIBXSMM_MELTW_COMP_FLAG_SCALE_MULT_ADD_BIAS_ROWS       = 11,
+  LIBXSMM_MELTW_COMP_FLAG_SCALE_MULT_SHIFT_ADD_BIAS_ROWS = 12,
+  LIBXSMM_MELTW_COMP_FLAG_SCALE_MULT_COLS                = 13,
+  LIBXSMM_MELTW_COMP_FLAG_SCALE_SHIFT_COLS               = 14,
+  LIBXSMM_MELTW_COMP_FLAG_SCALE_ADD_BIAS_COLS            = 15,
+  LIBXSMM_MELTW_COMP_FLAG_SCALE_MULT_SHIFT_COLS          = 16,
+  LIBXSMM_MELTW_COMP_FLAG_SCALE_ADD_BIAS_SHIFT_COLS      = 17,
+  LIBXSMM_MELTW_COMP_FLAG_SCALE_MULT_ADD_BIAS_COLS       = 18,
+  LIBXSMM_MELTW_COMP_FLAG_SCALE_MULT_SHIFT_ADD_BIAS_COLS = 19
+} libxsmm_meltw_comp_scal_flags;
+
+/* compressed metlw cvta strcuture */
+typedef enum libxsmm_meltw_comp_cvta_flags {
+  LIBXSMM_MELTW_COMP_FLAG_CVTA_NONE           = 0,
+  LIBXSMM_MELTW_COMP_FLAG_CVTA_FUSE_RELU      = 1,
+  LIBXSMM_MELTW_COMP_FLAG_CVTA_FUSE_TANH      = 2,
+  LIBXSMM_MELTW_COMP_FLAG_CVTA_FUSE_SIGM      = 3
+} libxsmm_meltw_comp_cvta_flags;
+
+/* compressed meltw acvt structure */
+typedef enum libxsmm_meltw_comp_acvt_flags {
+  LIBXSMM_MELTW_COMP_FLAG_ACVT_NONE           = 0,
+  LIBXSMM_MELTW_COMP_FLAG_ACVT_FUSE_TANH      = 1,
+  LIBXSMM_MELTW_COMP_FLAG_ACVT_FUSE_SIGM      = 2
+} libxsmm_meltw_comp_acvt_flags;
+
+/* compressed meltw cbiasact strcuture */
+typedef enum libxsmm_meltw_comp_cbiasact_flags {
+  LIBXSMM_MELTW_COMP_FLAG_CBIASACT_NONE                         =  0,
+  LIBXSMM_MELTW_COMP_FLAG_CBIASACT_COLBIAS                      =  1,
+  LIBXSMM_MELTW_COMP_FLAG_CBIASACT_ACT_RELU                     =  2,
+  LIBXSMM_MELTW_COMP_FLAG_CBIASACT_ACT_TANH                     =  3,
+  LIBXSMM_MELTW_COMP_FLAG_CBIASACT_ACT_SIGM                     =  4,
+  LIBXSMM_MELTW_COMP_FLAG_CBIASACT_ACT_GELU                     =  5,
+  LIBXSMM_MELTW_COMP_FLAG_CBIASACT_OVERWRITE_C                  =  6,
+  LIBXSMM_MELTW_COMP_FLAG_CBIASACT_COLBIAS_ACT_RELU             =  7,
+  LIBXSMM_MELTW_COMP_FLAG_CBIASACT_COLBIAS_ACT_TANH             =  8,
+  LIBXSMM_MELTW_COMP_FLAG_CBIASACT_COLBIAS_ACT_SIGM             =  9,
+  LIBXSMM_MELTW_COMP_FLAG_CBIASACT_COLBIAS_ACT_GELU             = 10,
+  LIBXSMM_MELTW_COMP_FLAG_CBIASACT_COLBIAS_ACT_RELU_OVERWRITE_C = 11,
+  LIBXSMM_MELTW_COMP_FLAG_CBIASACT_COLBIAS_ACT_TANH_OVERWRITE_C = 12,
+  LIBXSMM_MELTW_COMP_FLAG_CBIASACT_COLBIAS_ACT_SIGM_OVERWRITE_C = 13,
+  LIBXSMM_MELTW_COMP_FLAG_CBIASACT_COLBIAS_ACT_GELU_OVERWRITE_C = 14
+} libxsmm_meltw_comp_cbiasact_flags;
+
 LIBXSMM_API_INTERN
 void libxsmm_reset_loop_label_tracker( libxsmm_loop_label_tracker* io_loop_label_tracker );
 
@@ -450,7 +595,7 @@ void libxsmm_get_x86_gp_reg_name( const unsigned int i_gp_reg_number,
                                   const int          i_gp_reg_name_max_length );
 
 LIBXSMM_API_INTERN
-unsigned int libxsmm_check_x86_gp_reg_name_callee_save( const unsigned int i_gp_reg_number );
+unsigned int libxsmm_check_x86_gp_reg_callee_save( const unsigned int i_gp_reg_number );
 
 LIBXSMM_API_INTERN
 void libxsmm_get_x86_instr_name( const unsigned int i_instr_number,
@@ -494,5 +639,17 @@ LIBXSMM_API_INTERN unsigned int libxsmm_compute_equalized_blocking(
   unsigned int* o_range_1, unsigned int* o_block_1,
   unsigned int* o_range_2, unsigned int* o_block_2 );
 
+/** helper functions for compressing and decompressing meltw flags */
+LIBXSMM_API_INTERN libxsmm_meltw_comp_redu_flags libxsmm_get_meltw_comp_redu_flags( libxsmm_meltw_redu_flags flags );
+LIBXSMM_API_INTERN libxsmm_meltw_redu_flags libxsmm_get_meltw_redu_flags( libxsmm_meltw_comp_redu_flags flags );
+LIBXSMM_API_INTERN libxsmm_meltw_comp_scal_flags libxsmm_get_meltw_comp_scal_flags( libxsmm_meltw_scal_flags flags );
+LIBXSMM_API_INTERN libxsmm_meltw_scal_flags libxsmm_get_meltw_scal_flags( libxsmm_meltw_comp_scal_flags flags );
+LIBXSMM_API_INTERN libxsmm_meltw_comp_cvta_flags libxsmm_get_meltw_comp_cvta_flags( libxsmm_meltw_cvta_flags flags );
+LIBXSMM_API_INTERN libxsmm_meltw_cvta_flags libxsmm_get_meltw_cvta_flags( libxsmm_meltw_comp_cvta_flags flags );
+LIBXSMM_API_INTERN libxsmm_meltw_comp_acvt_flags libxsmm_get_meltw_comp_acvt_flags( libxsmm_meltw_acvt_flags flags );
+LIBXSMM_API_INTERN libxsmm_meltw_acvt_flags libxsmm_get_meltw_acvt_flags( libxsmm_meltw_comp_acvt_flags flags );
+LIBXSMM_API_INTERN libxsmm_meltw_comp_cbiasact_flags libxsmm_get_meltw_comp_cbiasact_flags( libxsmm_meltw_cbiasact_flags flags );
+LIBXSMM_API_INTERN libxsmm_meltw_cbiasact_flags libxsmm_get_meltw_cbiasact_flags( libxsmm_meltw_comp_cbiasact_flags flags );
+
 #endif /* GENERATOR_COMMON_H */
 
