@@ -1,30 +1,10 @@
 /******************************************************************************
-** Copyright (c) 2015-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Alexander Heinecke (Intel Corp.)
 ******************************************************************************/
@@ -32,16 +12,6 @@
 #include "generator_x86_instructions.h"
 #include "libxsmm_main.h"
 
-#if defined(LIBXSMM_OFFLOAD_TARGET)
-# pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
-#endif
-#include <stdlib.h>
-#include <string.h>
-#include <stdio.h>
-#if defined(LIBXSMM_OFFLOAD_TARGET)
-# pragma offload_attribute(pop)
-#endif
-
 LIBXSMM_API_INTERN
 void libxsmm_generator_gemm_sse3_microkernel( libxsmm_generated_code*             io_generated_code,
                                                const libxsmm_gp_reg_mapping*      i_gp_reg_mapping,
@@ -62,7 +32,7 @@ void libxsmm_generator_gemm_sse3_microkernel( libxsmm_generated_code*
   /* temp variable for b-offset to handle no-trans/trans B */
   int l_b_offset = 0;
 
-#if !defined(NDEBUG)
+  /* check that m_blocking is a multiple of vlen and that n_blocking is valid */
   if ( (i_n_blocking > 3) || (i_n_blocking < 1) ) {
     LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_N_BLOCK );
     return;
@@ -71,7 +41,6 @@ void libxsmm_generator_gemm_sse3_microkernel( libxsmm_generated_code*
     LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_M_BLOCK );
     return;
   }
-#endif
 
   if (l_m_blocking == 1) {
     /* load column vectors of A */
@@ -82,7 +51,7 @@ void libxsmm_generator_gemm_sse3_microkernel( libxsmm_generated_code*
                                   LIBXSMM_X86_GP_REG_UNDEF, 0,
                                   0,
                                   i_micro_kernel_config->vector_name,
-                                  i_n_blocking, i_micro_kernel_config->use_masking_a_c, 1, 0 );
+                                  i_n_blocking, 0, 1, 0 );
     /* loop over columns of B */
     for ( l_n = 0; l_n < i_n_blocking; l_n++ ) {
       /* post increment of a pointer early */
@@ -108,7 +77,7 @@ void libxsmm_generator_gemm_sse3_microkernel( libxsmm_generated_code*
                                       LIBXSMM_X86_GP_REG_UNDEF, 0,
                                       l_b_offset,
                                       i_micro_kernel_config->vector_name,
-                                      l_n, i_micro_kernel_config->use_masking_a_c, 1, 0 );
+                                      l_n, 0, 1, 0 );
         /* generate shuffle as SSE3 has no broadcast load for single precision */
         if ( LIBXSMM_GEMM_PRECISION_F32 == LIBXSMM_GETENUM_INP( i_xgemm_desc->datatype ) && ( i_micro_kernel_config->b_shuff_instruction != LIBXSMM_X86_INSTR_UNDEF ) ) {
           libxsmm_x86_instruction_vec_shuffle_reg( io_generated_code,
@@ -135,7 +104,7 @@ void libxsmm_generator_gemm_sse3_microkernel( libxsmm_generated_code*
                                       LIBXSMM_X86_GP_REG_UNDEF, 0,
                                       l_b_offset,
                                       i_micro_kernel_config->vector_name,
-                                      l_n, i_micro_kernel_config->use_masking_a_c, 1, 0 );
+                                      l_n, 0, 1, 0 );
         /* generate shuffle as SSE3 has no broadcast load for single precision */
         if ( LIBXSMM_GEMM_PRECISION_F32 == LIBXSMM_GETENUM_INP( i_xgemm_desc->datatype ) && ( i_micro_kernel_config->b_shuff_instruction != LIBXSMM_X86_INSTR_UNDEF ) ) {
           libxsmm_x86_instruction_vec_shuffle_reg( io_generated_code,
@@ -195,7 +164,7 @@ void libxsmm_generator_gemm_sse3_microkernel( libxsmm_generated_code*
                                       LIBXSMM_X86_GP_REG_UNDEF, 0,
                                       l_b_offset,
                                       i_micro_kernel_config->vector_name,
-                                      l_n, i_micro_kernel_config->use_masking_a_c, 1, 0 );
+                                      l_n, 0, 1, 0 );
         /* generate shuffle as SSE3 has no broadcast load for single precision */
         if ( LIBXSMM_GEMM_PRECISION_F32 == LIBXSMM_GETENUM_INP( i_xgemm_desc->datatype ) && ( i_micro_kernel_config->b_shuff_instruction != LIBXSMM_X86_INSTR_UNDEF ) ) {
           libxsmm_x86_instruction_vec_shuffle_reg( io_generated_code,
@@ -224,7 +193,7 @@ void libxsmm_generator_gemm_sse3_microkernel( libxsmm_generated_code*
                                       LIBXSMM_X86_GP_REG_UNDEF, 0,
                                       l_b_offset,
                                       i_micro_kernel_config->vector_name,
-                                      l_n, i_micro_kernel_config->use_masking_a_c, 1, 0 );
+                                      l_n, 0, 1, 0 );
         /* generate shuffle as SSE3 has no broadcast load for single precision */
         if ( LIBXSMM_GEMM_PRECISION_F32 == LIBXSMM_GETENUM_INP( i_xgemm_desc->datatype ) ) {
           libxsmm_x86_instruction_vec_shuffle_reg( io_generated_code,
@@ -260,7 +229,7 @@ void libxsmm_generator_gemm_sse3_microkernel( libxsmm_generated_code*
                                       LIBXSMM_X86_GP_REG_UNDEF, 0,
                                       (i_micro_kernel_config->datatype_size) * (i_micro_kernel_config->vector_length) * l_m,
                                       i_micro_kernel_config->vector_name,
-                                      i_n_blocking, i_micro_kernel_config->use_masking_a_c, 1, 0 );
+                                      i_n_blocking, 0, 1, 0 );
 
         for ( l_n = 0; l_n < i_n_blocking; l_n++ ) {
           /* post increment early */
@@ -307,7 +276,7 @@ void libxsmm_generator_gemm_sse3_microkernel( libxsmm_generated_code*
                                       LIBXSMM_X86_GP_REG_UNDEF, 0,
                                       (i_micro_kernel_config->datatype_size) * (i_micro_kernel_config->vector_length) * l_m,
                                       i_micro_kernel_config->vector_name,
-                                      i_n_blocking + l_m, i_micro_kernel_config->use_masking_a_c, 1, 0 );
+                                      i_n_blocking + l_m, 0, 1, 0 );
       }
       for ( l_m = 0; l_m < l_m_blocking; l_m++ ) {
         for ( l_n = 0; l_n < i_n_blocking; l_n++ ) {
