@@ -1,30 +1,10 @@
 /******************************************************************************
-** Copyright (c) 2017-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Alexander Heinecke, Greg Henry (Intel Corp.)
 ******************************************************************************/
@@ -33,20 +13,8 @@
 
 #include "generator_x86_instructions.h"
 #include "generator_common.h"
-
 #include <libxsmm_intrinsics_x86.h>
 
-#if defined(LIBXSMM_OFFLOAD_TARGET)
-# pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
-#endif
-#include <string.h>
-#include <stdlib.h>
-#include <assert.h>
-#include <stdio.h>
-#if defined(LIBXSMM_OFFLOAD_TARGET)
-# pragma offload_attribute(pop)
-#endif
-
 
 LIBXSMM_API_INLINE void compact_load_parameter_ (
      libxsmm_generated_code* io_code,
@@ -77,12 +45,12 @@ LIBXSMM_API_INLINE void compact_load_parameter_ (
 
      if ( datasize == 8 ) {
         double vector[16];
-        for ( i = 0 ; i < (int)number ; i++ ) vector[i]=alpha;
+        for ( i = 0; i < (int)number; i++ ) vector[i]=alpha;
 
         libxsmm_x86_instruction_full_vec_load_of_constants ( io_code, (unsigned char*) vector, "loadconst", regset, reg );
      } else {
         float vector[16];
-        for ( i = 0 ; i < (int)number ; i++ ) vector[i]=(float)alpha;
+        for ( i = 0; i < (int)number; i++ ) vector[i]=(float)alpha;
 
         libxsmm_x86_instruction_full_vec_load_of_constants ( io_code, (unsigned char*) vector, "loadconst", regset, reg );
      }
@@ -129,7 +97,7 @@ LIBXSMM_API_INLINE void compact_set_one_ (
         fprintf(stderr,"loading too large a parameter for compact_set_one_\n");
         exit(-1);
      }
-     for ( i = 0 ; i < (int)number ; i++ ) { dvector[i]=1.0; svector[i]=1.0; }
+     for ( i = 0; i < (int)number; i++ ) { dvector[i]=1.0; svector[i]=1.0; }
 
      if ( datasize == 4 )
         libxsmm_x86_instruction_full_vec_load_of_constants ( io_code, (unsigned char*) svector, "loadone", regset, reg );
@@ -470,12 +438,12 @@ LIBXSMM_API_INLINE void compact_mult_two_nums_ (
      unsigned int number,
      char regset )
 {
-     int datasize = 0;
      unsigned int i_vmove_instr;
      int i_instruction_set;
-
+#if 0
+     int datasize = 0;
      LIBXSMM_UNUSED(datasize);
-
+#endif
      if ( regset == 'z' )
      {
         i_instruction_set = LIBXSMM_X86_AVX512;
@@ -488,19 +456,27 @@ LIBXSMM_API_INLINE void compact_mult_two_nums_ (
 
      if ( (number==4) && (regset=='y') )
      {
+#if 0
         datasize = 8;
+#endif
         i_vmove_instr = LIBXSMM_X86_INSTR_VMULPD;
      } else if ( (number==8) && (regset=='z') )
      {
+#if 0
         datasize = 8;
+#endif
         i_vmove_instr = LIBXSMM_X86_INSTR_VMULPD;
      } else if ( (number==8) && (regset=='y') )
      {
+#if 0
         datasize = 4;
+#endif
         i_vmove_instr = LIBXSMM_X86_INSTR_VMULPS;
      } else if ( (number==16) && (regset=='z') )
      {
+#if 0
         datasize = 4;
+#endif
         i_vmove_instr = LIBXSMM_X86_INSTR_VMULPS;
      } else {
         fprintf(stderr,"Unsupported combo of number and regset in compact_mult_two_nums\n");
@@ -518,12 +494,12 @@ LIBXSMM_API_INLINE void compact_add_two_nums_ (
      unsigned int number,
      char regset )
 {
-     int datasize = 0;
      unsigned int i_vmove_instr;
      int i_instruction_set;
-
+#if 0
+     int datasize = 0;
      LIBXSMM_UNUSED(datasize);
-
+#endif
      if ( regset == 'z' )
      {
         i_instruction_set = LIBXSMM_X86_AVX512;
@@ -536,19 +512,27 @@ LIBXSMM_API_INLINE void compact_add_two_nums_ (
 
      if ( (number==4) && (regset=='y') )
      {
+#if 0
         datasize = 8;
+#endif
         i_vmove_instr = LIBXSMM_X86_INSTR_VADDPD;
      } else if ( (number==8) && (regset=='z') )
      {
+#if 0
         datasize = 8;
+#endif
         i_vmove_instr = LIBXSMM_X86_INSTR_VADDPD;
      } else if ( (number==8) && (regset=='y') )
      {
+#if 0
         datasize = 4;
+#endif
         i_vmove_instr = LIBXSMM_X86_INSTR_VADDPS;
      } else if ( (number==16) && (regset=='z') )
      {
+#if 0
         datasize = 4;
+#endif
         i_vmove_instr = LIBXSMM_X86_INSTR_VADDPS;
      } else {
         fprintf(stderr,"Unsupported combo of number and regset in compact_mult_two_nums\n");
@@ -566,12 +550,12 @@ LIBXSMM_API_INLINE void compact_sub_two_nums_ (
      unsigned int number,
      char regset )
 {
-     int datasize = 0;
      unsigned int i_vmove_instr;
      int i_instruction_set;
-
+#if 0
+     int datasize = 0;
      LIBXSMM_UNUSED(datasize);
-
+#endif
      if ( regset == 'z' )
      {
         i_instruction_set = LIBXSMM_X86_AVX512;
@@ -584,19 +568,27 @@ LIBXSMM_API_INLINE void compact_sub_two_nums_ (
 
      if ( (number==4) && (regset=='y') )
      {
+#if 0
         datasize = 8;
+#endif
         i_vmove_instr = LIBXSMM_X86_INSTR_VSUBPD;
      } else if ( (number==8) && (regset=='z') )
      {
+#if 0
         datasize = 8;
+#endif
         i_vmove_instr = LIBXSMM_X86_INSTR_VSUBPD;
      } else if ( (number==8) && (regset=='y') )
      {
+#if 0
         datasize = 4;
+#endif
         i_vmove_instr = LIBXSMM_X86_INSTR_VSUBPS;
      } else if ( (number==16) && (regset=='z') )
      {
+#if 0
         datasize = 4;
+#endif
         i_vmove_instr = LIBXSMM_X86_INSTR_VSUBPS;
      } else {
         fprintf(stderr,"Unsupported combo of number and regset in compact_mult_two_nums\n");
@@ -614,12 +606,12 @@ LIBXSMM_API_INLINE void compact_fms_cminusab_(
      unsigned int number,
      char regset )
 {
-     int datasize = 0;
      unsigned int i_vmove_instr;
      int i_instruction_set;
-
+#if 0
+     int datasize = 0;
      LIBXSMM_UNUSED(datasize);
-
+#endif
      if ( regset == 'z' )
      {
         i_instruction_set = LIBXSMM_X86_AVX512;
@@ -632,19 +624,27 @@ LIBXSMM_API_INLINE void compact_fms_cminusab_(
 
      if ( (number==4) && (regset=='y') )
      {
+#if 0
         datasize = 8;
+#endif
         i_vmove_instr = LIBXSMM_X86_INSTR_VFNMADD231PD;
      } else if ( (number==8) && (regset=='z') )
      {
+#if 0
         datasize = 8;
+#endif
         i_vmove_instr = LIBXSMM_X86_INSTR_VFNMADD231PD;
      } else if ( (number==8) && (regset=='y') )
      {
+#if 0
         datasize = 4;
+#endif
         i_vmove_instr = LIBXSMM_X86_INSTR_VFNMADD231PS;
      } else if ( (number==16) && (regset=='z') )
      {
+#if 0
         datasize = 4;
+#endif
         i_vmove_instr = LIBXSMM_X86_INSTR_VFNMADD231PS;
      } else {
         fprintf(stderr,"Unsupported combo of number and regset in compact_fms_cminusab\n");
@@ -662,12 +662,12 @@ LIBXSMM_API_INLINE void compact_fma_cplusab_(
      unsigned int number,
      char regset )
 {
-     int datasize = 0;
      unsigned int i_vmove_instr;
      int i_instruction_set;
-
+#if 0
+     int datasize = 0;
      LIBXSMM_UNUSED(datasize);
-
+#endif
      if ( regset == 'z' )
      {
         i_instruction_set = LIBXSMM_X86_AVX512;
@@ -680,19 +680,27 @@ LIBXSMM_API_INLINE void compact_fma_cplusab_(
 
      if ( (number==4) && (regset=='y') )
      {
+#if 0
         datasize = 8;
+#endif
         i_vmove_instr = LIBXSMM_X86_INSTR_VFMADD231PD;
      } else if ( (number==8) && (regset=='z') )
      {
+#if 0
         datasize = 8;
+#endif
         i_vmove_instr = LIBXSMM_X86_INSTR_VFMADD231PD;
      } else if ( (number==8) && (regset=='y') )
      {
+#if 0
         datasize = 4;
+#endif
         i_vmove_instr = LIBXSMM_X86_INSTR_VFMADD231PS;
      } else if ( (number==16) && (regset=='z') )
      {
+#if 0
         datasize = 4;
+#endif
         i_vmove_instr = LIBXSMM_X86_INSTR_VFMADD231PS;
      } else {
         fprintf(stderr,"Unsupported combo of number and regset in compact_fms_cminusab\n");
@@ -710,12 +718,12 @@ LIBXSMM_API_INLINE void compact_divide_two_nums_ (
      unsigned int number,
      char regset )
 {
-     int datasize = 0;
      unsigned int i_vmove_instr;
      int i_instruction_set;
-
+#if 0
+     int datasize = 0;
      LIBXSMM_UNUSED(datasize);
-
+#endif
      if ( regset == 'z' )
      {
         i_instruction_set = LIBXSMM_X86_AVX512;
@@ -728,19 +736,27 @@ LIBXSMM_API_INLINE void compact_divide_two_nums_ (
 
      if ( (number==4) && (regset=='y') )
      {
+#if 0
         datasize = 8;
+#endif
         i_vmove_instr = LIBXSMM_X86_INSTR_VDIVPD;
      } else if ( (number==8) && (regset=='z') )
      {
+#if 0
         datasize = 8;
+#endif
         i_vmove_instr = LIBXSMM_X86_INSTR_VDIVPD;
      } else if ( (number==8) && (regset=='y') )
      {
+#if 0
         datasize = 4;
+#endif
         i_vmove_instr = LIBXSMM_X86_INSTR_VDIVPS;
      } else if ( (number==16) && (regset=='z') )
      {
+#if 0
         datasize = 4;
+#endif
         i_vmove_instr = LIBXSMM_X86_INSTR_VDIVPS;
      } else {
         fprintf(stderr,"Unsupported combo of number and regset in compact_divide_two_nums\n");
