@@ -1,30 +1,10 @@
 /******************************************************************************
-** Copyright (c) 2017-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Alexander Heinecke, Greg Henry, Hans Pabst, Timothy Costa (Intel Corp.)
 ******************************************************************************/
@@ -34,19 +14,6 @@
 #include "generator_common.h"
 #include "libxsmm_main.h"
 
-#include <libxsmm_intrinsics_x86.h>
-
-#if defined(LIBXSMM_OFFLOAD_TARGET)
-# pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
-#endif
-#include <string.h>
-#include <stdlib.h>
-#include <assert.h>
-#include <stdio.h>
-#if defined(LIBXSMM_OFFLOAD_TARGET)
-# pragma offload_attribute(pop)
-#endif
-
 #if 0
 # define GENERATOR_PACKED_TRMM_DEBUG
 #endif
@@ -199,7 +166,7 @@ printf("Inside libxsmm_generator_packed_trmm_avx_avx512_kernel: %c%c%c%c m=%d n=
         i = io_code->code_size;
         buf[i++] = 0xc3; /* retq */
         io_code->code_size = i;
-        return ;
+        return;
      }
      if ( LIBXSMM_NEQ(1, alpha) )
      {
@@ -216,7 +183,7 @@ printf("Inside libxsmm_generator_packed_trmm_avx_avx512_kernel: %c%c%c%c m=%d n=
               /* Do LUN* cases: B<- alpha*inv(A)*B */
               for ( j = 1; j <= n1; j+=3 )
               {
-                 for ( k = 1 ; k <= m1 ; k++ )
+                 for ( k = 1; k <= m1; k++ )
                  {
                     compact_load_matrix_gen_ ( io_code, 0, ldb, k, j, 0, numb, datasz, regset, breg );
                     if ( j+1 <= n1 ) compact_load_matrix_gen_ ( io_code, 0, ldb, k, j+1, 4, numb, datasz, regset, breg );
@@ -226,7 +193,7 @@ printf("Inside libxsmm_generator_packed_trmm_avx_avx512_kernel: %c%c%c%c m=%d n=
                        if ( j+1 <= n1 ) compact_mult_two_nums_ ( io_code, 4, 2, 4, numb, regset );
                        if ( j+2 <= n1 ) compact_mult_two_nums_ ( io_code, 7, 2, 7, numb, regset );
                     }
-                    for ( i = 1 ; i <= k-1 ; i++ )
+                    for ( i = 1; i <= k-1; i++ )
                     {
                        compact_load_matrix_gen_ ( io_code, 0, ldb, i, j, 3, numb, datasz, regset, breg );
                        compact_load_matrix_gen_ ( io_code, 0, lda, i, k, 1, numb, datasz, regset, areg );
@@ -266,7 +233,7 @@ printf("Inside libxsmm_generator_packed_trmm_avx_avx512_kernel: %c%c%c%c m=%d n=
               /* Do LLN* cases: B <- alpha * inv(A)*B */
               for ( j = 1; j <= n1; j+=3 )
               {
-                 for ( k = m1; k >= 1 ; k-- )
+                 for ( k = m1; k >= 1; k-- )
                  {
                     compact_load_matrix_gen_ ( io_code, 0, ldb, k, j, 0, numb, datasz, regset, breg );
                     if ( j+1<=n1 ) compact_load_matrix_gen_ ( io_code, 0, ldb, k, j+1, 4, numb, datasz, regset, breg );
@@ -296,7 +263,7 @@ printf("Inside libxsmm_generator_packed_trmm_avx_avx512_kernel: %c%c%c%c m=%d n=
                           compact_store_matrix_gen_ ( io_code, 0, ldb, k, j+2, 8, numb, datasz, regset, breg );
                        }
                     }
-                    for ( i = k+1 ; i <= m1 ; i++ ) {
+                    for ( i = k+1; i <= m1; i++ ) {
                        compact_load_matrix_gen_ ( io_code, 0, ldb, i, j, 3, numb, datasz, regset, breg );
                        compact_load_matrix_gen_ ( io_code, 0, lda, i, k, 1, numb, datasz, regset, areg );
                        compact_fma_cplusab_ ( io_code, 3, 0, 1, numb, regset );
@@ -321,7 +288,7 @@ printf("Inside libxsmm_generator_packed_trmm_avx_avx512_kernel: %c%c%c%c m=%d n=
               /* Do LUT* cases: B<- alpha*A^T*B */
               for ( j = 1; j <= n1; j+=3 )
               {
-                 for ( i = m1 ; i >= 1 ; i-- ) {
+                 for ( i = m1; i >= 1; i-- ) {
                     compact_load_matrix_gen_ ( io_code, 0, ldb, i, j, 0, numb, datasz, regset, breg );
                     if ( j+1 <= n1 ) compact_load_matrix_gen_ ( io_code, 0, ldb, i, j+1, 4, numb, datasz, regset, breg );
                     if ( j+2 <= n1 ) compact_load_matrix_gen_ ( io_code, 0, ldb, i, j+2, 7, numb, datasz, regset, breg );
@@ -358,7 +325,7 @@ printf("Inside libxsmm_generator_packed_trmm_avx_avx512_kernel: %c%c%c%c m=%d n=
               /* Do LLT* cases: B <- alpha * A*B */
               for ( j = 1; j <= n1; j+=3 )
               {
-                 for ( i = 1 ; i <= m1 ; i++ ) {
+                 for ( i = 1; i <= m1; i++ ) {
                     compact_load_matrix_gen_ ( io_code, 0, ldb, i, j, 0, numb, datasz, regset, breg );
                     if ( j+1 <= n1 ) compact_load_matrix_gen_ ( io_code, 0, ldb, i, j+1, 4, numb, datasz, regset, breg );
                     if ( j+2 <= n1 ) compact_load_matrix_gen_ ( io_code, 0, ldb, i, j+2, 7, numb, datasz, regset, breg );
@@ -399,7 +366,7 @@ printf("Inside libxsmm_generator_packed_trmm_avx_avx512_kernel: %c%c%c%c m=%d n=
            if ( (uplo=='U') || (uplo=='u') )
            {
               /* Do RUN* cases: B<- alpha*B*A */
-              for ( j = n1 ; j >= 1 ; j-- ) {
+              for ( j = n1; j >= 1; j-- ) {
                  if ( nounit ) {
                     compact_load_matrix_gen_ ( io_code, 0, lda, j, j, 1, numb, datasz, regset, areg );
                  }
@@ -414,18 +381,18 @@ printf("Inside libxsmm_generator_packed_trmm_avx_avx512_kernel: %c%c%c%c m=%d n=
 #endif
                  }
                  if ( LIBXSMM_NEQ(1, alpha) || nounit ) {
-                    for ( i = 1 ; i <= m1 ; i++ ) {
+                    for ( i = 1; i <= m1; i++ ) {
                        compact_load_matrix_gen_ ( io_code, 0, ldb, i, j, 0, numb, datasz, regset, breg );
                        compact_mult_two_nums_ ( io_code, 0, 1, 0, numb, regset );
                        compact_store_matrix_gen_ ( io_code, 0, ldb, i, j, 0, numb, datasz, regset, breg );
                     }
                  }
-                 for ( k = 1 ; k <= j - 1 ; k++ ) {
+                 for ( k = 1; k <= j - 1; k++ ) {
                     compact_load_matrix_gen_ ( io_code, 0, lda, k, j, 1, numb, datasz, regset, areg );
                     if ( LIBXSMM_NEQ(1, alpha) ) {
                        compact_mult_two_nums_ ( io_code, 1, 2, 1, numb, regset );
                     }
-                    for ( i = 1; i <= m1 ; i++ ) {
+                    for ( i = 1; i <= m1; i++ ) {
                        compact_load_matrix_gen_ ( io_code, 0, ldb, i, j, 0, numb, datasz, regset, breg );
                        compact_load_matrix_gen_ ( io_code, 0, ldb, i, k, 3, numb, datasz, regset, breg );
                        compact_fma_cplusab_ ( io_code, 0, 1, 3, numb, regset );
@@ -435,7 +402,7 @@ printf("Inside libxsmm_generator_packed_trmm_avx_avx512_kernel: %c%c%c%c m=%d n=
               }
            } else {
               /* Do RLN* cases: B <- alpha * B * A */
-              for ( j = 1 ; j <= n1; j++ )
+              for ( j = 1; j <= n1; j++ )
               {
                  if ( nounit ) {
                     compact_load_matrix_gen_ ( io_code, 0, lda, j, j, 1, numb, datasz, regset, areg );
@@ -451,18 +418,18 @@ printf("Inside libxsmm_generator_packed_trmm_avx_avx512_kernel: %c%c%c%c m=%d n=
 #endif
                  }
                  if ( LIBXSMM_NEQ(1, alpha) || nounit ) {
-                    for ( i = 1 ; i <= m1 ; i++ ) {
+                    for ( i = 1; i <= m1; i++ ) {
                        compact_load_matrix_gen_ ( io_code, 0, ldb, i, j, 0, numb, datasz, regset, breg );
                        compact_mult_two_nums_ ( io_code, 0, 1, 0, numb, regset );
                        compact_store_matrix_gen_ ( io_code, 0, ldb, i, j, 0, numb, datasz, regset, breg );
                     }
                  }
-                 for ( k = j+1 ; k <= n1 ; k++ ) {
+                 for ( k = j+1; k <= n1; k++ ) {
                     compact_load_matrix_gen_ ( io_code, 0, lda, k, j, 1, numb, datasz, regset, areg );
                     if ( LIBXSMM_NEQ(1, alpha) ) {
                        compact_mult_two_nums_ ( io_code, 1, 2, 1, numb, regset );
                     }
-                    for ( i = 1; i <= m1 ; i++ ) {
+                    for ( i = 1; i <= m1; i++ ) {
                        compact_load_matrix_gen_ ( io_code, 0, ldb, i, j, 0, numb, datasz, regset, breg );
                        compact_load_matrix_gen_ ( io_code, 0, ldb, i, k, 3, numb, datasz, regset, breg );
                        compact_fma_cplusab_ ( io_code, 0, 1, 3, numb, regset );
@@ -475,15 +442,15 @@ printf("Inside libxsmm_generator_packed_trmm_avx_avx512_kernel: %c%c%c%c m=%d n=
            if ( (uplo=='U') || (uplo=='u') )
            {
               /* Do RUT* cases: B<- alpha*B *A^T */
-              for ( k = 1 ; k <= n1 ; k++ )
+              for ( k = 1; k <= n1; k++ )
               {
-                 for ( j = 1 ; j <= k-1 ; j++ )
+                 for ( j = 1; j <= k-1; j++ )
                  {
                     compact_load_matrix_gen_ ( io_code, 0, lda, j, k, 1, numb, datasz, regset, areg );
                     if ( LIBXSMM_NEQ(1, alpha) ) {
                        compact_mult_two_nums_ ( io_code, 1, 2, 1, numb, regset );
                     }
-                    for ( i = 1 ; i <= m1 ; i++ ) {
+                    for ( i = 1; i <= m1; i++ ) {
                        compact_load_matrix_gen_ ( io_code, 0, ldb, i, j, 0, numb, datasz, regset, breg );
                        compact_load_matrix_gen_ ( io_code, 0, ldb, i, k, 3, numb, datasz, regset, breg );
                        compact_fma_cplusab_ ( io_code, 0, 1, 3, numb, regset );
@@ -504,7 +471,7 @@ printf("Inside libxsmm_generator_packed_trmm_avx_avx512_kernel: %c%c%c%c m=%d n=
 #endif
                  }
                  if ( LIBXSMM_NEQ(1, alpha) || nounit ) {
-                    for ( i = 1 ; i <= m1 ; i++ ) {
+                    for ( i = 1; i <= m1; i++ ) {
                        compact_load_matrix_gen_ ( io_code, 0, ldb, i, k, 0, numb, datasz, regset, breg );
                        compact_mult_two_nums_ ( io_code, 0, 1, 0, numb, regset );
                        compact_store_matrix_gen_ ( io_code, 0, ldb, i, k, 0, numb, datasz, regset, breg );
@@ -513,15 +480,15 @@ printf("Inside libxsmm_generator_packed_trmm_avx_avx512_kernel: %c%c%c%c m=%d n=
               }
            } else {
               /* Do RLT* cases: B <- alpha * B *inv(A^T) */
-              for ( k = n1 ; k >= 1 ; k-- )
+              for ( k = n1; k >= 1; k-- )
               {
-                 for ( j = k+1 ; j <= n1 ; j++ )
+                 for ( j = k+1; j <= n1; j++ )
                  {
                     compact_load_matrix_gen_ ( io_code, 0, lda, j, k, 1, numb, datasz, regset, areg );
                     if ( LIBXSMM_NEQ(1, alpha) ) {
                        compact_mult_two_nums_ ( io_code, 1, 2, 1, numb, regset );
                     }
-                    for ( i = 1 ; i <= m1 ; i++ ) {
+                    for ( i = 1; i <= m1; i++ ) {
                        compact_load_matrix_gen_ ( io_code, 0, ldb, i, j, 0, numb, datasz, regset, breg );
                        compact_load_matrix_gen_ ( io_code, 0, ldb, i, k, 3, numb, datasz, regset, breg );
                        compact_fma_cplusab_ ( io_code, 0, 1, 3, numb, regset );
@@ -542,7 +509,7 @@ printf("Inside libxsmm_generator_packed_trmm_avx_avx512_kernel: %c%c%c%c m=%d n=
 #endif
                  }
                  if ( LIBXSMM_NEQ(1, alpha) || nounit ) {
-                    for ( i = 1 ; i <= m1 ; i++ ) {
+                    for ( i = 1; i <= m1; i++ ) {
                        compact_load_matrix_gen_ ( io_code, 0, ldb, i, k, 0, numb, datasz, regset, breg );
                        compact_mult_two_nums_ ( io_code, 0, 1, 0, numb, regset );
                        compact_store_matrix_gen_ ( io_code, 0, ldb, i, k, 0, numb, datasz, regset, breg );
@@ -567,13 +534,12 @@ printf("Inside libxsmm_generator_packed_trmm_avx_avx512_kernel: %c%c%c%c m=%d n=
 #define DEBUG_GIVE_BYTE_CODE_OUTPUT
 #endif
 #ifdef DEBUG_GIVE_BYTE_CODE_OUTPUT
-   buf = (unsigned char *) io_code->generated_code;
-   printf("#Final Routine: \n");
-   for ( i = 0 ; i < io_code->code_size ; i+=8 )
-   {
-      printf("#\tBytes %d-%d\n",i,i+7);
-      printf(".byte 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x\n",buf[i],buf[i+1],buf[i+2],buf[i+3],buf[i+4],buf[i+5],buf[i+6],buf[i+7]);
-   }
+  buf = (unsigned char *) io_code->generated_code;
+  printf("#Final Routine: \n");
+  for ( i = 0; i < io_code->code_size; i+=8 ) {
+    printf("#\tBytes %d-%d\n",i,i+7);
+    printf(".byte 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x\n",buf[i],buf[i+1],buf[i+2],buf[i+3],buf[i+4],buf[i+5],buf[i+6],buf[i+7]);
+  }
 #endif
 
 }
