@@ -56,8 +56,8 @@ void libxsmm_sparse_csc_reader( libxsmm_generated_code* io_generated_code,
                                 unsigned int**          o_row_idx,
                                 unsigned int**          o_column_idx,
                                 double**                o_values,
-                                unsigned int*           o_row_count,
-                                unsigned int*           o_column_count,
+                                unsigned int*           io_row_count,
+                                unsigned int*           io_column_count,
                                 unsigned int*           o_element_count ) {
   FILE *l_csc_file_handle;
   const unsigned int l_line_length = 512;
@@ -75,7 +75,8 @@ void libxsmm_sparse_csc_reader( libxsmm_generated_code* io_generated_code,
   while (fgets(l_line, l_line_length, l_csc_file_handle) != NULL) {
     if ( strlen(l_line) == l_line_length ) {
       free(*o_row_idx); free(*o_column_idx); free(*o_values); free(l_column_idx_id);
-      *o_row_idx = 0; *o_column_idx = 0; *o_values = 0;
+      *io_row_count = *io_column_count = *o_element_count = 0;
+      *o_row_idx = *o_column_idx = NULL; *o_values = NULL;
       fclose( l_csc_file_handle ); /* close mtx file */
       LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_CSC_READ_LEN );
       return;
@@ -86,18 +87,21 @@ void libxsmm_sparse_csc_reader( libxsmm_generated_code* io_generated_code,
     } else {
       /* if we are the first line after comment header, we allocate our data structures */
       if ( l_header_read == 0 ) {
-        if (3 == sscanf(l_line, "%u %u %u", o_row_count, o_column_count, o_element_count) &&
-            0 != *o_row_count && 0 != *o_column_count && 0 != *o_element_count)
+        unsigned int row_count, column_count;
+        if (3 == sscanf(l_line, "%u %u %u", &row_count, &column_count, o_element_count) &&
+            0 != row_count && 0 != column_count && 0 != *o_element_count)
         {
-          /* allocate CSC data structure matching mtx file */
+          *io_column_count = LIBXSMM_MAX(*io_column_count, column_count);
+          *io_row_count = LIBXSMM_MAX(*io_row_count, row_count);
+          /* allocate CSC data structure matching mtx file, and set everything to zero for init */
           /* coverity[tainted_data] */
-          *o_row_idx = (unsigned int*) malloc(sizeof(unsigned int) * (*o_element_count));
+          *o_row_idx = (unsigned int*)calloc(*o_element_count, sizeof(unsigned int));
           /* coverity[tainted_data] */
-          *o_column_idx = (unsigned int*) malloc(sizeof(unsigned int) * ((size_t)(*o_column_count) + 1));
+          *o_column_idx = (unsigned int*)calloc((size_t)*io_column_count + 1, sizeof(unsigned int));
           /* coverity[tainted_data] */
-          *o_values = (double*) malloc(sizeof(double) * (*o_element_count));
+          *o_values = (double*)calloc(*o_element_count, sizeof(double));
           /* coverity[tainted_data] */
-          l_column_idx_id = (unsigned int*) malloc(sizeof(unsigned int) * (*o_column_count));
+          l_column_idx_id = (unsigned int*)calloc(*io_column_count, sizeof(unsigned int));
 
           /* check if mallocs were successful */
           if ( ( *o_row_idx == NULL )      ||
@@ -105,25 +109,16 @@ void libxsmm_sparse_csc_reader( libxsmm_generated_code* io_generated_code,
                ( *o_values == NULL )       ||
                ( l_column_idx_id == NULL ) ) {
             free(*o_row_idx); free(*o_column_idx); free(*o_values); free(l_column_idx_id);
-            *o_row_idx = 0; *o_column_idx = 0; *o_values = 0;
+            *io_row_count = *io_column_count = *o_element_count = 0;
+            *o_row_idx = *o_column_idx = NULL; *o_values = NULL;
             fclose(l_csc_file_handle); /* close mtx file */
             LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_CSC_ALLOC_DATA );
             return;
           }
 
-          /* set everything to zero for init */
-          /* coverity[tainted_data] */
-          memset(*o_row_idx, 0, sizeof(unsigned int) * (*o_element_count));
-          /* coverity[tainted_data] */
-          memset(*o_column_idx, 0, sizeof(unsigned int) * ((size_t)(*o_column_count) + 1));
-          /* coverity[tainted_data] */
-          memset(*o_values, 0, sizeof(double) * (*o_element_count));
-          /* coverity[tainted_data] */
-          memset(l_column_idx_id, 0, sizeof(unsigned int) * (*o_column_count));
-
           /* init column idx */
           /* coverity[tainted_data] */
-          for (l_i = 0; l_i <= *o_column_count; ++l_i) {
+          for (l_i = 0; l_i <= *io_column_count; ++l_i) {
             (*o_column_idx)[l_i] = *o_element_count;
           }
           /* init */
@@ -140,9 +135,13 @@ void libxsmm_sparse_csc_reader( libxsmm_generated_code* io_generated_code,
         unsigned int l_row = 0, l_column = 0;
         double l_value = 0;
         /* read a line of content */
-        if ( sscanf(l_line, "%u %u %lf", &l_row, &l_column, &l_value) != 3 ) {
+        if ( sscanf(l_line, "%u %u %lf", &l_row, &l_column, &l_value) != 3
+          || l_row > *io_row_count || l_column > *io_column_count
+          || l_i >= *o_element_count )
+        {
           free(*o_row_idx); free(*o_column_idx); free(*o_values); free(l_column_idx_id);
-          *o_row_idx = 0; *o_column_idx = 0; *o_values = 0;
+          *io_row_count = *io_column_count = *o_element_count = 0;
+          *o_row_idx = *o_column_idx = NULL; *o_values = NULL;
           fclose(l_csc_file_handle); /* close mtx file */
           LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_CSC_READ_ELEMS );
           return;
@@ -168,14 +167,15 @@ void libxsmm_sparse_csc_reader( libxsmm_generated_code* io_generated_code,
   /* check if we read a file which was consistent */
   if ( l_i != (*o_element_count) ) {
     free(*o_row_idx); free(*o_column_idx); free(*o_values); free(l_column_idx_id);
-    *o_row_idx = 0; *o_column_idx = 0; *o_values = 0;
+    *io_row_count = *io_column_count = *o_element_count = 0;
+    *o_row_idx = *o_column_idx = NULL; *o_values = NULL;
     LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_CSC_LEN );
     return;
   }
 
   if ( l_column_idx_id != NULL ) {
     /* let's handle empty columns */
-    for ( l_i = 0; l_i < (*o_column_count); l_i++) {
+    for ( l_i = 0; l_i < (*io_column_count); l_i++) {
       if ( l_column_idx_id[l_i] == 0 ) {
         (*o_column_idx)[l_i+1] = (*o_column_idx)[l_i];
       }
