@@ -57,6 +57,7 @@ void libxsmm_generator_spgemm_csr_asparse_reg( libxsmm_generated_code*         i
   unsigned int l_z;
   unsigned int l_row_elements;
   unsigned int l_unique;
+  unsigned int l_reg_num;
   unsigned int l_hit;
   unsigned int l_n_blocking = 1;
   unsigned int l_n_row_idx = i_row_idx[i_xgemm_desc->m];
@@ -64,6 +65,7 @@ void libxsmm_generator_spgemm_csr_asparse_reg( libxsmm_generated_code*         i
   unsigned int *const l_unique_pos = (unsigned int*)(0 != l_n_row_idx ? malloc(sizeof(unsigned int) * l_n_row_idx) : NULL);
   double l_code_const_dp[8];
   float l_code_const_fp[16];
+  unsigned int l_const_perm_ops[16];
 
   libxsmm_micro_kernel_config l_micro_kernel_config;
   libxsmm_loop_label_tracker l_loop_label_tracker;
@@ -125,8 +127,9 @@ void libxsmm_generator_spgemm_csr_asparse_reg( libxsmm_generated_code*         i
     }
   }
 
-  /* check that we have enough registers (N=20) for now */
-  if ( l_unique > 31 ) {
+  /* check that we have enough registers for the datatype */
+  if ( (LIBXSMM_GEMM_PRECISION_F64 == LIBXSMM_GETENUM_INP( i_xgemm_desc->datatype ) && l_unique > LIBXSMM_SPGEMM_ASPARSE_REG_MAX_UNIQUE_L1_DP) ||
+       (LIBXSMM_GEMM_PRECISION_F32 == LIBXSMM_GETENUM_INP( i_xgemm_desc->datatype ) && l_unique > LIBXSMM_SPGEMM_ASPARSE_REG_MAX_UNIQUE_L1_SP) ) {
     free(l_unique_values); free(l_unique_pos);
     LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_UNIQUE_VAL );
     return;
@@ -176,28 +179,97 @@ void libxsmm_generator_spgemm_csr_asparse_reg( libxsmm_generated_code*         i
   /* open asm */
   libxsmm_x86_instruction_open_stream( io_generated_code, &l_gp_reg_mapping, i_xgemm_desc->prefetch );
 
-  /* load A into registers */
-  for ( l_z = 0; l_z < l_unique; l_z++) {
-    char l_id[65];
-    LIBXSMM_SNPRINTF(l_id, 64, "%u", l_z);
-    if ( LIBXSMM_GEMM_PRECISION_F64 == LIBXSMM_GETENUM_INP( i_xgemm_desc->datatype )  ) {
-      for ( l_m = 0; l_m < 8; l_m++) {
-        l_code_const_dp[l_m] = l_unique_values[l_z];
+  /*
+   * load A into registers
+   * pre-broadcast if possible, otherwise load for run-time broadcasting
+   */
+  if (l_unique <= 31 ) {
+    /* pre-broadcast A values into registers */
+    for ( l_z = 0; l_z < l_unique; l_z++) {
+      char l_id[65];
+      LIBXSMM_SNPRINTF(l_id, 64, "%u", l_z);
+      if ( LIBXSMM_GEMM_PRECISION_F64 == LIBXSMM_GETENUM_INP( i_xgemm_desc->datatype )  ) {
+        for ( l_m = 0; l_m < 8; l_m++) {
+          l_code_const_dp[l_m] = l_unique_values[l_z];
+        }
+        libxsmm_x86_instruction_full_vec_load_of_constants ( io_generated_code,
+                                                            (unsigned char*)l_code_const_dp,
+                                                            l_id,
+                                                            l_micro_kernel_config.vector_name,
+                                                            l_z );
+      } else {
+        for ( l_m = 0; l_m < 16; l_m++) {
+          l_code_const_fp[l_m] = (float)l_unique_values[l_z];
+        }
+        libxsmm_x86_instruction_full_vec_load_of_constants ( io_generated_code,
+                                                            (unsigned char*)l_code_const_fp,
+                                                            l_id,
+                                                            l_micro_kernel_config.vector_name,
+                                                            l_z );
       }
-      libxsmm_x86_instruction_full_vec_load_of_constants ( io_generated_code,
-                                                           (unsigned char*)l_code_const_dp,
-                                                           l_id,
-                                                           l_micro_kernel_config.vector_name,
-                                                           l_z );
-    } else {
-      for ( l_m = 0; l_m < 16; l_m++) {
-        l_code_const_fp[l_m] = (float)l_unique_values[l_z];
+    }
+  } else {
+    /* load packed A into registers */
+    l_z = 0;
+    l_reg_num = 0;
+    while (l_z < l_unique) {
+      char l_id[65];
+      LIBXSMM_SNPRINTF(l_id, 64, "%u", l_reg_num);
+      l_m = 0;
+
+      if ( LIBXSMM_GEMM_PRECISION_F64 == LIBXSMM_GETENUM_INP( i_xgemm_desc->datatype ) ) {
+        while (l_z < l_unique && l_m < 8) {
+          l_code_const_dp[l_m++] = l_unique_values[l_z++];
+        }
+        libxsmm_x86_instruction_full_vec_load_of_constants( io_generated_code,
+                                                            (unsigned char*)l_code_const_dp,
+                                                            l_id,
+                                                            l_micro_kernel_config.vector_name,
+                                                            l_reg_num++ );
+      } else {
+        while (l_z < l_unique && l_m < 16) {
+          l_code_const_fp[l_m++] = (float)l_unique_values[l_z++];
+        }
+        libxsmm_x86_instruction_full_vec_load_of_constants( io_generated_code,
+                                                            (unsigned char*)l_code_const_fp,
+                                                            l_id,
+                                                            l_micro_kernel_config.vector_name,
+                                                            l_reg_num++ );
+      }
+    }
+
+    /* load permute operands into registers if space is available (otherwise they are read from memory) */
+    if ( LIBXSMM_GEMM_PRECISION_F64 == LIBXSMM_GETENUM_INP( i_xgemm_desc->datatype ) && l_unique <= LIBXSMM_SPGEMM_ASPARSE_REG_MAX_UNIQUE_REG_DP ) {
+      for (l_reg_num=LIBXSMM_SPGEMM_ASPARSE_REG_PERM_FIRST_REG_OP_DP; l_reg_num<LIBXSMM_SPGEMM_ASPARSE_REG_PERM_FIRST_REG_OP_DP+8; l_reg_num++ ) {
+        char l_id[65];
+        LIBXSMM_SNPRINTF(l_id, 64, "%u", l_reg_num);
+        l_m = 0;
+        /* repeat pattern to select 64-bits using vpermd */
+        while (l_m<16) {
+          l_const_perm_ops[l_m++] = (l_reg_num-LIBXSMM_SPGEMM_ASPARSE_REG_PERM_FIRST_REG_OP_DP)*2;
+          l_const_perm_ops[l_m++] = (l_reg_num-LIBXSMM_SPGEMM_ASPARSE_REG_PERM_FIRST_REG_OP_DP)*2 + 1;
+        }
+        libxsmm_x86_instruction_full_vec_load_of_constants( io_generated_code,
+                                                            (unsigned char*)l_const_perm_ops,
+                                                            l_id,
+                                                            l_micro_kernel_config.vector_name,
+                                                            l_reg_num );
+      }
+    } else if ( LIBXSMM_GEMM_PRECISION_F32 == LIBXSMM_GETENUM_INP( i_xgemm_desc->datatype ) && l_unique <= LIBXSMM_SPGEMM_ASPARSE_REG_MAX_UNIQUE_REG_SP ){
+      for (l_reg_num=LIBXSMM_SPGEMM_ASPARSE_REG_PERM_FIRST_REG_OP_SP; l_reg_num<LIBXSMM_SPGEMM_ASPARSE_REG_PERM_FIRST_REG_OP_SP+16; l_reg_num++ ) {
+        char l_id[65];
+        LIBXSMM_SNPRINTF(l_id, 64, "%u", l_reg_num);
+        l_m = 0;
+        /* repeat pattern to select 32-bits using vpermd */
+        while (l_m<16) {
+          l_const_perm_ops[l_m++] = (l_reg_num-LIBXSMM_SPGEMM_ASPARSE_REG_PERM_FIRST_REG_OP_SP);
+        }
+        libxsmm_x86_instruction_full_vec_load_of_constants( io_generated_code,
+                                                            (unsigned char*)l_const_perm_ops,
+                                                            l_id,
+                                                            l_micro_kernel_config.vector_name,
+                                                            l_reg_num );
       }
-      libxsmm_x86_instruction_full_vec_load_of_constants ( io_generated_code,
-                                                           (unsigned char*)l_code_const_fp,
-                                                           l_id,
-                                                           l_micro_kernel_config.vector_name,
-                                                           l_z );
     }
   }
 
@@ -221,15 +293,15 @@ void libxsmm_generator_spgemm_csr_asparse_reg( libxsmm_generated_code*         i
                                             l_m*i_xgemm_desc->ldc*l_micro_kernel_config.datatype_size +
                                               l_n*l_micro_kernel_config.datatype_size*l_micro_kernel_config.vector_length,
                                             l_micro_kernel_config.vector_name,
-                                            l_unique+l_n, 0, 1, 0 );
+                                            LIBXSMM_SPGEMM_ASPARSE_REG_ACC_REG+l_n, 0, 1, 0 );
         } else {
           libxsmm_x86_instruction_vec_compute_reg( io_generated_code,
                                                    l_micro_kernel_config.instruction_set,
                                                    l_micro_kernel_config.vxor_instruction,
                                                    l_micro_kernel_config.vector_name,
-                                                   l_unique+l_n,
-                                                   l_unique+l_n,
-                                                   l_unique+l_n );
+                                                   LIBXSMM_SPGEMM_ASPARSE_REG_ACC_REG+l_n,
+                                                   LIBXSMM_SPGEMM_ASPARSE_REG_ACC_REG+l_n,
+                                                   LIBXSMM_SPGEMM_ASPARSE_REG_ACC_REG+l_n );
         }
 
         /* only prefetch if we do temporal stores */
@@ -245,9 +317,79 @@ void libxsmm_generator_spgemm_csr_asparse_reg( libxsmm_generated_code*         i
     }
     for ( l_z = 0; l_z < l_row_elements; l_z++ ) {
       /* check k such that we just use columns which actually need to be multiplied */
+      const unsigned int u = i_row_idx[l_m] + l_z;
+      unsigned int l_unique_reg;
+      LIBXSMM_ASSERT(u < l_n_row_idx);
+
+      /* broadcast unique element of A if not in pre-broadcast mode */
+      if (l_unique > 31 ) {
+        if ( LIBXSMM_GEMM_PRECISION_F64 == LIBXSMM_GETENUM_INP( i_xgemm_desc->datatype ) ) {
+          /* load permute selector operand if not stored in registers */
+          if (l_unique > LIBXSMM_SPGEMM_ASPARSE_REG_MAX_UNIQUE_REG_DP) {
+            libxsmm_x86_instruction_vec_move( io_generated_code,
+                                              l_micro_kernel_config.instruction_set,
+                                              l_micro_kernel_config.a_vmove_instruction,
+                                              l_gp_reg_mapping.gp_reg_a,
+                                              LIBXSMM_X86_GP_REG_UNDEF, 0,
+                                              (l_unique_pos[u] % 8)*64,
+                                              l_micro_kernel_config.vector_name,
+                                              LIBXSMM_SPGEMM_ASPARSE_REG_BCAST_REG, 0, 1, 0 );
+
+            libxsmm_x86_instruction_vec_compute_reg(io_generated_code,
+                                                    l_micro_kernel_config.instruction_set,
+                                                    LIBXSMM_X86_INSTR_VPERMD,
+                                                    l_micro_kernel_config.vector_name,
+                                                    l_unique_pos[u] / 8,
+                                                    LIBXSMM_SPGEMM_ASPARSE_REG_BCAST_REG,
+                                                    LIBXSMM_SPGEMM_ASPARSE_REG_BCAST_REG);
+
+          /* permute selector operand already in register */
+          } else {
+            libxsmm_x86_instruction_vec_compute_reg(io_generated_code,
+                                                    l_micro_kernel_config.instruction_set,
+                                                    LIBXSMM_X86_INSTR_VPERMD,
+                                                    l_micro_kernel_config.vector_name,
+                                                    l_unique_pos[u] / 8,
+                                                    LIBXSMM_SPGEMM_ASPARSE_REG_PERM_FIRST_REG_OP_DP + l_unique_pos[u] % 8,
+                                                    LIBXSMM_SPGEMM_ASPARSE_REG_BCAST_REG);
+          }
+        } else {
+          /* load permute selector operand if not stored in registers */
+          if (l_unique > LIBXSMM_SPGEMM_ASPARSE_REG_MAX_UNIQUE_REG_SP) {
+            libxsmm_x86_instruction_vec_move( io_generated_code,
+                                              l_micro_kernel_config.instruction_set,
+                                              l_micro_kernel_config.a_vmove_instruction,
+                                              l_gp_reg_mapping.gp_reg_a,
+                                              LIBXSMM_X86_GP_REG_UNDEF, 0,
+                                              (l_unique_pos[u] % 16)*64,
+                                              l_micro_kernel_config.vector_name,
+                                              LIBXSMM_SPGEMM_ASPARSE_REG_BCAST_REG, 0, 1, 0 );
+
+            libxsmm_x86_instruction_vec_compute_reg(io_generated_code,
+                                                    l_micro_kernel_config.instruction_set,
+                                                    LIBXSMM_X86_INSTR_VPERMD,
+                                                    l_micro_kernel_config.vector_name,
+                                                    l_unique_pos[u] / 16,
+                                                    LIBXSMM_SPGEMM_ASPARSE_REG_BCAST_REG,
+                                                    LIBXSMM_SPGEMM_ASPARSE_REG_BCAST_REG);
+
+          /* permute selector operand already in register */
+          } else {
+            libxsmm_x86_instruction_vec_compute_reg(io_generated_code,
+                                                    l_micro_kernel_config.instruction_set,
+                                                    LIBXSMM_X86_INSTR_VPERMD,
+                                                    l_micro_kernel_config.vector_name,
+                                                    l_unique_pos[u] / 16,
+                                                    LIBXSMM_SPGEMM_ASPARSE_REG_PERM_FIRST_REG_OP_SP + l_unique_pos[u] % 16,
+                                                    LIBXSMM_SPGEMM_ASPARSE_REG_BCAST_REG);
+          }
+        }
+      }
+
       for ( l_n = 0; l_n < l_n_blocking; l_n++ ) {
-        const unsigned int u = i_row_idx[l_m] + l_z;
-        LIBXSMM_ASSERT(u < l_n_row_idx);
+        /* select correct register depending on mode */
+        l_unique_reg = l_unique > 31 ? LIBXSMM_SPGEMM_ASPARSE_REG_BCAST_REG : l_unique_pos[u];
+
         libxsmm_x86_instruction_vec_compute_mem( io_generated_code,
                                                  l_micro_kernel_config.instruction_set,
                                                  l_micro_kernel_config.vmul_instruction,
@@ -258,8 +400,8 @@ void libxsmm_generator_spgemm_csr_asparse_reg( libxsmm_generated_code*         i
                                                  i_column_idx[u]*i_xgemm_desc->ldb*l_micro_kernel_config.datatype_size +
                                                    l_n*l_micro_kernel_config.datatype_size*l_micro_kernel_config.vector_length,
                                                  l_micro_kernel_config.vector_name,
-                                                 l_unique_pos[u],
-                                                 l_unique+l_n );
+                                                 l_unique_reg,
+                                                 LIBXSMM_SPGEMM_ASPARSE_REG_ACC_REG+l_n );
 
           libxsmm_x86_instruction_prefetch( io_generated_code,
                                             LIBXSMM_X86_INSTR_PREFETCHT2,
@@ -290,7 +432,7 @@ void libxsmm_generator_spgemm_csr_asparse_reg( libxsmm_generated_code*         i
                                           l_m*i_xgemm_desc->ldc*l_micro_kernel_config.datatype_size +
                                             l_n*l_micro_kernel_config.datatype_size*l_micro_kernel_config.vector_length,
                                           l_micro_kernel_config.vector_name,
-                                          l_unique+l_n, 0, 0, 1 );
+                                          LIBXSMM_SPGEMM_ASPARSE_REG_ACC_REG+l_n, 0, 0, 1 );
       }
     }
   }
@@ -307,4 +449,3 @@ void libxsmm_generator_spgemm_csr_asparse_reg( libxsmm_generated_code*         i
   free(l_unique_values);
   free(l_unique_pos);
 }
-
