@@ -65,9 +65,24 @@ LIBXSMM_API int libxsmm_cpuid_x86(libxsmm_cpuid_x86_info* info)
   unsigned int eax, ebx, ecx, edx;
   LIBXSMM_CPUID_X86(0, 0/*ecx*/, eax, ebx, ecx, edx);
   if (1 <= eax) { /* CPUID max. leaf */
-    if (LIBXSMM_TARGET_ARCH_UNKNOWN == result) { /* detect CPU-feature only once */
-      int feature_cpu = LIBXSMM_X86_GENERIC, feature_os = LIBXSMM_X86_GENERIC;
+    /* avoid redetecting features but redetect on request (info given) */
+    if (LIBXSMM_TARGET_ARCH_UNKNOWN == result || NULL != info) {
+      int feature_cpu = LIBXSMM_X86_GENERIC, feature_os = LIBXSMM_X86_GENERIC, has_context = 0;
       unsigned int maxleaf = eax;
+# if defined(__linux__)
+      if (0 == libxsmm_se) {
+        FILE *const selinux = fopen("/sys/fs/selinux/enforce", "rb");
+        if (NULL != selinux) {
+          if (1 == fread(&libxsmm_se, 1/*sizeof(char)*/, 1/*count*/, selinux)) {
+            libxsmm_se = ('0' != libxsmm_se ? 1 : 0);
+          }
+          else { /* conservative assumption in case of read-error */
+            libxsmm_se = 1;
+          }
+          fclose(selinux);
+        }
+      }
+# endif
       LIBXSMM_CPUID_X86(1, 0/*ecx*/, eax, ebx, ecx, edx);
       /* Check for CRC32 (this is not a proper test for SSE 4.2 as a whole!) */
       if (LIBXSMM_CPUID_CHECK(ecx, 0x00100000)) {
@@ -129,37 +144,53 @@ LIBXSMM_API int libxsmm_cpuid_x86(libxsmm_cpuid_x86_info* info)
           }
         }
       }
+      else if (LIBXSMM_X86_GENERIC <= feature_cpu) {
+        /* assume FXSAVE, which should be fine
+         * 16 years after the first x86_64 OS
+         */
+        feature_os = LIBXSMM_X86_SSE4;
+      }
       else feature_os = LIBXSMM_TARGET_ARCH_GENERIC;
-      if (0 != libxsmm_verbosity) { /* library code is expected to be mute */
+      has_context = (LIBXSMM_STATIC_TARGET_ARCH >= feature_cpu || feature_os >= feature_cpu) ? 1 : 0;
+      if (LIBXSMM_TARGET_ARCH_UNKNOWN == result && 0 != libxsmm_verbosity) { /* library code is expected to be mute */
         const int target_vlen32 = libxsmm_cpuid_vlen32(feature_cpu);
         const char *const compiler_support = (libxsmm_cpuid_vlen32(LIBXSMM_MAX_STATIC_TARGET_ARCH) < target_vlen32
           ? "" : (((2 <= libxsmm_verbosity || 0 > libxsmm_verbosity) && LIBXSMM_MAX_STATIC_TARGET_ARCH < feature_cpu)
             ? "highly " : NULL));
-        int warnings = 0;
 # if !defined(NDEBUG) && defined(__OPTIMIZE__)
         fprintf(stderr, "LIBXSMM WARNING: library is optimized without -DNDEBUG and contains debug code!\n");
-        ++warnings;
 # endif
         if (NULL != compiler_support) {
-          fprintf(stderr, "LIBXSMM WARNING: %soptimized non-JIT code paths are limited to \"%s\"!\n",
-            compiler_support, libxsmm_cpuid_name(LIBXSMM_MAX_STATIC_TARGET_ARCH));
-          ++warnings;
+          const char *const name = libxsmm_cpuid_name( /* exclude MIC when running on Core processors */
+            (((LIBXSMM_X86_AVX512_MIC == LIBXSMM_MAX_STATIC_TARGET_ARCH) ||
+              (LIBXSMM_X86_AVX512_KNM == LIBXSMM_MAX_STATIC_TARGET_ARCH)) && (LIBXSMM_X86_AVX512_CORE <= feature_cpu))
+              ? LIBXSMM_X86_AVX2 : LIBXSMM_MAX_STATIC_TARGET_ARCH);
+          fprintf(stderr, "LIBXSMM WARNING: %soptimized non-JIT code paths are limited to \"%s\"!\n", compiler_support, name);
         }
-        if (LIBXSMM_STATIC_TARGET_ARCH < feature_cpu && feature_os < feature_cpu) {
+# if !defined(__APPLE__) || !defined(__MACH__) /* permitted features */
+        if (0 == has_context) {
           fprintf(stderr, "LIBXSMM WARNING: detected CPU features are not permitted by the OS!\n");
-          ++warnings;
+          if (0 == libxsmm_se) {
+            fprintf(stderr, "LIBXSMM WARNING: downgraded code generation to supported features!\n");
+          }
         }
-        if (0 != warnings) fprintf(stderr, "\n");
-      }
-# if 0 /* permitted features */
-      result = LIBXSMM_MIN(feature_cpu, feature_os);
-# else /* opportunistic */
-      result = feature_cpu;
 # endif
-    }
-    if (NULL != info) {
-      LIBXSMM_CPUID_X86(0x80000007, 0/*ecx*/, eax, ebx, ecx, edx);
-      info->constant_tsc = LIBXSMM_CPUID_CHECK(edx, 0x00000100);
+      }
+      /* macOS is faulting AVX-512 (on-demand larger state) */
+      result = feature_cpu;
+# if !defined(__APPLE__) || !defined(__MACH__)
+#   if 0 /* opportunistic */
+      if (0 == libxsmm_se)
+#   endif
+      { /* only permitted features */
+        result = LIBXSMM_MIN(feature_cpu, feature_os);
+      }
+# endif
+      if (NULL != info) {
+        LIBXSMM_CPUID_X86(0x80000007, 0/*ecx*/, eax, ebx, ecx, edx);
+        info->constant_tsc = LIBXSMM_CPUID_CHECK(edx, 0x00000100);
+        info->has_context = has_context;
+      }
     }
   }
   else {
