@@ -19,6 +19,7 @@ LIBXSMM_API_INTERN libxsmm_dnn_err_t libxsmm_dnn_convolve_st_upd_custom_custom_b
 LIBXSMM_API_INTERN libxsmm_dnn_err_t libxsmm_dnn_convolve_st_upd_nhwc_custom_f32_f32(libxsmm_dnn_layer* handle, int start_thread, int tid);
 LIBXSMM_API_INTERN libxsmm_dnn_err_t libxsmm_dnn_convolve_st_upd_nhwc_rsck_f32_f32(libxsmm_dnn_layer* handle, int start_thread, int tid);
 
+
 LIBXSMM_API_INLINE LIBXSMM_INTRINSICS(LIBXSMM_X86_AVX512_CORE)
 void transpose_32x16(const libxsmm_bfloat16 *in, libxsmm_bfloat16 *out, int ld_in, int ld_out)
 {
@@ -29,22 +30,22 @@ void transpose_32x16(const libxsmm_bfloat16 *in, libxsmm_bfloat16 *out, int ld_i
   const __m512i idx_lo         = _mm512_set_epi64(13, 12, 5, 4, 9, 8, 1, 0);
   const __m512i idx_hi         = _mm512_set_epi64(7, 6, 15, 14, 3, 2, 11, 10);
 
-  r0 = _mm512_load_epi32(in + 0*in_width);
-  r1 = _mm512_load_epi32(in + 1*in_width);
-  r2 = _mm512_load_epi32(in + 2*in_width);
-  r3 = _mm512_load_epi32(in + 3*in_width);
-  r4 = _mm512_load_epi32(in + 4*in_width);
-  r5 = _mm512_load_epi32(in + 5*in_width);
-  r6 = _mm512_load_epi32(in + 6*in_width);
-  r7 = _mm512_load_epi32(in + 7*in_width);
-  r8 = _mm512_load_epi32(in + 8*in_width);
-  r9 = _mm512_load_epi32(in + 9*in_width);
-  ra = _mm512_load_epi32(in + 10*in_width);
-  rb = _mm512_load_epi32(in + 11*in_width);
-  rc = _mm512_load_epi32(in + 12*in_width);
-  rd = _mm512_load_epi32(in + 13*in_width);
-  re = _mm512_load_epi32(in + 14*in_width);
-  rf = _mm512_load_epi32(in + 15*in_width);
+  r0 = _mm512_loadu_si512(in + 0*in_width);
+  r1 = _mm512_loadu_si512(in + 1*in_width);
+  r2 = _mm512_loadu_si512(in + 2*in_width);
+  r3 = _mm512_loadu_si512(in + 3*in_width);
+  r4 = _mm512_loadu_si512(in + 4*in_width);
+  r5 = _mm512_loadu_si512(in + 5*in_width);
+  r6 = _mm512_loadu_si512(in + 6*in_width);
+  r7 = _mm512_loadu_si512(in + 7*in_width);
+  r8 = _mm512_loadu_si512(in + 8*in_width);
+  r9 = _mm512_loadu_si512(in + 9*in_width);
+  ra = _mm512_loadu_si512(in + 10*in_width);
+  rb = _mm512_loadu_si512(in + 11*in_width);
+  rc = _mm512_loadu_si512(in + 12*in_width);
+  rd = _mm512_loadu_si512(in + 13*in_width);
+  re = _mm512_loadu_si512(in + 14*in_width);
+  rf = _mm512_loadu_si512(in + 15*in_width);
 
   t0 = _mm512_unpacklo_epi16(r0,r1);
   t1 = _mm512_unpackhi_epi16(r0,r1);
@@ -181,101 +182,101 @@ void transpose_32xcols(const libxsmm_bfloat16 *in, libxsmm_bfloat16 *out, int co
 
   rf = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
   if (col == 15) {
-    r0 = _mm512_load_epi32(in + 0*in_width);
-    r1 = _mm512_load_epi32(in + 1*in_width);
-    r2 = _mm512_load_epi32(in + 2*in_width);
-    r3 = _mm512_load_epi32(in + 3*in_width);
-    r4 = _mm512_load_epi32(in + 4*in_width);
-    r5 = _mm512_load_epi32(in + 5*in_width);
-    r6 = _mm512_load_epi32(in + 6*in_width);
-    r7 = _mm512_load_epi32(in + 7*in_width);
-    r8 = _mm512_load_epi32(in + 8*in_width);
-    r9 = _mm512_load_epi32(in + 9*in_width);
-    ra = _mm512_load_epi32(in + 10*in_width);
-    rb = _mm512_load_epi32(in + 11*in_width);
-    rc = _mm512_load_epi32(in + 12*in_width);
-    rd = _mm512_load_epi32(in + 13*in_width);
-    re = _mm512_load_epi32(in + 14*in_width);
+    r0 = _mm512_loadu_si512(in + 0*in_width);
+    r1 = _mm512_loadu_si512(in + 1*in_width);
+    r2 = _mm512_loadu_si512(in + 2*in_width);
+    r3 = _mm512_loadu_si512(in + 3*in_width);
+    r4 = _mm512_loadu_si512(in + 4*in_width);
+    r5 = _mm512_loadu_si512(in + 5*in_width);
+    r6 = _mm512_loadu_si512(in + 6*in_width);
+    r7 = _mm512_loadu_si512(in + 7*in_width);
+    r8 = _mm512_loadu_si512(in + 8*in_width);
+    r9 = _mm512_loadu_si512(in + 9*in_width);
+    ra = _mm512_loadu_si512(in + 10*in_width);
+    rb = _mm512_loadu_si512(in + 11*in_width);
+    rc = _mm512_loadu_si512(in + 12*in_width);
+    rd = _mm512_loadu_si512(in + 13*in_width);
+    re = _mm512_loadu_si512(in + 14*in_width);
   } else if (col == 14) {
     re = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
-    r0 = _mm512_load_epi32(in + 0*in_width);
-    r1 = _mm512_load_epi32(in + 1*in_width);
-    r2 = _mm512_load_epi32(in + 2*in_width);
-    r3 = _mm512_load_epi32(in + 3*in_width);
-    r4 = _mm512_load_epi32(in + 4*in_width);
-    r5 = _mm512_load_epi32(in + 5*in_width);
-    r6 = _mm512_load_epi32(in + 6*in_width);
-    r7 = _mm512_load_epi32(in + 7*in_width);
-    r8 = _mm512_load_epi32(in + 8*in_width);
-    r9 = _mm512_load_epi32(in + 9*in_width);
-    ra = _mm512_load_epi32(in + 10*in_width);
-    rb = _mm512_load_epi32(in + 11*in_width);
-    rc = _mm512_load_epi32(in + 12*in_width);
-    rd = _mm512_load_epi32(in + 13*in_width);
+    r0 = _mm512_loadu_si512(in + 0*in_width);
+    r1 = _mm512_loadu_si512(in + 1*in_width);
+    r2 = _mm512_loadu_si512(in + 2*in_width);
+    r3 = _mm512_loadu_si512(in + 3*in_width);
+    r4 = _mm512_loadu_si512(in + 4*in_width);
+    r5 = _mm512_loadu_si512(in + 5*in_width);
+    r6 = _mm512_loadu_si512(in + 6*in_width);
+    r7 = _mm512_loadu_si512(in + 7*in_width);
+    r8 = _mm512_loadu_si512(in + 8*in_width);
+    r9 = _mm512_loadu_si512(in + 9*in_width);
+    ra = _mm512_loadu_si512(in + 10*in_width);
+    rb = _mm512_loadu_si512(in + 11*in_width);
+    rc = _mm512_loadu_si512(in + 12*in_width);
+    rd = _mm512_loadu_si512(in + 13*in_width);
   } else if (col == 13) {
     rd = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     re = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
-    r0 = _mm512_load_epi32(in + 0*in_width);
-    r1 = _mm512_load_epi32(in + 1*in_width);
-    r2 = _mm512_load_epi32(in + 2*in_width);
-    r3 = _mm512_load_epi32(in + 3*in_width);
-    r4 = _mm512_load_epi32(in + 4*in_width);
-    r5 = _mm512_load_epi32(in + 5*in_width);
-    r6 = _mm512_load_epi32(in + 6*in_width);
-    r7 = _mm512_load_epi32(in + 7*in_width);
-    r8 = _mm512_load_epi32(in + 8*in_width);
-    r9 = _mm512_load_epi32(in + 9*in_width);
-    ra = _mm512_load_epi32(in + 10*in_width);
-    rb = _mm512_load_epi32(in + 11*in_width);
-    rc = _mm512_load_epi32(in + 12*in_width);
+    r0 = _mm512_loadu_si512(in + 0*in_width);
+    r1 = _mm512_loadu_si512(in + 1*in_width);
+    r2 = _mm512_loadu_si512(in + 2*in_width);
+    r3 = _mm512_loadu_si512(in + 3*in_width);
+    r4 = _mm512_loadu_si512(in + 4*in_width);
+    r5 = _mm512_loadu_si512(in + 5*in_width);
+    r6 = _mm512_loadu_si512(in + 6*in_width);
+    r7 = _mm512_loadu_si512(in + 7*in_width);
+    r8 = _mm512_loadu_si512(in + 8*in_width);
+    r9 = _mm512_loadu_si512(in + 9*in_width);
+    ra = _mm512_loadu_si512(in + 10*in_width);
+    rb = _mm512_loadu_si512(in + 11*in_width);
+    rc = _mm512_loadu_si512(in + 12*in_width);
   } else if (col == 12) {
     rc = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     rd = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     re = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
-    r0 = _mm512_load_epi32(in + 0*in_width);
-    r1 = _mm512_load_epi32(in + 1*in_width);
-    r2 = _mm512_load_epi32(in + 2*in_width);
-    r3 = _mm512_load_epi32(in + 3*in_width);
-    r4 = _mm512_load_epi32(in + 4*in_width);
-    r5 = _mm512_load_epi32(in + 5*in_width);
-    r6 = _mm512_load_epi32(in + 6*in_width);
-    r7 = _mm512_load_epi32(in + 7*in_width);
-    r8 = _mm512_load_epi32(in + 8*in_width);
-    r9 = _mm512_load_epi32(in + 9*in_width);
-    ra = _mm512_load_epi32(in + 10*in_width);
-    rb = _mm512_load_epi32(in + 11*in_width);
+    r0 = _mm512_loadu_si512(in + 0*in_width);
+    r1 = _mm512_loadu_si512(in + 1*in_width);
+    r2 = _mm512_loadu_si512(in + 2*in_width);
+    r3 = _mm512_loadu_si512(in + 3*in_width);
+    r4 = _mm512_loadu_si512(in + 4*in_width);
+    r5 = _mm512_loadu_si512(in + 5*in_width);
+    r6 = _mm512_loadu_si512(in + 6*in_width);
+    r7 = _mm512_loadu_si512(in + 7*in_width);
+    r8 = _mm512_loadu_si512(in + 8*in_width);
+    r9 = _mm512_loadu_si512(in + 9*in_width);
+    ra = _mm512_loadu_si512(in + 10*in_width);
+    rb = _mm512_loadu_si512(in + 11*in_width);
   } else if (col == 11) {
     rb = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     rc = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     rd = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     re = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
-    r0 = _mm512_load_epi32(in + 0*in_width);
-    r1 = _mm512_load_epi32(in + 1*in_width);
-    r2 = _mm512_load_epi32(in + 2*in_width);
-    r3 = _mm512_load_epi32(in + 3*in_width);
-    r4 = _mm512_load_epi32(in + 4*in_width);
-    r5 = _mm512_load_epi32(in + 5*in_width);
-    r6 = _mm512_load_epi32(in + 6*in_width);
-    r7 = _mm512_load_epi32(in + 7*in_width);
-    r8 = _mm512_load_epi32(in + 8*in_width);
-    r9 = _mm512_load_epi32(in + 9*in_width);
-    ra = _mm512_load_epi32(in + 10*in_width);
+    r0 = _mm512_loadu_si512(in + 0*in_width);
+    r1 = _mm512_loadu_si512(in + 1*in_width);
+    r2 = _mm512_loadu_si512(in + 2*in_width);
+    r3 = _mm512_loadu_si512(in + 3*in_width);
+    r4 = _mm512_loadu_si512(in + 4*in_width);
+    r5 = _mm512_loadu_si512(in + 5*in_width);
+    r6 = _mm512_loadu_si512(in + 6*in_width);
+    r7 = _mm512_loadu_si512(in + 7*in_width);
+    r8 = _mm512_loadu_si512(in + 8*in_width);
+    r9 = _mm512_loadu_si512(in + 9*in_width);
+    ra = _mm512_loadu_si512(in + 10*in_width);
   } else if (col == 10) {
     ra = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     rb = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     rc = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     rd = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     re = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
-    r0 = _mm512_load_epi32(in + 0*in_width);
-    r1 = _mm512_load_epi32(in + 1*in_width);
-    r2 = _mm512_load_epi32(in + 2*in_width);
-    r3 = _mm512_load_epi32(in + 3*in_width);
-    r4 = _mm512_load_epi32(in + 4*in_width);
-    r5 = _mm512_load_epi32(in + 5*in_width);
-    r6 = _mm512_load_epi32(in + 6*in_width);
-    r7 = _mm512_load_epi32(in + 7*in_width);
-    r8 = _mm512_load_epi32(in + 8*in_width);
-    r9 = _mm512_load_epi32(in + 9*in_width);
+    r0 = _mm512_loadu_si512(in + 0*in_width);
+    r1 = _mm512_loadu_si512(in + 1*in_width);
+    r2 = _mm512_loadu_si512(in + 2*in_width);
+    r3 = _mm512_loadu_si512(in + 3*in_width);
+    r4 = _mm512_loadu_si512(in + 4*in_width);
+    r5 = _mm512_loadu_si512(in + 5*in_width);
+    r6 = _mm512_loadu_si512(in + 6*in_width);
+    r7 = _mm512_loadu_si512(in + 7*in_width);
+    r8 = _mm512_loadu_si512(in + 8*in_width);
+    r9 = _mm512_loadu_si512(in + 9*in_width);
   } else if (col == 9) {
     r9 = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     ra = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
@@ -283,15 +284,15 @@ void transpose_32xcols(const libxsmm_bfloat16 *in, libxsmm_bfloat16 *out, int co
     rc = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     rd = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     re = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
-    r0 = _mm512_load_epi32(in + 0*in_width);
-    r1 = _mm512_load_epi32(in + 1*in_width);
-    r2 = _mm512_load_epi32(in + 2*in_width);
-    r3 = _mm512_load_epi32(in + 3*in_width);
-    r4 = _mm512_load_epi32(in + 4*in_width);
-    r5 = _mm512_load_epi32(in + 5*in_width);
-    r6 = _mm512_load_epi32(in + 6*in_width);
-    r7 = _mm512_load_epi32(in + 7*in_width);
-    r8 = _mm512_load_epi32(in + 8*in_width);
+    r0 = _mm512_loadu_si512(in + 0*in_width);
+    r1 = _mm512_loadu_si512(in + 1*in_width);
+    r2 = _mm512_loadu_si512(in + 2*in_width);
+    r3 = _mm512_loadu_si512(in + 3*in_width);
+    r4 = _mm512_loadu_si512(in + 4*in_width);
+    r5 = _mm512_loadu_si512(in + 5*in_width);
+    r6 = _mm512_loadu_si512(in + 6*in_width);
+    r7 = _mm512_loadu_si512(in + 7*in_width);
+    r8 = _mm512_loadu_si512(in + 8*in_width);
   } else if (col == 8) {
     r8 = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     r9 = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
@@ -300,14 +301,14 @@ void transpose_32xcols(const libxsmm_bfloat16 *in, libxsmm_bfloat16 *out, int co
     rc = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     rd = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     re = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
-    r0 = _mm512_load_epi32(in + 0*in_width);
-    r1 = _mm512_load_epi32(in + 1*in_width);
-    r2 = _mm512_load_epi32(in + 2*in_width);
-    r3 = _mm512_load_epi32(in + 3*in_width);
-    r4 = _mm512_load_epi32(in + 4*in_width);
-    r5 = _mm512_load_epi32(in + 5*in_width);
-    r6 = _mm512_load_epi32(in + 6*in_width);
-    r7 = _mm512_load_epi32(in + 7*in_width);
+    r0 = _mm512_loadu_si512(in + 0*in_width);
+    r1 = _mm512_loadu_si512(in + 1*in_width);
+    r2 = _mm512_loadu_si512(in + 2*in_width);
+    r3 = _mm512_loadu_si512(in + 3*in_width);
+    r4 = _mm512_loadu_si512(in + 4*in_width);
+    r5 = _mm512_loadu_si512(in + 5*in_width);
+    r6 = _mm512_loadu_si512(in + 6*in_width);
+    r7 = _mm512_loadu_si512(in + 7*in_width);
   } else if (col == 7) {
     r7 = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     r8 = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
@@ -317,13 +318,13 @@ void transpose_32xcols(const libxsmm_bfloat16 *in, libxsmm_bfloat16 *out, int co
     rc = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     rd = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     re = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
-    r0 = _mm512_load_epi32(in + 0*in_width);
-    r1 = _mm512_load_epi32(in + 1*in_width);
-    r2 = _mm512_load_epi32(in + 2*in_width);
-    r3 = _mm512_load_epi32(in + 3*in_width);
-    r4 = _mm512_load_epi32(in + 4*in_width);
-    r5 = _mm512_load_epi32(in + 5*in_width);
-    r6 = _mm512_load_epi32(in + 6*in_width);
+    r0 = _mm512_loadu_si512(in + 0*in_width);
+    r1 = _mm512_loadu_si512(in + 1*in_width);
+    r2 = _mm512_loadu_si512(in + 2*in_width);
+    r3 = _mm512_loadu_si512(in + 3*in_width);
+    r4 = _mm512_loadu_si512(in + 4*in_width);
+    r5 = _mm512_loadu_si512(in + 5*in_width);
+    r6 = _mm512_loadu_si512(in + 6*in_width);
   } else if (col == 6) {
     r6 = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     r7 = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
@@ -334,12 +335,12 @@ void transpose_32xcols(const libxsmm_bfloat16 *in, libxsmm_bfloat16 *out, int co
     rc = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     rd = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     re = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
-    r0 = _mm512_load_epi32(in + 0*in_width);
-    r1 = _mm512_load_epi32(in + 1*in_width);
-    r2 = _mm512_load_epi32(in + 2*in_width);
-    r3 = _mm512_load_epi32(in + 3*in_width);
-    r4 = _mm512_load_epi32(in + 4*in_width);
-    r5 = _mm512_load_epi32(in + 5*in_width);
+    r0 = _mm512_loadu_si512(in + 0*in_width);
+    r1 = _mm512_loadu_si512(in + 1*in_width);
+    r2 = _mm512_loadu_si512(in + 2*in_width);
+    r3 = _mm512_loadu_si512(in + 3*in_width);
+    r4 = _mm512_loadu_si512(in + 4*in_width);
+    r5 = _mm512_loadu_si512(in + 5*in_width);
   } else if (col == 5) {
     r5 = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     r6 = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
@@ -351,11 +352,11 @@ void transpose_32xcols(const libxsmm_bfloat16 *in, libxsmm_bfloat16 *out, int co
     rc = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     rd = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     re = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
-    r0 = _mm512_load_epi32(in + 0*in_width);
-    r1 = _mm512_load_epi32(in + 1*in_width);
-    r2 = _mm512_load_epi32(in + 2*in_width);
-    r3 = _mm512_load_epi32(in + 3*in_width);
-    r4 = _mm512_load_epi32(in + 4*in_width);
+    r0 = _mm512_loadu_si512(in + 0*in_width);
+    r1 = _mm512_loadu_si512(in + 1*in_width);
+    r2 = _mm512_loadu_si512(in + 2*in_width);
+    r3 = _mm512_loadu_si512(in + 3*in_width);
+    r4 = _mm512_loadu_si512(in + 4*in_width);
   } else if (col == 4) {
     r4 = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     r5 = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
@@ -368,10 +369,10 @@ void transpose_32xcols(const libxsmm_bfloat16 *in, libxsmm_bfloat16 *out, int co
     rc = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     rd = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     re = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
-    r0 = _mm512_load_epi32(in + 0*in_width);
-    r1 = _mm512_load_epi32(in + 1*in_width);
-    r2 = _mm512_load_epi32(in + 2*in_width);
-    r3 = _mm512_load_epi32(in + 3*in_width);
+    r0 = _mm512_loadu_si512(in + 0*in_width);
+    r1 = _mm512_loadu_si512(in + 1*in_width);
+    r2 = _mm512_loadu_si512(in + 2*in_width);
+    r3 = _mm512_loadu_si512(in + 3*in_width);
   } else if (col == 3) {
     r3 = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     r4 = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
@@ -385,9 +386,9 @@ void transpose_32xcols(const libxsmm_bfloat16 *in, libxsmm_bfloat16 *out, int co
     rc = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     rd = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     re = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
-    r0 = _mm512_load_epi32(in + 0*in_width);
-    r1 = _mm512_load_epi32(in + 1*in_width);
-    r2 = _mm512_load_epi32(in + 2*in_width);
+    r0 = _mm512_loadu_si512(in + 0*in_width);
+    r1 = _mm512_loadu_si512(in + 1*in_width);
+    r2 = _mm512_loadu_si512(in + 2*in_width);
   } else if (col == 2) {
     r2 = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     r3 = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
@@ -402,8 +403,8 @@ void transpose_32xcols(const libxsmm_bfloat16 *in, libxsmm_bfloat16 *out, int co
     rc = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     rd = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     re = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
-    r0 = _mm512_load_epi32(in + 0*in_width);
-    r1 = _mm512_load_epi32(in + 1*in_width);
+    r0 = _mm512_loadu_si512(in + 0*in_width);
+    r1 = _mm512_loadu_si512(in + 1*in_width);
   } else if (col == 1) {
     r1 = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     r2 = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
@@ -419,7 +420,7 @@ void transpose_32xcols(const libxsmm_bfloat16 *in, libxsmm_bfloat16 *out, int co
     rc = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     rd = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     re = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
-    r0 = _mm512_load_epi32(in + 0*in_width);
+    r0 = _mm512_loadu_si512(in + 0*in_width);
   } else {
     r0 = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
     r1 = LIBXSMM_INTRINSICS_MM512_UNDEFINED_EPI32();
@@ -615,7 +616,13 @@ libxsmm_dnn_err_t libxsmm_dnn_convolve_st_upd_custom_custom_bf16_bf16_emu(libxsm
   typedef libxsmm_bfloat16 element_filter_type;
   typedef libxsmm_bsmmfunction gemm_function;
   typedef libxsmm_bsmmfunction_reducebatch_addr gemm_br_function;
+
+  /* some portable macrros fof BF16 <-> FP32 */
+# include "template/libxsmm_dnn_bf16_macros_define.tpl.c"
+
 # include "template/libxsmm_dnn_convolve_st_upd_custom_custom_generic_bf16.tpl.c"
+
+# include "template/libxsmm_dnn_bf16_macros_undefine.tpl.c"
 #else /* should not happen */
   LIBXSMM_UNUSED(handle); LIBXSMM_UNUSED(start_thread); LIBXSMM_UNUSED(tid);
   status = LIBXSMM_DNN_ERR_UNSUPPORTED_ARCH;
@@ -635,9 +642,15 @@ libxsmm_dnn_err_t libxsmm_dnn_convolve_st_upd_custom_custom_bf16_bf16(libxsmm_dn
   typedef libxsmm_bfloat16 element_filter_type;
   typedef libxsmm_bsmmfunction gemm_function;
   typedef libxsmm_bsmmfunction_reducebatch_addr gemm_br_function;
-#define LIBXSMM_DNN_CONVOLUTION_UPD_AVX512_CPX
+
+#define LIBXSMM_DNN_BF16_USE_CPX_AVX512_NI
+  /* some portable macrros fof BF16 <-> FP32 */
+# include "template/libxsmm_dnn_bf16_macros_define.tpl.c"
+
 # include "template/libxsmm_dnn_convolve_st_upd_custom_custom_generic_bf16.tpl.c"
-#undef LIBXSMM_DNN_CONVOLUTION_UPD_AVX512_CPX
+
+# include "template/libxsmm_dnn_bf16_macros_undefine.tpl.c"
+#undef LIBXSMM_DNN_BF16_USE_CPX_AVX512_NI
 #else /* should not happen */
   LIBXSMM_UNUSED(handle); LIBXSMM_UNUSED(start_thread); LIBXSMM_UNUSED(tid);
   status = LIBXSMM_DNN_ERR_UNSUPPORTED_ARCH;
@@ -700,13 +713,7 @@ LIBXSMM_API_INTERN libxsmm_dnn_err_t libxsmm_dnn_convolve_st_upd_custom_custom(l
   libxsmm_dnn_err_t status = LIBXSMM_DNN_SUCCESS;
 
   /* check if we have input, output and filter */
-  if (handle->reg_input == 0 || handle->grad_output == 0 || handle->grad_filter == 0 || handle->scratch3 == 0) {
-    status = LIBXSMM_DNN_ERR_DATA_NOT_BOUND;
-    return status;
-  }
-
-  /* check if we scratch for MB parallel execution */
-  if ( (handle->upd_use_thread_fil == 1) && (handle->scratch4 == 0) ) {
+  if (handle->reg_input == 0 || handle->grad_output == 0 || handle->grad_filter == 0 || handle->scratch == 0) {
     status = LIBXSMM_DNN_ERR_DATA_NOT_BOUND;
     return status;
   }
@@ -758,13 +765,7 @@ LIBXSMM_API_INTERN libxsmm_dnn_err_t libxsmm_dnn_convolve_st_upd_nhwc_custom(lib
   libxsmm_dnn_err_t status = LIBXSMM_DNN_SUCCESS;
 
   /* check if we have input, output and filter */
-  if (handle->reg_input == 0 || handle->grad_output == 0 || handle->grad_filter == 0 || handle->scratch3 == 0) {
-    status = LIBXSMM_DNN_ERR_DATA_NOT_BOUND;
-    return status;
-  }
-
-  /* check if we scratch for MB parallel execution */
-  if ( (handle->upd_use_thread_fil == 1) && (handle->scratch4 == 0) ) {
+  if (handle->reg_input == 0 || handle->grad_output == 0 || handle->grad_filter == 0 || handle->scratch == 0) {
     status = LIBXSMM_DNN_ERR_DATA_NOT_BOUND;
     return status;
   }
@@ -806,13 +807,7 @@ LIBXSMM_API_INTERN libxsmm_dnn_err_t libxsmm_dnn_convolve_st_upd_nhwc_rsck(libxs
   libxsmm_dnn_err_t status = LIBXSMM_DNN_SUCCESS;
 
   /* check if we have input, output and filter */
-  if (handle->reg_input == 0 || handle->grad_output == 0 || handle->grad_filter == 0 || handle->scratch3 == 0) {
-    status = LIBXSMM_DNN_ERR_DATA_NOT_BOUND;
-    return status;
-  }
-
-  /* check if we scratch for MB parallel execution */
-  if ( (handle->upd_use_thread_fil == 1) && (handle->scratch4 == 0) ) {
+  if (handle->reg_input == 0 || handle->grad_output == 0 || handle->grad_filter == 0 || handle->scratch == 0) {
     status = LIBXSMM_DNN_ERR_DATA_NOT_BOUND;
     return status;
   }
