@@ -1,30 +1,10 @@
 /******************************************************************************
-** Copyright (c) 2018-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Kunal Banerjee, Evangelos Georganas (Intel Corp.)
 ******************************************************************************/
@@ -356,7 +336,7 @@ LIBXSMM_API_INTERN void libxsmm_internal_recursive_step(libxsmm_blocked_gemm_han
 }
 
 LIBXSMM_API_INTERN void libxsmm_internal_matrix_zero_ld(libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint ld, LIBXSMM_DNN_ELTWISE_FTYPE *srcdst) {
-  libxsmm_blasint i, j;
+  libxsmm_blasint i = 0, j;
 
   for ( j = 0; j < n; ++j ) {
     LIBXSMM_PRAGMA_SIMD
@@ -367,7 +347,7 @@ LIBXSMM_API_INTERN void libxsmm_internal_matrix_zero_ld(libxsmm_blasint m, libxs
 }
 
 LIBXSMM_API_INTERN void libxsmm_internal_matrix_copy_ld(libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint ld, LIBXSMM_DNN_ELTWISE_FTYPE *src, LIBXSMM_DNN_ELTWISE_FTYPE *dst) {
-  libxsmm_blasint i, j;
+  libxsmm_blasint i = 0, j;
 
   for ( j = 0; j < n; ++j ) {
     LIBXSMM_PRAGMA_SIMD
@@ -378,7 +358,7 @@ LIBXSMM_API_INTERN void libxsmm_internal_matrix_copy_ld(libxsmm_blasint m, libxs
 }
 
 LIBXSMM_API_INTERN void libxsmm_internal_matrix_add_ld(libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint ld, LIBXSMM_DNN_ELTWISE_FTYPE *src0, LIBXSMM_DNN_ELTWISE_FTYPE *src1, LIBXSMM_DNN_ELTWISE_FTYPE *dst) {
-  libxsmm_blasint i, j;
+  libxsmm_blasint i = 0, j;
 
   for ( j = 0; j < n; ++j ) {
     LIBXSMM_PRAGMA_SIMD
@@ -389,7 +369,7 @@ LIBXSMM_API_INTERN void libxsmm_internal_matrix_add_ld(libxsmm_blasint m, libxsm
 }
 
 LIBXSMM_API_INTERN void libxsmm_internal_matrix_sub_ld(libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint ld, LIBXSMM_DNN_ELTWISE_FTYPE *src0, LIBXSMM_DNN_ELTWISE_FTYPE *src1, LIBXSMM_DNN_ELTWISE_FTYPE *dst) {
-  libxsmm_blasint i, j;
+  libxsmm_blasint i = 0, j;
 
   for ( j = 0; j < n; ++j ) {
     LIBXSMM_PRAGMA_SIMD
@@ -400,7 +380,7 @@ LIBXSMM_API_INTERN void libxsmm_internal_matrix_sub_ld(libxsmm_blasint m, libxsm
 }
 
 LIBXSMM_API_INTERN void libxsmm_internal_matrix_eltwise_mult_ld(libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint ld, LIBXSMM_DNN_ELTWISE_FTYPE *src0, LIBXSMM_DNN_ELTWISE_FTYPE *src1, LIBXSMM_DNN_ELTWISE_FTYPE *dst) {
-  libxsmm_blasint i, j;
+  libxsmm_blasint i = 0, j;
 
   for ( j = 0; j < n; ++j ) {
     LIBXSMM_PRAGMA_SIMD
@@ -411,7 +391,7 @@ LIBXSMM_API_INTERN void libxsmm_internal_matrix_eltwise_mult_ld(libxsmm_blasint
 }
 
 LIBXSMM_API_INTERN void libxsmm_internal_matrix_inplace_eltwise_mult_ld(libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint ld, LIBXSMM_DNN_ELTWISE_FTYPE *src0, LIBXSMM_DNN_ELTWISE_FTYPE *srcdst) {
-  libxsmm_blasint i, j;
+  libxsmm_blasint i = 0, j;
 
   for ( j = 0; j < n; ++j ) {
     LIBXSMM_PRAGMA_SIMD
@@ -422,7 +402,7 @@ LIBXSMM_API_INTERN void libxsmm_internal_matrix_inplace_eltwise_mult_ld(libxsmm_
 }
 
 LIBXSMM_API_INTERN void libxsmm_internal_matrix_eltwise_fma_ld(libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint ld, LIBXSMM_DNN_ELTWISE_FTYPE *src0, LIBXSMM_DNN_ELTWISE_FTYPE *src1, LIBXSMM_DNN_ELTWISE_FTYPE *dst) {
-  libxsmm_blasint i, j;
+  libxsmm_blasint i = 0, j;
 
   for ( j = 0; j < n; ++j ) {
     LIBXSMM_PRAGMA_SIMD
@@ -433,7 +413,7 @@ LIBXSMM_API_INTERN void libxsmm_internal_matrix_eltwise_fma_ld(libxsmm_blasint m
 }
 
 LIBXSMM_API_INTERN void libxsmm_internal_matrix_add_colvector_ld(libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint ld, LIBXSMM_DNN_ELTWISE_FTYPE *srcdst, LIBXSMM_DNN_ELTWISE_FTYPE *colv) {
-  libxsmm_blasint i, j;
+  libxsmm_blasint i = 0, j;
 
   for ( j = 0; j < n; ++j ) {
     LIBXSMM_PRAGMA_SIMD
@@ -444,7 +424,7 @@ LIBXSMM_API_INTERN void libxsmm_internal_matrix_add_colvector_ld(libxsmm_blasint
 }
 
 LIBXSMM_API_INTERN void libxsmm_internal_matrix_bcst_colvector_ld(libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint ld, LIBXSMM_DNN_ELTWISE_FTYPE *srcdst, LIBXSMM_DNN_ELTWISE_FTYPE *colv) {
-  libxsmm_blasint i, j;
+  libxsmm_blasint i = 0, j;
 
   for ( j = 0; j < n; ++j ) {
     LIBXSMM_PRAGMA_SIMD
@@ -468,7 +448,7 @@ LIBXSMM_API_INTERN void libxsmm_internal_matrix_bcst_cvt_bf16_fp32_colvector_ld(
 }
 
 LIBXSMM_API_INTERN void libxsmm_internal_matrix_bcst_colvector_const_ld(libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint ld, LIBXSMM_DNN_ELTWISE_FTYPE *srcdst, LIBXSMM_DNN_ELTWISE_FTYPE *colv, LIBXSMM_DNN_ELTWISE_FTYPE const_bias) {
-  libxsmm_blasint i, j;
+  libxsmm_blasint i = 0, j;
 
   for ( j = 0; j < n; ++j ) {
     LIBXSMM_PRAGMA_SIMD
@@ -492,7 +472,7 @@ LIBXSMM_API_INTERN void libxsmm_internal_matrix_bcst_cvt_bf16_fp32_colvector_con
 }
 
 LIBXSMM_API_INTERN void libxsmm_internal_matrix_sigmoid_ld(libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint ld, LIBXSMM_DNN_ELTWISE_FTYPE *src, LIBXSMM_DNN_ELTWISE_FTYPE *dst) {
-  libxsmm_blasint i, j;
+  libxsmm_blasint i = 0, j;
 
   for ( j = 0; j < n; ++j ) {
     LIBXSMM_PRAGMA_SIMD
@@ -504,7 +484,7 @@ LIBXSMM_API_INTERN void libxsmm_internal_matrix_sigmoid_ld(libxsmm_blasint m, li
 }
 
 LIBXSMM_API_INTERN void libxsmm_internal_matrix_tanh_ld(libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint ld, LIBXSMM_DNN_ELTWISE_FTYPE *src, LIBXSMM_DNN_ELTWISE_FTYPE *dst) {
-  libxsmm_blasint i, j;
+  libxsmm_blasint i = 0, j;
 
   for ( j = 0; j < n; ++j ) {
     LIBXSMM_PRAGMA_SIMD
@@ -515,7 +495,7 @@ LIBXSMM_API_INTERN void libxsmm_internal_matrix_tanh_ld(libxsmm_blasint m, libxs
 }
 
 LIBXSMM_API_INTERN void libxsmm_internal_matrix_relu_ld(libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint ld, LIBXSMM_DNN_ELTWISE_FTYPE *src, LIBXSMM_DNN_ELTWISE_FTYPE *dst) {
-  libxsmm_blasint i, j;
+  libxsmm_blasint i = 0, j;
 
   for ( j = 0; j < n; ++j ) {
     LIBXSMM_PRAGMA_SIMD
@@ -526,7 +506,7 @@ LIBXSMM_API_INTERN void libxsmm_internal_matrix_relu_ld(libxsmm_blasint m, libxs
 }
 
 LIBXSMM_API_INTERN void libxsmm_internal_matrix_sigmoid_inverse_ld(libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint ld, LIBXSMM_DNN_ELTWISE_FTYPE *src, LIBXSMM_DNN_ELTWISE_FTYPE *dst) {
-  libxsmm_blasint i, j;
+  libxsmm_blasint i = 0, j;
 
   for ( j = 0; j < n; ++j ) {
     LIBXSMM_PRAGMA_SIMD
@@ -539,7 +519,7 @@ LIBXSMM_API_INTERN void libxsmm_internal_matrix_sigmoid_inverse_ld(libxsmm_blasi
 }
 
 LIBXSMM_API_INTERN void libxsmm_internal_matrix_tanh_inverse_ld(libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint ld, LIBXSMM_DNN_ELTWISE_FTYPE *src, LIBXSMM_DNN_ELTWISE_FTYPE *dst) {
-  libxsmm_blasint i, j;
+  libxsmm_blasint i = 0, j;
 
   for ( j = 0; j < n; ++j ) {
     LIBXSMM_PRAGMA_SIMD
@@ -551,7 +531,7 @@ LIBXSMM_API_INTERN void libxsmm_internal_matrix_tanh_inverse_ld(libxsmm_blasint
 }
 
 LIBXSMM_API_INTERN void libxsmm_internal_matrix_relu_inverse_ld(libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint ld, LIBXSMM_DNN_ELTWISE_FTYPE *src, LIBXSMM_DNN_ELTWISE_FTYPE *dst) {
-  libxsmm_blasint i, j;
+  libxsmm_blasint i = 0, j;
 
   for ( j = 0; j < n; ++j ) {
     LIBXSMM_PRAGMA_SIMD
@@ -562,7 +542,7 @@ LIBXSMM_API_INTERN void libxsmm_internal_matrix_relu_inverse_ld(libxsmm_blasint
 }
 
 LIBXSMM_API_INTERN void libxsmm_internal_matrix_sigmoid_inverse_inplace_eltwise_mult_ld(libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint ld, LIBXSMM_DNN_ELTWISE_FTYPE *src, LIBXSMM_DNN_ELTWISE_FTYPE *dst) {
-  libxsmm_blasint i, j;
+  libxsmm_blasint i = 0, j;
 
   for ( j = 0; j < n; ++j ) {
     LIBXSMM_PRAGMA_SIMD
@@ -575,7 +555,7 @@ LIBXSMM_API_INTERN void libxsmm_internal_matrix_sigmoid_inverse_inplace_eltwise_
 }
 
 LIBXSMM_API_INTERN void libxsmm_internal_matrix_tanh_inverse_inplace_eltwise_mult_ld(libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint ld, LIBXSMM_DNN_ELTWISE_FTYPE *src, LIBXSMM_DNN_ELTWISE_FTYPE *dst) {
-  libxsmm_blasint i, j;
+  libxsmm_blasint i = 0, j;
 
   for ( j = 0; j < n; ++j ) {
     LIBXSMM_PRAGMA_SIMD
@@ -587,7 +567,7 @@ LIBXSMM_API_INTERN void libxsmm_internal_matrix_tanh_inverse_inplace_eltwise_mul
 }
 
 LIBXSMM_API_INTERN void libxsmm_internal_matrix_relu_inverse_inplace_eltwise_mult_ld(libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint ld, LIBXSMM_DNN_ELTWISE_FTYPE *src, LIBXSMM_DNN_ELTWISE_FTYPE *dst) {
-  libxsmm_blasint i, j;
+  libxsmm_blasint i = 0, j;
 
   for ( j = 0; j < n; ++j ) {
     LIBXSMM_PRAGMA_SIMD
@@ -598,7 +578,7 @@ LIBXSMM_API_INTERN void libxsmm_internal_matrix_relu_inverse_inplace_eltwise_mul
 }
 
 LIBXSMM_API_INTERN void libxsmm_internal_matrix_complement_ld(libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint ld, LIBXSMM_DNN_ELTWISE_FTYPE *src, LIBXSMM_DNN_ELTWISE_FTYPE *dst) {
-  libxsmm_blasint i, j;
+  libxsmm_blasint i = 0, j;
 
   for ( j = 0; j < n; ++j ) {
     LIBXSMM_PRAGMA_SIMD
@@ -609,7 +589,7 @@ LIBXSMM_API_INTERN void libxsmm_internal_matrix_complement_ld(libxsmm_blasint m,
 }
 
 LIBXSMM_API_INTERN void libxsmm_internal_matrix_complement_square_ld(libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint ld, LIBXSMM_DNN_ELTWISE_FTYPE *src, LIBXSMM_DNN_ELTWISE_FTYPE *dst) {
-  libxsmm_blasint i, j;
+  libxsmm_blasint i = 0, j;
 
   for ( j = 0; j < n; ++j ) {
     LIBXSMM_PRAGMA_SIMD
@@ -687,3 +667,4 @@ LIBXSMM_API_INTERN void libxsmm_internal_matrix_cvt_bf16_fp32_ld(libxsmm_blasint
     }
   }
 }
+
