@@ -27,14 +27,14 @@ void bf16_vnni_transpose_16x16(void* source_void, void* dest_void, int source_st
   __m512i tmp0, tmp1, tmp2, tmp3;
   const __m512i abcdefgh_to_abefcdgh = _mm512_set4_epi32(0x0f0e0b0a, 0x0d0c0908, 0x07060302, 0x05040100);
 
-  zmm0 = _mm512_load_epi32(source);
-  zmm1 = _mm512_load_epi32(source + source_stride);
-  zmm2 = _mm512_load_epi32(source + source_stride*2);
-  zmm3 = _mm512_load_epi32(source + source_stride*3);
-  zmm4 = _mm512_load_epi32(source + source_stride*4);
-  zmm5 = _mm512_load_epi32(source + source_stride*5);
-  zmm6 = _mm512_load_epi32(source + source_stride*6);
-  zmm7 = _mm512_load_epi32(source + source_stride*7);
+  zmm0 = _mm512_loadu_si512(source);
+  zmm1 = _mm512_loadu_si512(source + source_stride);
+  zmm2 = _mm512_loadu_si512(source + source_stride*2);
+  zmm3 = _mm512_loadu_si512(source + source_stride*3);
+  zmm4 = _mm512_loadu_si512(source + source_stride*4);
+  zmm5 = _mm512_loadu_si512(source + source_stride*5);
+  zmm6 = _mm512_loadu_si512(source + source_stride*6);
+  zmm7 = _mm512_loadu_si512(source + source_stride*7);
 
   zmm0 = _mm512_shuffle_epi8(zmm0, abcdefgh_to_abefcdgh);
   zmm1 = _mm512_shuffle_epi8(zmm1, abcdefgh_to_abefcdgh);
@@ -72,14 +72,14 @@ void bf16_vnni_transpose_16x16(void* source_void, void* dest_void, int source_st
   zmm6 = _mm512_shuffle_i32x4(zmm6, tmp1, 0xdd);
   zmm7 = _mm512_shuffle_i32x4(zmm7, tmp3, 0xdd);
 
-  _mm512_store_epi32(dest, zmm0);
-  _mm512_store_epi32(dest + dest_stride, zmm1);
-  _mm512_store_epi32(dest + dest_stride * 2, zmm2);
-  _mm512_store_epi32(dest + dest_stride * 3, zmm3);
-  _mm512_store_epi32(dest + dest_stride * 4, zmm4);
-  _mm512_store_epi32(dest + dest_stride * 5, zmm5);
-  _mm512_store_epi32(dest + dest_stride * 6, zmm6);
-  _mm512_store_epi32(dest + dest_stride * 7, zmm7);
+  _mm512_storeu_si512(dest, zmm0);
+  _mm512_storeu_si512(dest + dest_stride, zmm1);
+  _mm512_storeu_si512(dest + dest_stride * 2, zmm2);
+  _mm512_storeu_si512(dest + dest_stride * 3, zmm3);
+  _mm512_storeu_si512(dest + dest_stride * 4, zmm4);
+  _mm512_storeu_si512(dest + dest_stride * 5, zmm5);
+  _mm512_storeu_si512(dest + dest_stride * 6, zmm6);
+  _mm512_storeu_si512(dest + dest_stride * 7, zmm7);
 #else
   LIBXSMM_UNUSED(source_void); LIBXSMM_UNUSED(dest_void); LIBXSMM_UNUSED(source_stride); LIBXSMM_UNUSED(dest_stride);
 #endif
@@ -111,22 +111,22 @@ void bf16_transpose_32x16(libxsmm_bfloat16 *in, libxsmm_bfloat16 *out, int ld_in
   const __m512i idx_lo         = _mm512_set_epi64(13, 12, 5, 4, 9, 8, 1, 0);
   const __m512i idx_hi         = _mm512_set_epi64(7, 6, 15, 14, 3, 2, 11, 10);
 
-  r0 = _mm512_load_epi32(in + 0*in_width);
-  r1 = _mm512_load_epi32(in + 1*in_width);
-  r2 = _mm512_load_epi32(in + 2*in_width);
-  r3 = _mm512_load_epi32(in + 3*in_width);
-  r4 = _mm512_load_epi32(in + 4*in_width);
-  r5 = _mm512_load_epi32(in + 5*in_width);
-  r6 = _mm512_load_epi32(in + 6*in_width);
-  r7 = _mm512_load_epi32(in + 7*in_width);
-  r8 = _mm512_load_epi32(in + 8*in_width);
-  r9 = _mm512_load_epi32(in + 9*in_width);
-  ra = _mm512_load_epi32(in + 10*in_width);
-  rb = _mm512_load_epi32(in + 11*in_width);
-  rc = _mm512_load_epi32(in + 12*in_width);
-  rd = _mm512_load_epi32(in + 13*in_width);
-  re = _mm512_load_epi32(in + 14*in_width);
-  rf = _mm512_load_epi32(in + 15*in_width);
+  r0 = _mm512_loadu_si512(in + 0*in_width);
+  r1 = _mm512_loadu_si512(in + 1*in_width);
+  r2 = _mm512_loadu_si512(in + 2*in_width);
+  r3 = _mm512_loadu_si512(in + 3*in_width);
+  r4 = _mm512_loadu_si512(in + 4*in_width);
+  r5 = _mm512_loadu_si512(in + 5*in_width);
+  r6 = _mm512_loadu_si512(in + 6*in_width);
+  r7 = _mm512_loadu_si512(in + 7*in_width);
+  r8 = _mm512_loadu_si512(in + 8*in_width);
+  r9 = _mm512_loadu_si512(in + 9*in_width);
+  ra = _mm512_loadu_si512(in + 10*in_width);
+  rb = _mm512_loadu_si512(in + 11*in_width);
+  rc = _mm512_loadu_si512(in + 12*in_width);
+  rd = _mm512_loadu_si512(in + 13*in_width);
+  re = _mm512_loadu_si512(in + 14*in_width);
+  rf = _mm512_loadu_si512(in + 15*in_width);
 
   t0 = _mm512_unpacklo_epi16(r0,r1);
   t1 = _mm512_unpackhi_epi16(r0,r1);
@@ -262,140 +262,140 @@ void bf16_transpose_32xcols(libxsmm_bfloat16 *in, libxsmm_bfloat16 *out, int col
   __mmask16 store_mask         = LIBXSMM_INTRINSICS_MM512_CVTU32_MASK16(((unsigned int)1 << col) - 1);
 
   if (col == 15) {
-    r0 = _mm512_load_epi32(in + 0*in_width);
-    r1 = _mm512_load_epi32(in + 1*in_width);
-    r2 = _mm512_load_epi32(in + 2*in_width);
-    r3 = _mm512_load_epi32(in + 3*in_width);
-    r4 = _mm512_load_epi32(in + 4*in_width);
-    r5 = _mm512_load_epi32(in + 5*in_width);
-    r6 = _mm512_load_epi32(in + 6*in_width);
-    r7 = _mm512_load_epi32(in + 7*in_width);
-    r8 = _mm512_load_epi32(in + 8*in_width);
-    r9 = _mm512_load_epi32(in + 9*in_width);
-    ra = _mm512_load_epi32(in + 10*in_width);
-    rb = _mm512_load_epi32(in + 11*in_width);
-    rc = _mm512_load_epi32(in + 12*in_width);
-    rd = _mm512_load_epi32(in + 13*in_width);
-    re = _mm512_load_epi32(in + 14*in_width);
+    r0 = _mm512_loadu_si512(in + 0*in_width);
+    r1 = _mm512_loadu_si512(in + 1*in_width);
+    r2 = _mm512_loadu_si512(in + 2*in_width);
+    r3 = _mm512_loadu_si512(in + 3*in_width);
+    r4 = _mm512_loadu_si512(in + 4*in_width);
+    r5 = _mm512_loadu_si512(in + 5*in_width);
+    r6 = _mm512_loadu_si512(in + 6*in_width);
+    r7 = _mm512_loadu_si512(in + 7*in_width);
+    r8 = _mm512_loadu_si512(in + 8*in_width);
+    r9 = _mm512_loadu_si512(in + 9*in_width);
+    ra = _mm512_loadu_si512(in + 10*in_width);
+    rb = _mm512_loadu_si512(in + 11*in_width);
+    rc = _mm512_loadu_si512(in + 12*in_width);
+    rd = _mm512_loadu_si512(in + 13*in_width);
+    re = _mm512_loadu_si512(in + 14*in_width);
   } else if (col == 14) {
-    r0 = _mm512_load_epi32(in + 0*in_width);
-    r1 = _mm512_load_epi32(in + 1*in_width);
-    r2 = _mm512_load_epi32(in + 2*in_width);
-    r3 = _mm512_load_epi32(in + 3*in_width);
-    r4 = _mm512_load_epi32(in + 4*in_width);
-    r5 = _mm512_load_epi32(in + 5*in_width);
-    r6 = _mm512_load_epi32(in + 6*in_width);
-    r7 = _mm512_load_epi32(in + 7*in_width);
-    r8 = _mm512_load_epi32(in + 8*in_width);
-    r9 = _mm512_load_epi32(in + 9*in_width);
-    ra = _mm512_load_epi32(in + 10*in_width);
-    rb = _mm512_load_epi32(in + 11*in_width);
-    rc = _mm512_load_epi32(in + 12*in_width);
-    rd = _mm512_load_epi32(in + 13*in_width);
+    r0 = _mm512_loadu_si512(in + 0*in_width);
+    r1 = _mm512_loadu_si512(in + 1*in_width);
+    r2 = _mm512_loadu_si512(in + 2*in_width);
+    r3 = _mm512_loadu_si512(in + 3*in_width);
+    r4 = _mm512_loadu_si512(in + 4*in_width);
+    r5 = _mm512_loadu_si512(in + 5*in_width);
+    r6 = _mm512_loadu_si512(in + 6*in_width);
+    r7 = _mm512_loadu_si512(in + 7*in_width);
+    r8 = _mm512_loadu_si512(in + 8*in_width);
+    r9 = _mm512_loadu_si512(in + 9*in_width);
+    ra = _mm512_loadu_si512(in + 10*in_width);
+    rb = _mm512_loadu_si512(in + 11*in_width);
+    rc = _mm512_loadu_si512(in + 12*in_width);
+    rd = _mm512_loadu_si512(in + 13*in_width);
   } else if (col == 13) {
-    r0 = _mm512_load_epi32(in + 0*in_width);
-    r1 = _mm512_load_epi32(in + 1*in_width);
-    r2 = _mm512_load_epi32(in + 2*in_width);
-    r3 = _mm512_load_epi32(in + 3*in_width);
-    r4 = _mm512_load_epi32(in + 4*in_width);
-    r5 = _mm512_load_epi32(in + 5*in_width);
-    r6 = _mm512_load_epi32(in + 6*in_width);
-    r7 = _mm512_load_epi32(in + 7*in_width);
-    r8 = _mm512_load_epi32(in + 8*in_width);
-    r9 = _mm512_load_epi32(in + 9*in_width);
-    ra = _mm512_load_epi32(in + 10*in_width);
-    rb = _mm512_load_epi32(in + 11*in_width);
-    rc = _mm512_load_epi32(in + 12*in_width);
+    r0 = _mm512_loadu_si512(in + 0*in_width);
+    r1 = _mm512_loadu_si512(in + 1*in_width);
+    r2 = _mm512_loadu_si512(in + 2*in_width);
+    r3 = _mm512_loadu_si512(in + 3*in_width);
+    r4 = _mm512_loadu_si512(in + 4*in_width);
+    r5 = _mm512_loadu_si512(in + 5*in_width);
+    r6 = _mm512_loadu_si512(in + 6*in_width);
+    r7 = _mm512_loadu_si512(in + 7*in_width);
+    r8 = _mm512_loadu_si512(in + 8*in_width);
+    r9 = _mm512_loadu_si512(in + 9*in_width);
+    ra = _mm512_loadu_si512(in + 10*in_width);
+    rb = _mm512_loadu_si512(in + 11*in_width);
+    rc = _mm512_loadu_si512(in + 12*in_width);
   } else if (col == 12) {
-    r0 = _mm512_load_epi32(in + 0*in_width);
-    r1 = _mm512_load_epi32(in + 1*in_width);
-    r2 = _mm512_load_epi32(in + 2*in_width);
-    r3 = _mm512_load_epi32(in + 3*in_width);
-    r4 = _mm512_load_epi32(in + 4*in_width);
-    r5 = _mm512_load_epi32(in + 5*in_width);
-    r6 = _mm512_load_epi32(in + 6*in_width);
-    r7 = _mm512_load_epi32(in + 7*in_width);
-    r8 = _mm512_load_epi32(in + 8*in_width);
-    r9 = _mm512_load_epi32(in + 9*in_width);
-    ra = _mm512_load_epi32(in + 10*in_width);
-    rb = _mm512_load_epi32(in + 11*in_width);
+    r0 = _mm512_loadu_si512(in + 0*in_width);
+    r1 = _mm512_loadu_si512(in + 1*in_width);
+    r2 = _mm512_loadu_si512(in + 2*in_width);
+    r3 = _mm512_loadu_si512(in + 3*in_width);
+    r4 = _mm512_loadu_si512(in + 4*in_width);
+    r5 = _mm512_loadu_si512(in + 5*in_width);
+    r6 = _mm512_loadu_si512(in + 6*in_width);
+    r7 = _mm512_loadu_si512(in + 7*in_width);
+    r8 = _mm512_loadu_si512(in + 8*in_width);
+    r9 = _mm512_loadu_si512(in + 9*in_width);
+    ra = _mm512_loadu_si512(in + 10*in_width);
+    rb = _mm512_loadu_si512(in + 11*in_width);
   } else if (col == 11) {
-    r0 = _mm512_load_epi32(in + 0*in_width);
-    r1 = _mm512_load_epi32(in + 1*in_width);
-    r2 = _mm512_load_epi32(in + 2*in_width);
-    r3 = _mm512_load_epi32(in + 3*in_width);
-    r4 = _mm512_load_epi32(in + 4*in_width);
-    r5 = _mm512_load_epi32(in + 5*in_width);
-    r6 = _mm512_load_epi32(in + 6*in_width);
-    r7 = _mm512_load_epi32(in + 7*in_width);
-    r8 = _mm512_load_epi32(in + 8*in_width);
-    r9 = _mm512_load_epi32(in + 9*in_width);
-    ra = _mm512_load_epi32(in + 10*in_width);
+    r0 = _mm512_loadu_si512(in + 0*in_width);
+    r1 = _mm512_loadu_si512(in + 1*in_width);
+    r2 = _mm512_loadu_si512(in + 2*in_width);
+    r3 = _mm512_loadu_si512(in + 3*in_width);
+    r4 = _mm512_loadu_si512(in + 4*in_width);
+    r5 = _mm512_loadu_si512(in + 5*in_width);
+    r6 = _mm512_loadu_si512(in + 6*in_width);
+    r7 = _mm512_loadu_si512(in + 7*in_width);
+    r8 = _mm512_loadu_si512(in + 8*in_width);
+    r9 = _mm512_loadu_si512(in + 9*in_width);
+    ra = _mm512_loadu_si512(in + 10*in_width);
   } else if (col == 10) {
-    r0 = _mm512_load_epi32(in + 0*in_width);
-    r1 = _mm512_load_epi32(in + 1*in_width);
-    r2 = _mm512_load_epi32(in + 2*in_width);
-    r3 = _mm512_load_epi32(in + 3*in_width);
-    r4 = _mm512_load_epi32(in + 4*in_width);
-    r5 = _mm512_load_epi32(in + 5*in_width);
-    r6 = _mm512_load_epi32(in + 6*in_width);
-    r7 = _mm512_load_epi32(in + 7*in_width);
-    r8 = _mm512_load_epi32(in + 8*in_width);
-    r9 = _mm512_load_epi32(in + 9*in_width);
+    r0 = _mm512_loadu_si512(in + 0*in_width);
+    r1 = _mm512_loadu_si512(in + 1*in_width);
+    r2 = _mm512_loadu_si512(in + 2*in_width);
+    r3 = _mm512_loadu_si512(in + 3*in_width);
+    r4 = _mm512_loadu_si512(in + 4*in_width);
+    r5 = _mm512_loadu_si512(in + 5*in_width);
+    r6 = _mm512_loadu_si512(in + 6*in_width);
+    r7 = _mm512_loadu_si512(in + 7*in_width);
+    r8 = _mm512_loadu_si512(in + 8*in_width);
+    r9 = _mm512_loadu_si512(in + 9*in_width);
   } else if (col == 9) {
-    r0 = _mm512_load_epi32(in + 0*in_width);
-    r1 = _mm512_load_epi32(in + 1*in_width);
-    r2 = _mm512_load_epi32(in + 2*in_width);
-    r3 = _mm512_load_epi32(in + 3*in_width);
-    r4 = _mm512_load_epi32(in + 4*in_width);
-    r5 = _mm512_load_epi32(in + 5*in_width);
-    r6 = _mm512_load_epi32(in + 6*in_width);
-    r7 = _mm512_load_epi32(in + 7*in_width);
-    r8 = _mm512_load_epi32(in + 8*in_width);
+    r0 = _mm512_loadu_si512(in + 0*in_width);
+    r1 = _mm512_loadu_si512(in + 1*in_width);
+    r2 = _mm512_loadu_si512(in + 2*in_width);
+    r3 = _mm512_loadu_si512(in + 3*in_width);
+    r4 = _mm512_loadu_si512(in + 4*in_width);
+    r5 = _mm512_loadu_si512(in + 5*in_width);
+    r6 = _mm512_loadu_si512(in + 6*in_width);
+    r7 = _mm512_loadu_si512(in + 7*in_width);
+    r8 = _mm512_loadu_si512(in + 8*in_width);
   } else if (col == 8) {
-    r0 = _mm512_load_epi32(in + 0*in_width);
-    r1 = _mm512_load_epi32(in + 1*in_width);
-    r2 = _mm512_load_epi32(in + 2*in_width);
-    r3 = _mm512_load_epi32(in + 3*in_width);
-    r4 = _mm512_load_epi32(in + 4*in_width);
-    r5 = _mm512_load_epi32(in + 5*in_width);
-    r6 = _mm512_load_epi32(in + 6*in_width);
-    r7 = _mm512_load_epi32(in + 7*in_width);
+    r0 = _mm512_loadu_si512(in + 0*in_width);
+    r1 = _mm512_loadu_si512(in + 1*in_width);
+    r2 = _mm512_loadu_si512(in + 2*in_width);
+    r3 = _mm512_loadu_si512(in + 3*in_width);
+    r4 = _mm512_loadu_si512(in + 4*in_width);
+    r5 = _mm512_loadu_si512(in + 5*in_width);
+    r6 = _mm512_loadu_si512(in + 6*in_width);
+    r7 = _mm512_loadu_si512(in + 7*in_width);
   } else if (col == 7) {
-    r0 = _mm512_load_epi32(in + 0*in_width);
-    r1 = _mm512_load_epi32(in + 1*in_width);
-    r2 = _mm512_load_epi32(in + 2*in_width);
-    r3 = _mm512_load_epi32(in + 3*in_width);
-    r4 = _mm512_load_epi32(in + 4*in_width);
-    r5 = _mm512_load_epi32(in + 5*in_width);
-    r6 = _mm512_load_epi32(in + 6*in_width);
+    r0 = _mm512_loadu_si512(in + 0*in_width);
+    r1 = _mm512_loadu_si512(in + 1*in_width);
+    r2 = _mm512_loadu_si512(in + 2*in_width);
+    r3 = _mm512_loadu_si512(in + 3*in_width);
+    r4 = _mm512_loadu_si512(in + 4*in_width);
+    r5 = _mm512_loadu_si512(in + 5*in_width);
+    r6 = _mm512_loadu_si512(in + 6*in_width);
   } else if (col == 6) {
-    r0 = _mm512_load_epi32(in + 0*in_width);
-    r1 = _mm512_load_epi32(in + 1*in_width);
-    r2 = _mm512_load_epi32(in + 2*in_width);
-    r3 = _mm512_load_epi32(in + 3*in_width);
-    r4 = _mm512_load_epi32(in + 4*in_width);
-    r5 = _mm512_load_epi32(in + 5*in_width);
+    r0 = _mm512_loadu_si512(in + 0*in_width);
+    r1 = _mm512_loadu_si512(in + 1*in_width);
+    r2 = _mm512_loadu_si512(in + 2*in_width);
+    r3 = _mm512_loadu_si512(in + 3*in_width);
+    r4 = _mm512_loadu_si512(in + 4*in_width);
+    r5 = _mm512_loadu_si512(in + 5*in_width);
   } else if (col == 5) {
-    r0 = _mm512_load_epi32(in + 0*in_width);
-    r1 = _mm512_load_epi32(in + 1*in_width);
-    r2 = _mm512_load_epi32(in + 2*in_width);
-    r3 = _mm512_load_epi32(in + 3*in_width);
-    r4 = _mm512_load_epi32(in + 4*in_width);
+    r0 = _mm512_loadu_si512(in + 0*in_width);
+    r1 = _mm512_loadu_si512(in + 1*in_width);
+    r2 = _mm512_loadu_si512(in + 2*in_width);
+    r3 = _mm512_loadu_si512(in + 3*in_width);
+    r4 = _mm512_loadu_si512(in + 4*in_width);
   } else if (col == 4) {
-    r0 = _mm512_load_epi32(in + 0*in_width);
-    r1 = _mm512_load_epi32(in + 1*in_width);
-    r2 = _mm512_load_epi32(in + 2*in_width);
-    r3 = _mm512_load_epi32(in + 3*in_width);
+    r0 = _mm512_loadu_si512(in + 0*in_width);
+    r1 = _mm512_loadu_si512(in + 1*in_width);
+    r2 = _mm512_loadu_si512(in + 2*in_width);
+    r3 = _mm512_loadu_si512(in + 3*in_width);
   } else if (col == 3) {
-    r0 = _mm512_load_epi32(in + 0*in_width);
-    r1 = _mm512_load_epi32(in + 1*in_width);
-    r2 = _mm512_load_epi32(in + 2*in_width);
+    r0 = _mm512_loadu_si512(in + 0*in_width);
+    r1 = _mm512_loadu_si512(in + 1*in_width);
+    r2 = _mm512_loadu_si512(in + 2*in_width);
   } else if (col == 2) {
-    r0 = _mm512_load_epi32(in + 0*in_width);
-    r1 = _mm512_load_epi32(in + 1*in_width);
+    r0 = _mm512_loadu_si512(in + 0*in_width);
+    r1 = _mm512_loadu_si512(in + 1*in_width);
   } else if (col == 1) {
-    r0 = _mm512_load_epi32(in + 0*in_width);
+    r0 = _mm512_loadu_si512(in + 0*in_width);
   }
 
   t0 = _mm512_unpacklo_epi16(r0,r1);
@@ -720,6 +720,9 @@ libxsmm_dnn_err_t libxsmm_dnn_fullyconnected_st_bwdupd_ncnc_kcck_bf16_bf16_emu(l
   libxsmm_bsmmfunction_reducebatch_strd batchreduce_kernel_upd = handle->gemm_upd.xgemm.bsmrs;
   libxsmm_bmmfunction_reducebatch_strd batchreduce_kernel_upd_zerobeta = handle->gemm_upd2.xgemm.bmrs;
 
+  /* some portable macrros fof BF16 <-> FP32 */
+# include "template/libxsmm_dnn_bf16_macros_define.tpl.c"
+
   if ( handle->desc.fuse_ops == LIBXSMM_DNN_FULLYCONNECTED_FUSE_NONE ) {
 # include "template/libxsmm_dnn_fullyconnected_st_bwdupd_ncnc_kcck_generic_bf16.tpl.c"
   } else if ( handle->desc.fuse_ops == LIBXSMM_DNN_FULLYCONNECTED_FUSE_BIAS ) {
@@ -749,6 +752,8 @@ libxsmm_dnn_err_t libxsmm_dnn_fullyconnected_st_bwdupd_ncnc_kcck_bf16_bf16_emu(l
   } else {
     status = LIBXSMM_DNN_ERR_FC_UNSUPPORTED_FUSION;
   }
+
+# include "template/libxsmm_dnn_bf16_macros_undefine.tpl.c"
 #else /* should not happen */
   LIBXSMM_UNUSED(handle); LIBXSMM_UNUSED(kind); LIBXSMM_UNUSED(start_thread); LIBXSMM_UNUSED(tid);
   status = LIBXSMM_DNN_ERR_UNSUPPORTED_ARCH;
@@ -770,6 +775,10 @@ libxsmm_dnn_err_t libxsmm_dnn_fullyconnected_st_bwdupd_ncnc_kcck_bf16_bf16(libxs
   libxsmm_bsmmfunction_reducebatch_strd batchreduce_kernel_upd = handle->gemm_upd.xgemm.bsmrs;
   libxsmm_bmmfunction_reducebatch_strd batchreduce_kernel_upd_zerobeta = handle->gemm_upd2.xgemm.bmrs;
 
+#define LIBXSMM_DNN_BF16_USE_CPX_AVX512_NI
+  /* some portable macrros fof BF16 <-> FP32 */
+# include "template/libxsmm_dnn_bf16_macros_define.tpl.c"
+
   if ( handle->desc.fuse_ops == LIBXSMM_DNN_FULLYCONNECTED_FUSE_NONE ) {
 # include "template/libxsmm_dnn_fullyconnected_st_bwdupd_ncnc_kcck_generic_bf16.tpl.c"
   } else if ( handle->desc.fuse_ops == LIBXSMM_DNN_FULLYCONNECTED_FUSE_BIAS ) {
@@ -799,6 +808,9 @@ libxsmm_dnn_err_t libxsmm_dnn_fullyconnected_st_bwdupd_ncnc_kcck_bf16_bf16(libxs
   } else {
     status = LIBXSMM_DNN_ERR_FC_UNSUPPORTED_FUSION;
   }
+
+# include "template/libxsmm_dnn_bf16_macros_undefine.tpl.c"
+#undef LIBXSMM_DNN_BF16_USE_CPX_AVX512_NI
 #else /* should not happen */
   LIBXSMM_UNUSED(handle); LIBXSMM_UNUSED(kind); LIBXSMM_UNUSED(start_thread); LIBXSMM_UNUSED(tid);
   status = LIBXSMM_DNN_ERR_UNSUPPORTED_ARCH;
@@ -818,11 +830,25 @@ LIBXSMM_API_INTERN libxsmm_dnn_err_t libxsmm_dnn_fullyconnected_st_bwdupd_custom
   libxsmm_dnn_err_t status = LIBXSMM_DNN_SUCCESS;
 
   /* check if all required tensors are bound */
-  if (handle->grad_input == 0 || handle->grad_output == 0 ||
-      handle->reg_input  == 0 || handle->grad_filter == 0 ||
-      handle->reg_filter == 0 || handle->scratch == 0         ) {
-    status = LIBXSMM_DNN_ERR_DATA_NOT_BOUND;
-    return status;
+  if ( kind == LIBXSMM_DNN_COMPUTE_KIND_BWD ) {
+    if (handle->grad_input == 0 || handle->grad_output == 0 ||
+        handle->reg_filter == 0 || handle->scratch == 0         ) {
+      status = LIBXSMM_DNN_ERR_DATA_NOT_BOUND;
+      return status;
+    }
+  } else if ( kind == LIBXSMM_DNN_COMPUTE_KIND_UPD ) {
+    if (handle->reg_input == 0   || handle->grad_output == 0 ||
+        handle->grad_filter == 0 || handle->scratch == 0         ) {
+      status = LIBXSMM_DNN_ERR_DATA_NOT_BOUND;
+      return status;
+    }
+  } else {
+    if (handle->grad_input == 0 || handle->grad_output == 0 ||
+        handle->reg_input  == 0 || handle->grad_filter == 0 ||
+        handle->reg_filter == 0 || handle->scratch == 0         ) {
+      status = LIBXSMM_DNN_ERR_DATA_NOT_BOUND;
+      return status;
+    }
   }
 
   /* check if we are on an AVX512 platform */
@@ -904,12 +930,27 @@ LIBXSMM_API_INTERN libxsmm_dnn_err_t libxsmm_dnn_fullyconnected_st_bwdupd_ncnc_k
   libxsmm_dnn_err_t status = LIBXSMM_DNN_SUCCESS;
 
   /* check if all required tensors are bound */
-  if (handle->grad_input == 0 || handle->grad_output == 0 ||
-      handle->reg_input  == 0 || handle->grad_filter == 0 ||
-      handle->reg_filter == 0 || handle->scratch == 0         ) {
-    status = LIBXSMM_DNN_ERR_DATA_NOT_BOUND;
-    return status;
+  if ( kind == LIBXSMM_DNN_COMPUTE_KIND_BWD ) {
+    if (handle->grad_input == 0 || handle->grad_output == 0 ||
+        handle->reg_filter == 0 || handle->scratch == 0         ) {
+      status = LIBXSMM_DNN_ERR_DATA_NOT_BOUND;
+      return status;
+    }
+  } else if ( kind == LIBXSMM_DNN_COMPUTE_KIND_UPD ) {
+    if (handle->reg_input == 0   || handle->grad_output == 0 ||
+        handle->grad_filter == 0 || handle->scratch == 0         ) {
+      status = LIBXSMM_DNN_ERR_DATA_NOT_BOUND;
+      return status;
+    }
+  } else {
+    if (handle->grad_input == 0 || handle->grad_output == 0 ||
+        handle->reg_input  == 0 || handle->grad_filter == 0 ||
+        handle->reg_filter == 0 || handle->scratch == 0         ) {
+      status = LIBXSMM_DNN_ERR_DATA_NOT_BOUND;
+      return status;
+    }
   }
+
   if ( ((handle->desc.fuse_ops & LIBXSMM_DNN_FULLYCONNECTED_FUSE_BIAS ) != 0) && ( handle->grad_bias == 0 ) )  {
     status = LIBXSMM_DNN_ERR_DATA_NOT_BOUND;
     return status;
