@@ -17,6 +17,9 @@ LIBXSMM_API libxsmm_dnn_fusedgroupnorm* libxsmm_dnn_create_fusedgroupnorm(libxsm
   libxsmm_dnn_fusedgroupnorm* handle = 0;
   int lpb;
 
+  /* init libxsmm */
+  LIBXSMM_INIT
+
   if ( ((fusedgroupnorm_desc.datatype_in == LIBXSMM_DNN_DATATYPE_BF16) && (fusedgroupnorm_desc.datatype_out == LIBXSMM_DNN_DATATYPE_BF16)) ||
        ((fusedgroupnorm_desc.datatype_in == LIBXSMM_DNN_DATATYPE_F32) && (fusedgroupnorm_desc.datatype_out == LIBXSMM_DNN_DATATYPE_F32))    ) {
     handle = (libxsmm_dnn_fusedgroupnorm*)malloc(sizeof(libxsmm_dnn_fusedgroupnorm));
