@@ -1,38 +1,14 @@
 /******************************************************************************
-** Copyright (c) 2015-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Alexander Heinecke (Intel Corp.)
 ******************************************************************************/
-
 #include <libxsmm.h>
-#include <stdlib.h>
-#include <string.h>
-#include <stdio.h>
 
 
 LIBXSMM_INLINE void print_help(void) {
@@ -72,9 +48,7 @@ LIBXSMM_INLINE void print_help(void) {
   printf("    0: unaligned C, otherwise aligned\n");
   printf("    ARCH: noarch, wsm, snb, hsw, knl, knm, skx, clx, cpx\n");
   printf("    PREFETCH: nopf (none), pfsigonly, BL2viaC, AL2, curAL2,\n"
-         "              AL2jpst, AL2_BL2viaC, curAL2_BL2viaC,\n"
-         "              AL2jpst_BL2viaC, AL1, BL1, CL1,\n"
-         "              AL1_BL1, BL1_CL1, AL1_CL1, AL1_BL1_CL1\n");
+         "              AL2_BL2viaC, curAL2_BL2viaC,\n");
   printf("    PRECISION: I16, SP, DP\n");
   printf("\n\n\n\n");
 }
@@ -186,33 +160,6 @@ int main(int argc, char* argv []) {
   else if (strcmp("AL2_BL2viaC", argv[15]) == 0) {
     l_prefetch = LIBXSMM_GEMM_PREFETCH_AL2BL2_VIA_C;
   }
-  else if (strcmp("AL2jpst", argv[15]) == 0) {
-    l_prefetch = LIBXSMM_GEMM_PREFETCH_AL2_JPST;
-  }
-  else if (strcmp("AL2jpst_BL2viaC", argv[15]) == 0) {
-    l_prefetch = LIBXSMM_GEMM_PREFETCH_AL2BL2_VIA_C_JPST;
-  }
-  else if (strcmp("AL1", argv[15]) == 0) {
-    l_prefetch = LIBXSMM_GEMM_PREFETCH_AL1;
-  }
-  else if (strcmp("BL1", argv[15]) == 0) {
-    l_prefetch = LIBXSMM_GEMM_PREFETCH_BL1;
-  }
-  else if (strcmp("CL1", argv[15]) == 0) {
-    l_prefetch = LIBXSMM_GEMM_PREFETCH_CL1;
-  }
-  else if (strcmp("AL1_BL1", argv[15]) == 0) {
-    l_prefetch = LIBXSMM_GEMM_PREFETCH_AL1_BL1;
-  }
-  else if (strcmp("BL1_CL1", argv[15]) == 0) {
-    l_prefetch = LIBXSMM_GEMM_PREFETCH_BL1_CL1;
-  }
-  else if (strcmp("AL1_CL1", argv[15]) == 0) {
-    l_prefetch = LIBXSMM_GEMM_PREFETCH_AL1_CL1;
-  }
-  else if (strcmp("AL1_BL1_CL1", argv[15]) == 0) {
-    l_prefetch = LIBXSMM_GEMM_PREFETCH_AL1_BL1_CL1;
-  }
   else {
     print_help();
     return EXIT_FAILURE;
