@@ -1,44 +1,16 @@
 /******************************************************************************
-** Copyright (c) 2015-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Hans Pabst (Intel Corp.)
 ******************************************************************************/
 #include "libxsmm_hash.h"
 #include "libxsmm_main.h"
 
-#if defined(LIBXSMM_OFFLOAD_TARGET)
-# pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
-#endif
-#include <stdio.h>
-#if defined(LIBXSMM_OFFLOAD_TARGET)
-# pragma offload_attribute(pop)
-#endif
-
 #if !defined(LIBXSMM_HASH_ALIGNMENT)
 # define LIBXSMM_HASH_ALIGNMENT 8
 #endif
@@ -71,10 +43,9 @@
 #define LIBXSMM_HASH_CRC32_U16(SEED, PVALUE) _mm_crc32_u16(SEED, *(const uint16_t*)(PVALUE))
 #define LIBXSMM_HASH_CRC32_U32(SEED, PVALUE) _mm_crc32_u32(SEED, *(const uint32_t*)(PVALUE))
 
-#if (64 > (LIBXSMM_BITS))
+#if (64 > (LIBXSMM_BITS)) || defined(__PGI)
 # define LIBXSMM_HASH_CRC32_U64(SEED, PVALUE) \
-    LIBXSMM_HASH_CRC32_U32(((const uint32_t*)(PVALUE))[1], \
-    LIBXSMM_HASH_CRC32_U32(((const uint32_t*)(PVALUE))[0], (uint32_t)(SEED)))
+  LIBXSMM_HASH_CRC32_U32(LIBXSMM_HASH_CRC32_U32((uint32_t)(SEED), PVALUE), (const uint32_t*)(PVALUE) + 1)
 #else
 # define LIBXSMM_HASH_CRC32_U64(SEED, PVALUE) _mm_crc32_u64(SEED, *(const uint64_t*)(PVALUE))
 #endif
@@ -126,47 +97,51 @@
 #endif
 
 typedef uint32_t internal_crc32_entry_type[256];
-LIBXSMM_APIVAR_PRIVATE(const internal_crc32_entry_type* internal_crc32_table);
-LIBXSMM_APIVAR_PRIVATE(libxsmm_hash_function internal_hash_u32_function);
-LIBXSMM_APIVAR_PRIVATE(libxsmm_hash_function internal_hash_u64_function);
-LIBXSMM_APIVAR_PRIVATE(libxsmm_hash_function internal_hash_u128_function);
-LIBXSMM_APIVAR_PRIVATE(libxsmm_hash_function internal_hash_u256_function);
-LIBXSMM_APIVAR_PRIVATE(libxsmm_hash_function internal_hash_u384_function);
-LIBXSMM_APIVAR_PRIVATE(libxsmm_hash_function internal_hash_u512_function);
-LIBXSMM_APIVAR_PRIVATE(libxsmm_hash_function internal_hash_function);
+LIBXSMM_APIVAR_DEFINE(const internal_crc32_entry_type* internal_crc32_table);
+LIBXSMM_APIVAR_DEFINE(libxsmm_hash_function internal_hash_u32_function);
+LIBXSMM_APIVAR_DEFINE(libxsmm_hash_function internal_hash_u64_function);
+LIBXSMM_APIVAR_DEFINE(libxsmm_hash_function internal_hash_u128_function);
+LIBXSMM_APIVAR_DEFINE(libxsmm_hash_function internal_hash_u256_function);
+LIBXSMM_APIVAR_DEFINE(libxsmm_hash_function internal_hash_u384_function);
+LIBXSMM_APIVAR_DEFINE(libxsmm_hash_function internal_hash_u512_function);
+LIBXSMM_APIVAR_DEFINE(libxsmm_hash_function internal_hash_function);
 
 
 LIBXSMM_API_INLINE unsigned int internal_crc32_u8(unsigned int seed, const void* value)
 {
-  const uint8_t u8 = *(const uint8_t*)value;
-  LIBXSMM_ASSERT(NULL != internal_crc32_table);
-  return internal_crc32_table[0][(seed^u8) & 0xFF] ^ (seed >> 8);
+  const uint8_t *const pu8 = (const uint8_t*)value;
+  LIBXSMM_ASSERT(NULL != pu8 && NULL != internal_crc32_table);
+  return internal_crc32_table[0][(seed^(*pu8)) & 0xFF] ^ (seed >> 8);
 }
 
 
 LIBXSMM_API_INLINE unsigned int internal_crc32_u16(unsigned int seed, const void* value)
 {
   const uint8_t *const pu8 = (const uint8_t*)value;
+  LIBXSMM_ASSERT(NULL != pu8);
   seed = internal_crc32_u8(seed, pu8 + 0);
   seed = internal_crc32_u8(seed, pu8 + 1);
   return seed;
 }
 
 
-LIBXSMM_API_INLINE unsigned int internal_crc32_u32(unsigned int seed, const void* value, ...)
+LIBXSMM_API_INTERN unsigned int internal_crc32_u32(unsigned int seed, const void* value, ...);
+LIBXSMM_API_INTERN unsigned int internal_crc32_u32(unsigned int seed, const void* value, ...)
 {
-  const uint32_t u32 = *(const uint32_t*)value, s = seed ^ u32;
-  uint32_t c0, c1, c2, c3;
-  LIBXSMM_ASSERT(NULL != internal_crc32_table);
+  const uint32_t *const pu32 = (const uint32_t*)value;
+  uint32_t c0, c1, c2, c3, s;
+  LIBXSMM_ASSERT(NULL != pu32 && NULL != internal_crc32_table);
+  s = seed ^ (*pu32);
   c0 = internal_crc32_table[0][(s >> 24) & 0xFF];
   c1 = internal_crc32_table[1][(s >> 16) & 0xFF];
   c2 = internal_crc32_table[2][(s >> 8) & 0xFF];
-  c3 = internal_crc32_table[3][s & 0xFF];
+  c3 = internal_crc32_table[3][(s & 0xFF)];
   return (c0 ^ c1) ^ (c2 ^ c3);
 }
 
 
-LIBXSMM_API_INLINE LIBXSMM_INTRINSICS(LIBXSMM_X86_SSE4)
+LIBXSMM_API_INTERN unsigned int internal_crc32_u32_sse4(unsigned int seed, const void* value, ...);
+LIBXSMM_API_INTERN LIBXSMM_INTRINSICS(LIBXSMM_X86_SSE4)
 unsigned int internal_crc32_u32_sse4(unsigned int seed, const void* value, ...)
 {
 #if defined(LIBXSMM_INTRINSICS_SSE4)
@@ -177,16 +152,19 @@ unsigned int internal_crc32_u32_sse4(unsigned int seed, const void* value, ...)
 }
 
 
-LIBXSMM_API_INLINE unsigned int internal_crc32_u64(unsigned int seed, const void* value, ...)
+LIBXSMM_API_INTERN unsigned int internal_crc32_u64(unsigned int seed, const void* value, ...);
+LIBXSMM_API_INTERN unsigned int internal_crc32_u64(unsigned int seed, const void* value, ...)
 {
   const uint32_t *const pu32 = (const uint32_t*)value;
+  LIBXSMM_ASSERT(NULL != pu32);
   seed = internal_crc32_u32(seed, pu32 + 0);
   seed = internal_crc32_u32(seed, pu32 + 1);
   return seed;
 }
 
 
-LIBXSMM_API_INLINE LIBXSMM_INTRINSICS(LIBXSMM_X86_SSE4)
+LIBXSMM_API_INTERN unsigned int internal_crc32_u64_sse4(unsigned int seed, const void* value, ...);
+LIBXSMM_API_INTERN LIBXSMM_INTRINSICS(LIBXSMM_X86_SSE4)
 unsigned int internal_crc32_u64_sse4(unsigned int seed, const void* value, ...)
 {
 #if defined(LIBXSMM_INTRINSICS_SSE4)
@@ -197,41 +175,51 @@ unsigned int internal_crc32_u64_sse4(unsigned int seed, const void* value, ...)
 }
 
 
-LIBXSMM_API_INLINE unsigned int internal_crc32_u128(unsigned int seed, const void* value, ...)
+LIBXSMM_API_INTERN unsigned int internal_crc32_u128(unsigned int seed, const void* value, ...);
+LIBXSMM_API_INTERN unsigned int internal_crc32_u128(unsigned int seed, const void* value, ...)
 {
   const uint64_t *const pu64 = (const uint64_t*)value;
+  LIBXSMM_ASSERT(NULL != pu64);
   seed = internal_crc32_u64(seed, pu64 + 0);
   seed = internal_crc32_u64(seed, pu64 + 1);
   return seed;
 }
 
 
-LIBXSMM_API_INLINE LIBXSMM_INTRINSICS(LIBXSMM_X86_SSE4)
+LIBXSMM_API_INTERN unsigned int internal_crc32_u128_sse4(unsigned int seed, const void* value, ...);
+LIBXSMM_API_INTERN LIBXSMM_INTRINSICS(LIBXSMM_X86_SSE4)
 unsigned int internal_crc32_u128_sse4(unsigned int seed, const void* value, ...)
 {
 #if defined(LIBXSMM_INTRINSICS_SSE4)
   const uint64_t *const pu64 = (const uint64_t*)value;
-  return (unsigned int)LIBXSMM_HASH_CRC32_U64(LIBXSMM_HASH_CRC32_U64(seed, pu64), pu64 + 1);
+  LIBXSMM_ASSERT(NULL != pu64);
+  seed = (unsigned int)LIBXSMM_HASH_CRC32_U64(seed, pu64 + 0);
+  seed = (unsigned int)LIBXSMM_HASH_CRC32_U64(seed, pu64 + 1);
 #else
-  return internal_crc32_u128(seed, value);
+  seed = internal_crc32_u128(seed, value);
 #endif
+  return seed;
 }
 
 
-LIBXSMM_API_INLINE unsigned int internal_crc32_u256(unsigned int seed, const void* value, ...)
+LIBXSMM_API_INTERN unsigned int internal_crc32_u256(unsigned int seed, const void* value, ...);
+LIBXSMM_API_INTERN unsigned int internal_crc32_u256(unsigned int seed, const void* value, ...)
 {
   const uint8_t *const pu8 = (const uint8_t*)value;
+  LIBXSMM_ASSERT(NULL != pu8);
   seed = internal_crc32_u128(seed, pu8 + 0x00);
   seed = internal_crc32_u128(seed, pu8 + 0x10);
   return seed;
 }
 
 
-LIBXSMM_API_INLINE LIBXSMM_INTRINSICS(LIBXSMM_X86_SSE4)
+LIBXSMM_API_INTERN unsigned int internal_crc32_u256_sse4(unsigned int seed, const void* value, ...);
+LIBXSMM_API_INTERN LIBXSMM_INTRINSICS(LIBXSMM_X86_SSE4)
 unsigned int internal_crc32_u256_sse4(unsigned int seed, const void* value, ...)
 {
 #if defined(LIBXSMM_INTRINSICS_SSE4)
   const uint8_t *const pu8 = (const uint8_t*)value;
+  LIBXSMM_ASSERT(NULL != pu8);
   seed = internal_crc32_u128_sse4(seed, pu8 + 0x00);
   seed = internal_crc32_u128_sse4(seed, pu8 + 0x10);
   return seed;
@@ -241,20 +229,24 @@ unsigned int internal_crc32_u256_sse4(unsigned int seed, const void* value, ...)
 }
 
 
-LIBXSMM_API_INLINE unsigned int internal_crc32_u384(unsigned int seed, const void* value, ...)
+LIBXSMM_API_INTERN unsigned int internal_crc32_u384(unsigned int seed, const void* value, ...);
+LIBXSMM_API_INTERN unsigned int internal_crc32_u384(unsigned int seed, const void* value, ...)
 {
   const uint8_t *const pu8 = (const uint8_t*)value;
+  LIBXSMM_ASSERT(NULL != pu8);
   seed = internal_crc32_u256(seed, pu8 + 0x00);
   seed = internal_crc32_u128(seed, pu8 + 0x20);
   return seed;
 }
 
 
-LIBXSMM_API_INLINE LIBXSMM_INTRINSICS(LIBXSMM_X86_SSE4)
+LIBXSMM_API_INTERN unsigned int internal_crc32_u384_sse4(unsigned int seed, const void* value, ...);
+LIBXSMM_API_INTERN LIBXSMM_INTRINSICS(LIBXSMM_X86_SSE4)
 unsigned int internal_crc32_u384_sse4(unsigned int seed, const void* value, ...)
 {
 #if defined(LIBXSMM_INTRINSICS_SSE4)
   const uint8_t *const pu8 = (const uint8_t*)value;
+  LIBXSMM_ASSERT(NULL != pu8);
   seed = internal_crc32_u256_sse4(seed, pu8 + 0x00);
   seed = internal_crc32_u128_sse4(seed, pu8 + 0x20);
   return seed;
@@ -264,20 +256,24 @@ unsigned int internal_crc32_u384_sse4(unsigned int seed, const void* value, ...)
 }
 
 
-LIBXSMM_API_INLINE unsigned int internal_crc32_u512(unsigned int seed, const void* value, ...)
+LIBXSMM_API_INTERN unsigned int internal_crc32_u512(unsigned int seed, const void* value, ...);
+LIBXSMM_API_INTERN unsigned int internal_crc32_u512(unsigned int seed, const void* value, ...)
 {
   const uint8_t *const pu8 = (const uint8_t*)value;
+  LIBXSMM_ASSERT(NULL != pu8);
   seed = internal_crc32_u256(seed, pu8 + 0x00);
   seed = internal_crc32_u256(seed, pu8 + 0x20);
   return seed;
 }
 
 
-LIBXSMM_API_INLINE LIBXSMM_INTRINSICS(LIBXSMM_X86_SSE4)
+LIBXSMM_API_INTERN unsigned int internal_crc32_u512_sse4(unsigned int seed, const void* value, ...);
+LIBXSMM_API_INTERN LIBXSMM_INTRINSICS(LIBXSMM_X86_SSE4)
 unsigned int internal_crc32_u512_sse4(unsigned int seed, const void* value, ...)
 {
 #if defined(LIBXSMM_INTRINSICS_SSE4)
   const uint8_t *const pu8 = (const uint8_t*)value;
+  LIBXSMM_ASSERT(NULL != pu8);
   seed = internal_crc32_u256_sse4(seed, pu8 + 0x00);
   seed = internal_crc32_u256_sse4(seed, pu8 + 0x20);
   return seed;
@@ -287,14 +283,16 @@ unsigned int internal_crc32_u512_sse4(unsigned int seed, const void* value, ...)
 }
 
 
-LIBXSMM_API_INLINE unsigned int internal_crc32(unsigned int seed, const void* data, size_t size)
+LIBXSMM_API_INTERN unsigned int internal_crc32(unsigned int seed, const void* data, size_t size);
+LIBXSMM_API_INTERN unsigned int internal_crc32(unsigned int seed, const void* data, size_t size)
 {
   LIBXSMM_ASSERT(NULL != data || 0 == size);
   LIBXSMM_HASH(internal_crc32_u64, internal_crc32_u32, internal_crc32_u16, internal_crc32_u8, seed, data, size);
 }
 
 
-LIBXSMM_API_INLINE LIBXSMM_INTRINSICS(LIBXSMM_X86_SSE4)
+LIBXSMM_API_INTERN unsigned int internal_crc32_sse4(unsigned int seed, const void* data, size_t size);
+LIBXSMM_API_INTERN LIBXSMM_INTRINSICS(LIBXSMM_X86_SSE4)
 unsigned int internal_crc32_sse4(unsigned int seed, const void* data, size_t size)
 {
   LIBXSMM_ASSERT(NULL != data || 0 == size);
@@ -463,10 +461,7 @@ LIBXSMM_API_INTERN void libxsmm_hash_init(int target_arch)
     internal_hash_function = (libxsmm_hash_function)internal_crc32_sse4;
   }
 #if (LIBXSMM_X86_SSE4 > LIBXSMM_STATIC_TARGET_ARCH)
-  else
-#endif
-#if (LIBXSMM_X86_SSE4 > LIBXSMM_STATIC_TARGET_ARCH)
-  {
+  else {
 # if !defined(LIBXSMM_INTRINSICS_SSE4)
     static int error_once = 0;
     if (0 == error_once && 0 != libxsmm_verbosity) { /* library code is expected to be mute */
@@ -512,6 +507,8 @@ LIBXSMM_API_INTERN unsigned int libxsmm_crc32_u32(unsigned int seed, const void*
 {
 #if (LIBXSMM_X86_SSE4 <= LIBXSMM_STATIC_TARGET_ARCH)
   return LIBXSMM_HASH_CRC32_U32(seed, value);
+#elif (LIBXSMM_X86_SSE4 > LIBXSMM_MAX_STATIC_TARGET_ARCH)
+  return internal_crc32_u32(seed, value);
 #else /* pointer based function call */
   LIBXSMM_ASSERT(NULL != internal_hash_u32_function);
   return internal_hash_u32_function(seed, value);
@@ -523,6 +520,8 @@ LIBXSMM_API_INTERN unsigned int libxsmm_crc32_u64(unsigned int seed, const void*
 {
 #if (LIBXSMM_X86_SSE4 <= LIBXSMM_STATIC_TARGET_ARCH)
   return (unsigned int)LIBXSMM_HASH_CRC32_U64(seed, value);
+#elif (LIBXSMM_X86_SSE4 > LIBXSMM_MAX_STATIC_TARGET_ARCH)
+  return internal_crc32_u64(seed, value);
 #else /* pointer based function call */
   LIBXSMM_ASSERT(NULL != internal_hash_u64_function);
   return internal_hash_u64_function(seed, value);
@@ -534,6 +533,8 @@ LIBXSMM_API_INTERN unsigned int libxsmm_crc32_u128(unsigned int seed, const void
 {
 #if (LIBXSMM_X86_SSE4 <= LIBXSMM_STATIC_TARGET_ARCH)
   return internal_crc32_u128_sse4(seed, value);
+#elif (LIBXSMM_X86_SSE4 > LIBXSMM_MAX_STATIC_TARGET_ARCH)
+  return internal_crc32_u128(seed, value);
 #else /* pointer based function call */
   LIBXSMM_ASSERT(NULL != internal_hash_u128_function);
   return internal_hash_u128_function(seed, value);
@@ -545,6 +546,8 @@ LIBXSMM_API_INTERN unsigned int libxsmm_crc32_u256(unsigned int seed, const void
 {
 #if (LIBXSMM_X86_SSE4 <= LIBXSMM_STATIC_TARGET_ARCH)
   return internal_crc32_u256_sse4(seed, value);
+#elif (LIBXSMM_X86_SSE4 > LIBXSMM_MAX_STATIC_TARGET_ARCH)
+  return internal_crc32_u256(seed, value);
 #else /* pointer based function call */
   LIBXSMM_ASSERT(NULL != internal_hash_u256_function);
   return internal_hash_u256_function(seed, value);
@@ -552,10 +555,25 @@ LIBXSMM_API_INTERN unsigned int libxsmm_crc32_u256(unsigned int seed, const void
 }
 
 
+LIBXSMM_API_INTERN unsigned int libxsmm_crc32_u384(unsigned int seed, const void* value, ...)
+{
+#if (LIBXSMM_X86_SSE4 <= LIBXSMM_STATIC_TARGET_ARCH)
+  return internal_crc32_u384_sse4(seed, value);
+#elif (LIBXSMM_X86_SSE4 > LIBXSMM_MAX_STATIC_TARGET_ARCH)
+  return internal_crc32_u384(seed, value);
+#else /* pointer based function call */
+  LIBXSMM_ASSERT(NULL != internal_hash_u384_function);
+  return internal_hash_u384_function(seed, value);
+#endif
+}
+
+
 LIBXSMM_API_INTERN unsigned int libxsmm_crc32_u512(unsigned int seed, const void* value, ...)
 {
 #if (LIBXSMM_X86_SSE4 <= LIBXSMM_STATIC_TARGET_ARCH)
   return internal_crc32_u512_sse4(seed, value);
+#elif (LIBXSMM_X86_SSE4 > LIBXSMM_MAX_STATIC_TARGET_ARCH)
+  return internal_crc32_u512(seed, value);
 #else /* pointer based function call */
   LIBXSMM_ASSERT(NULL != internal_hash_u512_function);
   return internal_hash_u512_function(seed, value);
@@ -567,6 +585,8 @@ LIBXSMM_API_INTERN unsigned int libxsmm_crc32(unsigned int seed, const void* dat
 {
 #if (LIBXSMM_X86_SSE4 <= LIBXSMM_STATIC_TARGET_ARCH)
   return internal_crc32_sse4(seed, data, size);
+#elif (LIBXSMM_X86_SSE4 > LIBXSMM_MAX_STATIC_TARGET_ARCH)
+  return internal_crc32(seed, data, size);
 #else /* pointer based function call */
   LIBXSMM_ASSERT(NULL != internal_hash_function);
   return internal_hash_function(seed, data, size);
