@@ -51,6 +51,16 @@
 #if !defined(LIBXSMM_MALLOC_LIMIT)
 # define LIBXSMM_MALLOC_LIMIT (2U << 20) /* 2 MB */
 #endif
+#if !defined(LIBXSMM_MALLOC_HOOK_REALLOC) && 1
+# define LIBXSMM_MALLOC_HOOK_REALLOC
+#endif
+#if !defined(LIBXSMM_MALLOC_HOOK_CALLOC) && 1
+# define LIBXSMM_MALLOC_HOOK_CALLOC
+#endif
+/* align even if interceptor is disabled at runtime */
+#if !defined(LIBXSMM_MALLOC_ALIGN_ALL) && 1
+# define LIBXSMM_MALLOC_ALIGN_ALL
+#endif
 #if !defined(LIBXSMM_MALLOC_INTERNAL_CALLER_ID)
 # define LIBXSMM_MALLOC_INTERNAL_CALLER_ID ((uintptr_t)LIBXSMM_UNLIMITED)
 #endif
@@ -60,11 +70,29 @@
 
 #if !defined(LIBXSMM_INTERCEPT_DYNAMIC) && defined(LIBXSMM_BUILD) && \
   (defined(__GNUC__) || defined(_CRAYC)) && !defined(_WIN32) && !defined(__CYGWIN__) && \
-  !(defined(__APPLE__) && defined(__MACH__) && LIBXSMM_VERSION3(6, 1, 0) >= \
-    LIBXSMM_VERSION3(__clang_major__, __clang_minor__, __clang_patchlevel__))
+  !(defined(__APPLE__) && defined(__MACH__) && LIBXSMM_VERSION2(6, 1) >= \
+    LIBXSMM_VERSION2(__clang_major__, __clang_minor__))
 # define LIBXSMM_INTERCEPT_DYNAMIC
 #endif
 
+#if !defined(LIBXSMM_MALLOC_HOOK_DYNAMIC) && defined(LIBXSMM_INTERCEPT_DYNAMIC) && \
+  defined(LIBXSMM_MALLOC) && (0 != LIBXSMM_MALLOC) && \
+  (!defined(_CRAYC) && !defined(__TRACE)) /* TODO */ && \
+  (defined(LIBXSMM_BUILD) && (1 < (LIBXSMM_BUILD))) /* GLIBC */
+# define LIBXSMM_MALLOC_HOOK_DYNAMIC
+#endif
+#if !defined(LIBXSMM_MALLOC_HOOK_STATIC) && \
+  defined(LIBXSMM_MALLOC) && (0 != LIBXSMM_MALLOC) && \
+  (!defined(_WIN32)) /* TODO */ && \
+  (defined(LIBXSMM_BUILD) && (1 < (LIBXSMM_BUILD))) /* GLIBC */
+# define LIBXSMM_MALLOC_HOOK_STATIC
+#endif
+#if !defined(LIBXSMM_DNN_CONVOLUTION_SETUP_USE_NTS) && \
+     defined(LIBXSMM_MALLOC_HOOK_DYNAMIC) && \
+     defined(LIBXSMM_MALLOC_ALIGN_ALL)
+# define LIBXSMM_DNN_CONVOLUTION_SETUP_USE_NTS
+#endif
+
 #if defined(LIBXSMM_INTERCEPT_DYNAMIC)
 # if defined(LIBXSMM_OFFLOAD_TARGET)
 #   pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
@@ -137,7 +165,10 @@
     | (LIBXSMM_NEQ(0, BETA) ? 0 : LIBXSMM_GEMM_FLAG_BETA_0)); \
   (DESCRIPTOR).m   = (unsigned int)(M);   (DESCRIPTOR).n   = (unsigned int)(N);   (DESCRIPTOR).k   = (unsigned int)(K); \
   (DESCRIPTOR).lda = (unsigned int)(LDA); (DESCRIPTOR).ldb = (unsigned int)(LDB); (DESCRIPTOR).ldc = (unsigned int)(LDC); \
-  LIBXSMM_PAD((DESCRIPTOR).pad = 0) (DESCRIPTOR).c1 = 0; (DESCRIPTOR).c2 = 0; (DESCRIPTOR).c3 = 0
+  LIBXSMM_PAD((DESCRIPTOR).pad = 0) (DESCRIPTOR).c1 = 0; (DESCRIPTOR).c2 = 0; (DESCRIPTOR).c3 = 0; \
+  (DESCRIPTOR).meltw_ldx = 0; (DESCRIPTOR).meltw_ldy = 0; (DESCRIPTOR).meltw_ldz = 0; \
+  (DESCRIPTOR).meltw_datatype_aux = 0; (DESCRIPTOR).meltw_flags = 0; \
+  (DESCRIPTOR).meltw_operation = 0
 
 /** Similar to LIBXSMM_GEMM_DESCRIPTOR, but separately taking the input-/output-precision. */
 #define LIBXSMM_GEMM_DESCRIPTOR2(DESCRIPTOR, IPREC, OPREC, FLAGS, M, N, K, LDA, LDB, LDC, ALPHA, BETA, PREFETCH) \
@@ -156,6 +187,7 @@
 #define LIBXSMM_REGDESC(START, MODIFIER) \
   START libxsmm_gemm_descriptor MODIFIER gemm; \
   START libxsmm_mcopy_descriptor MODIFIER mcopy; \
+  START libxsmm_meltw_descriptor MODIFIER meltw; \
   START libxsmm_trans_descriptor MODIFIER trans; \
   START libxsmm_pgemm_descriptor MODIFIER pgemm; \
   START libxsmm_getrf_descriptor MODIFIER getrf; \
@@ -179,13 +211,21 @@ LIBXSMM_EXTERN_C LIBXSMM_PACKED(struct LIBXSMM_RETARGETABLE) libxsmm_gemm_descri
   /** Denotes the data-type. */
   unsigned char datatype;
   /** Ignored entry. */
-  LIBXSMM_PAD(unsigned short pad)
+  LIBXSMM_PAD(unsigned char pad)
   /** multipurpose 64bit field, currently used for: a) stride_a in brgemm */
   unsigned long long c1;
   /** multipurpose 64bit field, currently used for: a) stride_b in brgemm */
   unsigned long long c2;
   /** multipurpose 8bit field, currently used for: a) unroll hint in brgemm */
   unsigned char c3;
+  /** LDx, LDy, LDz,  additional meltw LDs */
+  unsigned int meltw_ldx, meltw_ldy, meltw_ldz;
+  /** Size of data element. */
+  unsigned char meltw_datatype_aux;
+  /** Set of flags */
+  unsigned char meltw_flags;
+  /** operation specifier */
+  unsigned char meltw_operation;
 };
 
 /** Packed structure storing the matcopy argument description. */
@@ -202,6 +242,19 @@ LIBXSMM_EXTERN_C LIBXSMM_PACKED(struct LIBXSMM_RETARGETABLE) libxsmm_mcopy_descr
   unsigned char flags;
 };
 
+/** Packed structure storing the mateltw argument description. */
+LIBXSMM_EXTERN_C LIBXSMM_PACKED(struct LIBXSMM_RETARGETABLE) libxsmm_meltw_descriptor {
+  /** LDx, M, and N. */
+  unsigned int m, n, ldi, ldo, ldx, ldy;
+  /** Size of data element. */
+  unsigned char datatype;
+  unsigned char datatype2;
+  /** Set of flags */
+  unsigned char flags;
+  /** operation specifier */
+  unsigned char operation;
+};
+
 /** Packed structure storing the transpose argument description. */
 LIBXSMM_EXTERN_C LIBXSMM_PACKED(struct LIBXSMM_RETARGETABLE) libxsmm_trans_descriptor {
   /** LD, M, and N. */
@@ -287,6 +340,7 @@ LIBXSMM_EXTERN_C typedef union LIBXSMM_RETARGETABLE libxsmm_code_pointer {
   intptr_t ival;
   libxsmm_xmmfunction xgemm; /* GEMM: smm, dmm, wimm, or void-function */
   libxsmm_xmcopyfunction xmatcopy;
+  libxsmm_xmeltwfunction xmateltw;
   libxsmm_xtransfunction xtrans;
   libxsmm_pgemm_xfunction xpgemm;
   libxsmm_getrf_xfunction xgetrf;
@@ -318,10 +372,6 @@ LIBXSMM_EXTERN_C struct LIBXSMM_RETARGETABLE libxsmm_dnn_layer {
   libxsmm_dnn_conv_fuse_op fuse_ops;
   libxsmm_dnn_conv_option options;
 
-  /* These are the batchnorm handles in case of fusion */
-  libxsmm_dnn_fusedbatchnorm* pre_bn;
-  libxsmm_dnn_fusedbatchnorm* post_bn;
-
   /* additional size for internal data types */
   int ifhp;
   int ifwp;
@@ -334,18 +384,12 @@ LIBXSMM_EXTERN_C struct LIBXSMM_RETARGETABLE libxsmm_dnn_layer {
   int blocksifm;
   int blocksofm;
   int fwd_ofw_rb;
-  int fwd_ofw_rb_2;
   int fwd_ofh_rb;
   int bwd_ofw_rb;
   int bwd_ofh_rb;
   int upd_ofw_rb;
   int upd_ofh_rb;
   int fm_lp_block; /* additional blocking for low precision datatypes of feature maps */
-  int upd_use_thread_fil;
-  int upd_use_external_reduce;
-  int filter_transposed;
-  int nBImg;
-  int nbImg;
   int blocksifm_blocking;
   int blocksofm_blocking;
   int avoid_acc_load;
@@ -354,20 +398,12 @@ LIBXSMM_EXTERN_C struct LIBXSMM_RETARGETABLE libxsmm_dnn_layer {
   int pack_input_bwd;
   int spread_input_bwd;
   int weight_copies;
-  int fuse_batchstats_fwd;
-  int fuse_batchstats_bwd;
-  int fuse_eltwise_bwd;
-  int fuse_relu_bwd;
-  int use_lp_kernel;
-  int use_vperm_transposes;
   int loop_order;
   int use_ofm_parallelization;
   int use_ifm_parallelization;
   int avoid_fmas_in_rim;
-  int avoid_init_weights;
   int upd_use_batchreduce;
   int upd_pack_input;
-  int upd_img_br_block;
   int upd_loop_order;
   int upd_linearized_tasklist;
   int upd_avoid_rim_fmas;
@@ -389,8 +425,19 @@ LIBXSMM_EXTERN_C struct LIBXSMM_RETARGETABLE libxsmm_dnn_layer {
   int use_hybrid_imgofm_parallelization;
   int compute_pixels;
   int upd_trans_w_only;
+  int fwd_padding_copy;
+  int upd_padding_copy;
+  int block_fwd_oj;
+  int block_fwd_ifm;
+  int block_fwd_ofm;
+  int block_bwd_oj;
+  int block_bwd_ifm;
+  int block_bwd_ofm;
+  int block_upd_ifm;
+  int block_upd_ofm;
 
   libxsmm_xtransfunction tr_kernel;
+  libxsmm_meltwfunction_cvtfp32bf16 fwd_cvtfp32bf16_kernel;
 
   /* internal data representation */
   libxsmm_dnn_tensor* reg_input;
@@ -415,32 +462,39 @@ LIBXSMM_EXTERN_C struct LIBXSMM_RETARGETABLE libxsmm_dnn_layer {
   libxsmm_barrier* barrier;
 
   /* scratch */
-  void* scratch1;
-  size_t scratch1_size;
-  void* scratch2;
-  size_t scratch2_size;
-  void* scratch3;
-  size_t scratch3_size;
-  void* scratch4;             /* TLS: used to reduce weights */
-  size_t scratch4_size;
-  void* scratch5;             /* TLS: copy-buffer (if padding is needed), or [H][W][c-block]-tensor (generic FWD/BWD) */
-  size_t max_scratch5_size;
-  void* scratch6;             /* TLS: output_scratch (generic WU), or float-accumulation buffer */
-  size_t scratch6_size;
-  void* scratch7;             /* TLS: filter_scratch (generic WU) */
-  size_t scratch7_size;
-  size_t minibatch_scratch_size;
-  size_t fwdbwd_scratch_size;
-  int padding_flag;           /* Flag that dictates if we should apply padding in the input */
-  void* scratchIw;            /* Winograd input buffer */
-  size_t scratchIw_size;
-  void* scratchOw;            /* Winograd output buffer */
-  size_t scratchOw_size;
-  void* scratchVk;            /* Winograd weight buffer */
-  size_t scratchVk_size;
+  size_t fwd_packing_padding_scratch_size;
+  size_t fwd_lp_output_full_scratch_size;
+  size_t fwd_lp_output_block_scratch_size;
+  size_t fwd_packing_padding_scratch_offset;
+  size_t fwd_lp_output_full_scratch_offset;
+  size_t fwd_lp_output_block_scratch_offset;
+  size_t fwd_scratch_size;
+
+  size_t bwd_filter_trans_scratch_size;
+  size_t bwd_packing_padding_scratch_size;
+  size_t bwd_lp_input_full_scratch_size;
+  size_t bwd_filter_trans_scratch_offset;
+  size_t bwd_packing_padding_scratch_offset;
+  size_t bwd_lp_input_full_scratch_offset;
+  size_t bwd_scratch_size;
+
+  size_t upd_packing_padding_scratch_size;
+  size_t upd_lp_output_full_scratch_size;
+  size_t upd_lp_input_full_scratch_size;
+  size_t upd_filter_scratch_size;
+  size_t upd_lp_filter_full_scratch_size;
+  size_t upd_packing_padding_scratch_offset;
+  size_t upd_lp_output_full_scratch_offset;
+  size_t upd_lp_input_full_scratch_offset;
+  size_t upd_lp_filter_full_scratch_offset;
+  size_t upd_filter_scratch_offset;
+  size_t upd_scratch_size;
+
+  void* scratch;
+  size_t scratch_size;
 
   libxsmm_code_pointer gemm_fwd;     /* ability to hoist forward GEMMs */
-  libxsmm_code_pointer gemm_fwd2;     /* ability to hoist forward GEMMs */
+  libxsmm_code_pointer gemm_fwd2;    /* ability to hoist forward GEMMs */
 
   unsigned long long *A_offsets;
   unsigned long long *B_offsets;
@@ -453,16 +507,6 @@ LIBXSMM_EXTERN_C struct LIBXSMM_RETARGETABLE libxsmm_dnn_layer {
   libxsmm_code_pointer matcopy_fwd[4];
   libxsmm_code_pointer matcopy_bwd[4];
   libxsmm_code_pointer matcopy_upd[3];
-
-  /* Data structures and metadata related to per-thread private JITing */
-  int block_fwd_oj;
-  int block_fwd_ifm;
-  int block_fwd_ofm;
-  int block_bwd_oj;
-  int block_bwd_ifm;
-  int block_bwd_ofm;
-  int block_upd_ifm;
-  int block_upd_ofm;
 };
 
 LIBXSMM_EXTERN_C struct LIBXSMM_RETARGETABLE libxsmm_dnn_fusedbatchnorm {
@@ -490,6 +534,37 @@ LIBXSMM_EXTERN_C struct LIBXSMM_RETARGETABLE libxsmm_dnn_fusedbatchnorm {
   void* scratch;
 };
 
+LIBXSMM_EXTERN_C struct LIBXSMM_RETARGETABLE libxsmm_dnn_softmaxloss {
+  libxsmm_dnn_softmaxloss_desc desc;
+  libxsmm_dnn_tensor* reg_input;      /* input tensor */
+  libxsmm_dnn_tensor* reg_output;     /* output tensor */
+  libxsmm_dnn_tensor* grad_input;     /* grad input tensor */
+  libxsmm_dnn_tensor* label;          /* labels tensor */
+  libxsmm_barrier* barrier;           /* barrier */
+  int bc;
+  int Bc;
+  int bn;
+  int Bn;
+  float loss;
+  size_t scratch_size;
+  void* scratch;
+};
+
+LIBXSMM_EXTERN_C struct LIBXSMM_RETARGETABLE libxsmm_dnn_optimizer {
+  libxsmm_dnn_optimizer_desc desc;
+  libxsmm_dnn_tensor* reg_filter;      /* filter tensor */
+  libxsmm_dnn_tensor* grad_filter;     /* grad filter tensor */
+  libxsmm_dnn_tensor* master_filter;   /* master filter tensor */
+  libxsmm_barrier* barrier;            /* barrier */
+  int bc;
+  int Bc;
+  int bk;
+  int Bk;
+  int fm_lp_block;
+  size_t scratch_size;
+  void* scratch;
+};
+
 LIBXSMM_EXTERN_C struct LIBXSMM_RETARGETABLE libxsmm_dnn_fusedgroupnorm {
   libxsmm_dnn_fusedgroupnorm_desc desc;
   libxsmm_dnn_tensor* reg_input;      /* input tensor */
@@ -555,6 +630,7 @@ LIBXSMM_EXTERN_C struct LIBXSMM_RETARGETABLE libxsmm_dnn_fullyconnected {
   size_t doutput_scratch_mark;
   void* scratch;
 
+  libxsmm_xtransfunction tr_kernel;
   libxsmm_code_pointer gemm_fwd;     /* ability to hoist forward GEMMs */
   libxsmm_code_pointer gemm_fwd2;    /* ability to hoist forward GEMMs */
   libxsmm_code_pointer gemm_fwd3;    /* ability to hoist forward GEMMs */
@@ -666,6 +742,7 @@ struct LIBXSMM_RETARGETABLE libxsmm_dfsspmdm {
   int ldb;
   int ldc;
   int N_chunksize;
+  unsigned int* permute_operands;
   double* a_dense;
   libxsmm_dmmfunction kernel;
 };
@@ -677,6 +754,7 @@ struct LIBXSMM_RETARGETABLE libxsmm_sfsspmdm {
   int ldb;
   int ldc;
   int N_chunksize;
+  unsigned int* permute_operands;
   float* a_dense;
   libxsmm_smmfunction kernel;
 };
@@ -684,6 +762,7 @@ struct LIBXSMM_RETARGETABLE libxsmm_sfsspmdm {
 typedef enum libxsmm_build_kind {
   LIBXSMM_BUILD_KIND_GEMM       = LIBXSMM_KERNEL_KIND_MATMUL,
   LIBXSMM_BUILD_KIND_MCOPY      = LIBXSMM_KERNEL_KIND_MCOPY,
+  LIBXSMM_BUILD_KIND_MELTW      = LIBXSMM_KERNEL_KIND_MELTW,
   LIBXSMM_BUILD_KIND_TRANS      = LIBXSMM_KERNEL_KIND_TRANS,
   LIBXSMM_BUILD_KIND_PGEMM      = LIBXSMM_KERNEL_KIND_PGEMM,
   LIBXSMM_BUILD_KIND_GETRF      = LIBXSMM_KERNEL_KIND_GETRF,
@@ -732,29 +811,64 @@ typedef enum libxsmm_malloc_flags {
   LIBXSMM_MALLOC_FLAG_SCRATCH = 1,
   LIBXSMM_MALLOC_FLAG_PRIVATE = 2,
   LIBXSMM_MALLOC_FLAG_REALLOC = 4,
-  LIBXSMM_MALLOC_FLAG_MMAP    = 8,
-  LIBXSMM_MALLOC_FLAG_R       = 16,
-  LIBXSMM_MALLOC_FLAG_W       = 32,
-  LIBXSMM_MALLOC_FLAG_X       = 64,
+  LIBXSMM_MALLOC_FLAG_PHUGE   = 8,
+  LIBXSMM_MALLOC_FLAG_PLOCK   = 16,
+  LIBXSMM_MALLOC_FLAG_MMAP    = 32,
+  LIBXSMM_MALLOC_FLAG_R       = 64,
+  LIBXSMM_MALLOC_FLAG_W       = 128,
+  LIBXSMM_MALLOC_FLAG_X       = 256,
   LIBXSMM_MALLOC_FLAG_RW  = LIBXSMM_MALLOC_FLAG_R | LIBXSMM_MALLOC_FLAG_W,
   LIBXSMM_MALLOC_FLAG_WX  = LIBXSMM_MALLOC_FLAG_X | LIBXSMM_MALLOC_FLAG_W,
   LIBXSMM_MALLOC_FLAG_RWX = LIBXSMM_MALLOC_FLAG_X | LIBXSMM_MALLOC_FLAG_RW,
   LIBXSMM_MALLOC_FLAG_VALID       = LIBXSMM_MALLOC_FLAG_SCRATCH |
       LIBXSMM_MALLOC_FLAG_PRIVATE | LIBXSMM_MALLOC_FLAG_REALLOC |
+      LIBXSMM_MALLOC_FLAG_PHUGE   | LIBXSMM_MALLOC_FLAG_PLOCK |
       LIBXSMM_MALLOC_FLAG_MMAP    | LIBXSMM_MALLOC_FLAG_RWX
 } libxsmm_malloc_flags;
 
-/**
- * Format for instance an amount of Bytes like libxsmm_format_size(result, sizeof(result), nbytes, "KMGT", "B", 10).
- * The value returned is in requested/determined unit so that the user can decide about printing the buffer.
- */
-LIBXSMM_API_INTERN size_t libxsmm_format_size(char buffer[32], int buffer_size, size_t nbytes, const char scale[], const char* unit, int base);
+LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE void* (*libxsmm_realloc_fun)(void* /*ptr*/, size_t /*size*/);
 
-/** Returns the type-name of data-type (can be also libxsmm_gemm_precision). */
-LIBXSMM_API_INTERN const char* libxsmm_typename(libxsmm_datatype datatype);
+#if defined(LIBXSMM_MALLOC_HOOK_DYNAMIC)
+LIBXSMM_EXTERN_C typedef struct LIBXSMM_RETARGETABLE libxsmm_malloc_fntype {
+  union { const void* dlsym; void* (*ptr)(size_t, size_t);  } alignmem;
+  union { const void* dlsym; void* (*ptr)(size_t, size_t);  } memalign;
+  union { const void* dlsym; libxsmm_malloc_fun ptr;        } malloc;
+# if defined(LIBXSMM_MALLOC_HOOK_CALLOC)
+  union { const void* dlsym; void* (*ptr)(size_t, size_t);  } calloc;
+# endif
+# if defined(LIBXSMM_MALLOC_HOOK_REALLOC)
+  union { const void* dlsym; libxsmm_realloc_fun ptr;      } realloc;
+# endif
+  union { const void* dlsym; libxsmm_free_fun ptr;          } free;
+} libxsmm_malloc_fntype;
+LIBXSMM_APIVAR_PRIVATE(libxsmm_malloc_fntype libxsmm_malloc_fn);
+#endif
 
-/** Returns the type-size of data-type (can be also libxsmm_gemm_precision). */
-LIBXSMM_API unsigned char libxsmm_typesize(libxsmm_datatype datatype);
+#if (defined(LIBXSMM_BUILD) && (1 < (LIBXSMM_BUILD)))
+/* prototypes for GLIBC internal implementation */
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void* __libc_memalign(size_t alignment, size_t size);
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void* __libc_malloc(size_t size);
+#if defined(LIBXSMM_MALLOC_HOOK_CALLOC)
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void* __libc_calloc(size_t num, size_t size);
+#endif
+#if defined(LIBXSMM_MALLOC_HOOK_REALLOC)
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void* __libc_realloc(void* ptr, size_t size);
+#endif
+LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE void  __libc_free(void* ptr);
+#endif /*(defined(LIBXSMM_BUILD) && (1 < (LIBXSMM_BUILD)))*/
+
+LIBXSMM_API_INTERN void* libxsmm_memalign_internal(size_t alignment, size_t size);
+
+/* See https://sourceware.org/binutils/docs-2.34/ld/Options.html#index-_002d_002dwrap_003dsymbol */
+LIBXSMM_API_INTERN LIBXSMM_ATTRIBUTE_WEAK void* __real_memalign(size_t alignment, size_t size);
+LIBXSMM_API_INTERN LIBXSMM_ATTRIBUTE_WEAK void* __real_malloc(size_t size);
+#if defined(LIBXSMM_MALLOC_HOOK_CALLOC)
+LIBXSMM_API_INTERN LIBXSMM_ATTRIBUTE_WEAK void* __real_calloc(size_t num, size_t size);
+#endif
+#if defined(LIBXSMM_MALLOC_HOOK_REALLOC)
+LIBXSMM_API_INTERN LIBXSMM_ATTRIBUTE_WEAK void* __real_realloc(void* ptr, size_t size);
+#endif
+LIBXSMM_API_INTERN LIBXSMM_ATTRIBUTE_WEAK void __real_free(void* ptr);
 
 /** Retrieve internal information about a buffer (default memory domain). */
 LIBXSMM_API int libxsmm_get_malloc_xinfo(const void* memory, size_t* size, int* flags, void** extra);
@@ -780,9 +894,6 @@ LIBXSMM_API_INTERN int libxsmm_xset_scratch_allocator(LIBXSMM_LOCK_TYPE(LIBXSMM_
 LIBXSMM_API_INTERN int libxsmm_xget_scratch_allocator(LIBXSMM_LOCK_TYPE(LIBXSMM_LOCK)* lock,
   const void** context, libxsmm_malloc_function* malloc_fn, libxsmm_free_function* free_fn);
 
-/** intern function to calculate blockings, that's private API hence it's in this function */
-LIBXSMM_API_INTERN libxsmm_dnn_err_t libxsmm_dnn_get_feature_map_blocks( int C, int K, int* C_block, int* K_block, int* fm_lp_block, libxsmm_dnn_datatype datatype_in, libxsmm_dnn_datatype datatype_out );
-
 /**
  * Attribute memory allocation and protect with only the necessary flags.
  * This procedure is expected to run only one time per buffer, and may
@@ -802,12 +913,24 @@ LIBXSMM_API_INTERN void libxsmm_xfree(const void* memory, int check);
 /** Like libxsmm_release_scratch, but takes a lock (can be NULL). */
 LIBXSMM_API_INTERN void libxsmm_xrelease_scratch(LIBXSMM_LOCK_TYPE(LIBXSMM_LOCK)* lock);
 
+/**
+ * Format for instance an amount of Bytes like libxsmm_format_size(result, sizeof(result), nbytes, "KMGT", "B", 10).
+ * The value returned is in requested/determined unit so that the user can decide about printing the buffer.
+ */
+LIBXSMM_API_INTERN size_t libxsmm_format_size(char buffer[32], int buffer_size, size_t nbytes, const char scale[], const char* unit, int base);
+
+/** Returns the type-name of data-type (can be also libxsmm_gemm_precision). */
+LIBXSMM_API_INTERN const char* libxsmm_typename(libxsmm_datatype datatype);
+
 /** Determines the given value in double-precision based on the given type. */
 LIBXSMM_API_INTERN int libxsmm_dvalue(libxsmm_datatype datatype, const void* value, double* dvalue);
 
 /** Services a build request, and (optionally) registers the code (use regindex=LIBXSMM_CAPACITY_REGISTRY for unmanaged code). */
 LIBXSMM_API_INTERN int libxsmm_build(const libxsmm_build_request* request, unsigned int regindex, libxsmm_code_pointer* code);
 
+/** Returns the type-size of data-type (can be also libxsmm_gemm_precision). */
+LIBXSMM_API unsigned char libxsmm_typesize(libxsmm_datatype datatype);
+
 LIBXSMM_EXTERN_C typedef struct LIBXSMM_RETARGETABLE libxsmm_kernel_xinfo {
   /** Non-zero of kernel is registered. */
   unsigned int registered;
@@ -831,6 +954,11 @@ LIBXSMM_API_INTERN void libxsmm_memory_finalize(void);
 LIBXSMM_API_INTERN void libxsmm_dnn_init(int target_arch);
 LIBXSMM_API_INTERN void libxsmm_dnn_finalize(void);
 
+/** intern function to calculate blockings, that's private API hence it's in this function */
+LIBXSMM_API_INTERN libxsmm_dnn_err_t libxsmm_dnn_get_feature_map_blocks(
+  int C, int K, int* C_block, int* K_block, int* fm_lp_block,
+  libxsmm_dnn_datatype datatype_in, libxsmm_dnn_datatype datatype_out);
+
 /** Global lock; create an own lock for an independent domain. */
 LIBXSMM_APIVAR_PUBLIC(LIBXSMM_LOCK_TYPE(LIBXSMM_LOCK) libxsmm_lock_global);
 /** Determines whether a threaded implementation is synchronized or not. */
@@ -858,8 +986,6 @@ LIBXSMM_APIVAR_PRIVATE(double libxsmm_timer_scale);
 LIBXSMM_APIVAR_PRIVATE(unsigned int libxsmm_statistic_num_spmdm);
 /** Counts the maximum number of thread that have been active. */
 LIBXSMM_APIVAR_PRIVATE(unsigned int libxsmm_thread_count);
-/** Security-enhanced environment. */
-LIBXSMM_APIVAR_PRIVATE(int libxsmm_se);
 
 #if (0 != LIBXSMM_SYNC)
 LIBXSMM_APIVAR_PRIVATE(LIBXSMM_TLS_TYPE libxsmm_tlskey);
