@@ -54,12 +54,12 @@ if (use_2d_blocking == 1) {
   column_teams = handle->fwd_column_teams;
   my_col_id = ltid % column_teams;
   my_row_id = ltid / column_teams;
-  im_tasks_per_thread = (nBlocksMB + row_teams-1)/row_teams;
-  in_tasks_per_thread = (nBlocksOFm + column_teams-1)/column_teams;
-  my_im_start = LIBXSMM_MIN( my_row_id * im_tasks_per_thread, nBlocksMB);
-  my_im_end = LIBXSMM_MIN( (my_row_id+1) * im_tasks_per_thread, nBlocksMB);
-  my_in_start = LIBXSMM_MIN( my_col_id * in_tasks_per_thread, nBlocksOFm);
-  my_in_end = LIBXSMM_MIN( (my_col_id+1) * in_tasks_per_thread, nBlocksOFm);
+  im_tasks_per_thread = LIBXSMM_UPDIV(nBlocksMB, row_teams);
+  in_tasks_per_thread = LIBXSMM_UPDIV(nBlocksOFm, column_teams);
+  my_im_start = LIBXSMM_MIN(my_row_id * im_tasks_per_thread, nBlocksMB);
+  my_im_end = LIBXSMM_MIN((my_row_id+1) * im_tasks_per_thread, nBlocksMB);
+  my_in_start = LIBXSMM_MIN(my_col_id * in_tasks_per_thread, nBlocksOFm);
+  my_in_end = LIBXSMM_MIN((my_col_id+1) * in_tasks_per_thread, nBlocksOFm);
 }
 
 /* lazy barrier init */
