@@ -1,30 +1,10 @@
 /******************************************************************************
-** Copyright (c) 2017-2018, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Kunal Banerjee (Intel Corp.)
 ******************************************************************************/
@@ -49,7 +29,7 @@ element_input_type  *xt    = (element_input_type* )handle->xt->data;
 element_input_type  *hpD   = (element_input_type* )handle->hp->data;
 element_filter_type *w     = (element_filter_type*)handle->w->data;
 element_filter_type *r     = (element_filter_type*)handle->r->data;
-element_output_type *ht    = handle->ht ? (element_output_type*)handle->ht->data : (element_output_type*)NULL;
+element_output_type *ht    = (element_output_type*)(handle->ht ? handle->ht->data : NULL);
 element_output_type *it    = (element_output_type*)handle->it->data;
 element_output_type *ct    = (element_output_type*)handle->cit->data;
 element_output_type *ft    = (element_output_type*)handle->ft->data;
@@ -148,14 +128,14 @@ LIBXSMM_VLA_DECL(2, element_output_type, hT,  scratch_hT, N);
 LIBXSMM_VLA_DECL(2, element_output_type, oT,  scratch_oT, N);
 element_output_type *dout_ptr = NULL;
 /* define batch-reduce gemm kernels */
-const libxsmm_smmfunction_reducebatch batchreduce_kernela  = libxsmm_smmdispatch_reducebatch( bc, bn, bk, &bc, &K, &C,  NULL, NULL, NULL, NULL );
+const libxsmm_smmfunction_reducebatch_addr batchreduce_kernela  = libxsmm_smmdispatch_reducebatch_addr( bc, bn, bk, &bc, &K, &C,  NULL, NULL, NULL, NULL );
 #if 0
-const libxsmm_smmfunction_reducebatch batchreduce_kernelb  = libxsmm_smmdispatch_reducebatch( bk, bk, bn, &bk, &N, &bk, NULL, NULL, NULL, NULL );
-const libxsmm_smmfunction_reducebatch batchreduce_kernelc  = libxsmm_smmdispatch_reducebatch( bk, bc, bn, &bk, &N, &bk, NULL, NULL, NULL, NULL );
+const libxsmm_smmfunction_reducebatch_addr batchreduce_kernelb  = libxsmm_smmdispatch_reducebatch_addr( bk, bk, bn, &bk, &N, &bk, NULL, NULL, NULL, NULL );
+const libxsmm_smmfunction_reducebatch_addr batchreduce_kernelc  = libxsmm_smmdispatch_reducebatch_addr( bk, bc, bn, &bk, &N, &bk, NULL, NULL, NULL, NULL );
 #endif
-const libxsmm_smmfunction_reducebatch batchreduce_kernelb1 = libxsmm_smmdispatch_reducebatch( bk, bk, bn, &K,  &N, &bk, NULL, NULL, NULL, NULL );
-const libxsmm_smmfunction_reducebatch batchreduce_kernelc1 = libxsmm_smmdispatch_reducebatch( bk, bc, bn, &K,  &N, &bk, NULL, NULL, NULL, NULL );
-const libxsmm_smmfunction_reducebatch batchreduce_kerneld  = libxsmm_smmdispatch_reducebatch( bk, bn, bk, &bk, &K, &K,  NULL, NULL, NULL, NULL );
+const libxsmm_smmfunction_reducebatch_addr batchreduce_kernelb1 = libxsmm_smmdispatch_reducebatch_addr( bk, bk, bn, &K,  &N, &bk, NULL, NULL, NULL, NULL );
+const libxsmm_smmfunction_reducebatch_addr batchreduce_kernelc1 = libxsmm_smmdispatch_reducebatch_addr( bk, bc, bn, &K,  &N, &bk, NULL, NULL, NULL, NULL );
+const libxsmm_smmfunction_reducebatch_addr batchreduce_kerneld  = libxsmm_smmdispatch_reducebatch_addr( bk, bn, bk, &bk, &K, &K,  NULL, NULL, NULL, NULL );
 
 /* Auxiliary arrays for batch-reduce gemm calls */
 const element_filter_type *A_array[1024];
@@ -294,6 +274,7 @@ for (j = t-1; j >= 0; --j) {
     if (0 == j) {
       libxsmm_internal_matrix_sub_ld(             bk, bn, K, &LIBXSMM_VLA_ACCESS(2, hp, in, ik, K),        &LIBXSMM_VLA_ACCESS(3, f, j, in, ik, N, K), &LIBXSMM_VLA_ACCESS(2, t2, in, ik, K) );
     } else {
+      LIBXSMM_ASSERT(NULL != ht); /* coverity[var_deref_op] */
       libxsmm_internal_matrix_sub_ld(             bk, bn, K, &LIBXSMM_VLA_ACCESS(3, h, j-1, in, ik, N, K), &LIBXSMM_VLA_ACCESS(3, f, j, in, ik, N, K), &LIBXSMM_VLA_ACCESS(2, t2, in, ik, K) );
     }
     libxsmm_internal_matrix_eltwise_mult_ld(      bk, bn, K, &LIBXSMM_VLA_ACCESS(2, t1, in, ik, K),   &LIBXSMM_VLA_ACCESS(2, t2, in, ik, K), &LIBXSMM_VLA_ACCESS(2, dc, in, ik, K) );
