@@ -12,15 +12,50 @@
 #define PROFILE
 #endif
 
+#define MATRIX_CVT_BF16_FP32_LD(m, n, ld, _src, _dst) \
+do { \
+  libxsmm_bfloat16 *src = _src; \
+  float *dst = _dst; \
+  libxsmm_blasint __i,__j; \
+  for ( __j = 0; __j < n; ++__j ) { \
+    for ( __i = 0; __i < m; __i+=16 ) { \
+      _mm512_storeu_ps((float*)&dst[(__j*ld)+__i], LIBXSMM_INTRINSICS_MM512_CVTPBH_PS(_mm256_loadu_si256((__m256i*)&src[(__j*ld)+__i]))); \
+    } \
+  } \
+} while (0)
+
+#define MATRIX_BCST_CVT_BF16_FP32_COLVECTOR_LD(m, n, ld, _srcdst, _colv) \
+do { \
+  libxsmm_bfloat16 *colv = _colv; \
+  float *srcdst = _srcdst; \
+  libxsmm_blasint __i,__j; \
+  for ( __j = 0; __j < n; ++__j ) { \
+    for ( __i = 0; __i < m; __i+=16 ) { \
+      _mm512_storeu_ps((float*)&srcdst[(__j*ld)+__i], LIBXSMM_INTRINSICS_MM512_CVTPBH_PS(_mm256_loadu_si256((__m256i*)&colv[__i]))); \
+    } \
+  } \
+} while (0)
+
+#define MATRIX_BCST_CVT_BF16_FP32_COLVECTOR_CONST_LD(m, n, ld, _srcdst, _colv, const_bias) \
+do { \
+  libxsmm_bfloat16 *colv = _colv; \
+  float *srcdst = _srcdst; \
+  libxsmm_blasint __i,__j; \
+  __m512 vbias = _mm512_set1_ps(const_bias); \
+  for ( __j = 0; __j < n; ++__j ) { \
+    for ( __i = 0; __i < m; __i+=16 ) { \
+      _mm512_storeu_ps((float*)&srcdst[(__j*ld)+__i], _mm512_add_ps(vbias, LIBXSMM_INTRINSICS_MM512_CVTPBH_PS(_mm256_loadu_si256((__m256i*)&colv[__i])))); \
+    } \
+  } \
+} while (0)
+
 /* helper variables */
-libxsmm_blasint /*j,*/ ik, ikb, in, ic, /*icb,*/ inik, BF, /*CB, CB_BLOCKS, KB_BLOCKS,*/ ikic, jk, jc;
+libxsmm_blasint j, ik, ikb, in, ic, /*icb,*/ inik, BF, CB, CB_BLOCKS, KB_BLOCKS, ikic, jk, jc;
 /* input sizes */
 const libxsmm_blasint K =  handle->desc.K;
 const libxsmm_blasint N =  handle->desc.N;
 const libxsmm_blasint C =  handle->desc.C;
-#if 0
 const libxsmm_blasint t =  handle->T;
-#endif
 const libxsmm_blasint bk = handle->bk;
 const libxsmm_blasint bn = handle->bn;
 const libxsmm_blasint bc = handle->bc;
@@ -29,22 +64,17 @@ const libxsmm_blasint kBlocks = K/bk;
 const int lpb = handle->lpb;
 const int bc_lp = bc/lpb;
 const int bk_lp = bk/lpb;
-#if 0
-unsigned long long blocks;
-#endif
+unsigned long long blocks, blocksa, blocksb;
 
 /* define tensors */
-#if 0
 element_input_type  *xt  = (element_input_type* )handle->xt->data;
 element_input_type  *hpD = (element_input_type* )handle->hp->data;
 element_output_type *b   = (element_output_type*)handle->b->data;
-#endif
 element_input_type  *csp = (element_input_type* )handle->csp->data;
 element_filter_type *w   = (element_filter_type*)handle->w->data;
 element_filter_type *r   = (element_filter_type*)handle->r->data;
 element_filter_type *w_scratch   = (element_filter_type*)handle->scratch_w;
 element_filter_type *r_scratch   = (element_filter_type*)handle->scratch_r;
-#if 0
 /* These buffers are scratch for fp32 output of gemms (intermmediate results) */
 float *cst = (float*)handle->cst_scratch;
 float *ht  = (float*)handle->ht_scratch;
@@ -53,10 +83,8 @@ float *ft  = (float*)handle->ft_scratch;
 float *ot  = (float*)handle->ot_scratch;
 float *cit = (float*)handle->cit_scratch;
 float *cot = (float*)handle->cot_scratch;
-#endif
 /* This has to be also upconverted since it is used in the elementwise functions  */
 float *csp_f32 = (float*)handle->csp_scratch;
-#if 0
 /* These are the output bf16 data  */
 element_output_type *cst_bf16 = (element_output_type*)handle->cst->data;
 element_output_type *ht_bf16  = (element_output_type*)handle->ht->data;
@@ -65,7 +93,6 @@ element_output_type *ft_bf16  = (element_output_type*)handle->ft->data;
 element_output_type *ot_bf16  = (element_output_type*)handle->ot->data;
 element_output_type *cit_bf16 = (element_output_type*)handle->cit->data;
 element_output_type *cot_bf16 = (element_output_type*)handle->cot->data;
-#endif
 element_filter_type *wiD = &(w[0]);
 element_filter_type *wcD = &(w[K]);
 element_filter_type *wfD = &(w[2*K]);
@@ -82,18 +109,14 @@ element_filter_type *riD_scratch = &(r_scratch[0]);
 element_filter_type *rcD_scratch = &(r_scratch[K*K]);
 element_filter_type *rfD_scratch = &(r_scratch[2*K*K]);
 element_filter_type *roD_scratch = &(r_scratch[3*K*K]);
-#if 0
 element_output_type *bi  = &(b[0]);
 element_output_type *bd  = &(b[K]);
 element_output_type *bf  = &(b[2*K]);
 element_output_type *bo  = &(b[3*K]);
-#endif
 LIBXSMM_VLA_DECL(2, float,  cp, csp_f32, K);
 LIBXSMM_VLA_DECL(2, element_input_type,  cp_bf16, csp, K);
-#if 0
 LIBXSMM_VLA_DECL(3, element_input_type,  x, xt, N, C);
 LIBXSMM_VLA_DECL(2, element_input_type,  hp, hpD, K);
-#endif
 LIBXSMM_VLA_DECL(5, element_filter_type, wi, wiD_scratch, cBlocks, bc_lp, bk, lpb);
 LIBXSMM_VLA_DECL(5, element_filter_type, wf, wfD_scratch, cBlocks, bc_lp, bk, lpb);
 LIBXSMM_VLA_DECL(5, element_filter_type, wo, woD_scratch, cBlocks, bc_lp, bk, lpb);
@@ -110,7 +133,6 @@ LIBXSMM_VLA_DECL(2, element_filter_type, ri_ck, riD, 4*K);
 LIBXSMM_VLA_DECL(2, element_filter_type, rf_ck, rfD, 4*K);
 LIBXSMM_VLA_DECL(2, element_filter_type, ro_ck, roD, 4*K);
 LIBXSMM_VLA_DECL(2, element_filter_type, rc_ck, rcD, 4*K);
-#if 0
 LIBXSMM_VLA_DECL(3, float, cs, cst, N, K);
 LIBXSMM_VLA_DECL(3, float, h, ht, N, K);
 LIBXSMM_VLA_DECL(3, float, i, it, N, K);
@@ -126,13 +148,10 @@ LIBXSMM_VLA_DECL(3, element_output_type, o_out, ot_bf16, N, K);
 LIBXSMM_VLA_DECL(3, element_output_type, ci_out, cit_bf16, N, K);
 LIBXSMM_VLA_DECL(3, element_output_type, co_out, cot_bf16, N, K);
 /* define batch-reduce gemm kernels */
-const libxsmm_bsmmfunction_reducebatch_addr batchreduce_kernela = libxsmm_bsmmdispatch_reducebatch_addr( bk, bn, bc, &bk, &C, &K, NULL, NULL, NULL, NULL );
-const libxsmm_bsmmfunction_reducebatch_addr batchreduce_kernelb = libxsmm_bsmmdispatch_reducebatch_addr( bk, bn, bk, &bk, &K, &K, NULL, NULL, NULL, NULL );
-/* Auxiliary arrays for batch-reduce gemms */
-const element_filter_type *A_array[1024];
-const element_input_type  *B_array[1024];
+const libxsmm_bsmmfunction_reducebatch_strd batchreduce_kernela = handle->fwd_kernela;
+const libxsmm_bsmmfunction_reducebatch_strd batchreduce_kernelb = handle->fwd_kernelb;
+
 float *cps_ptr = NULL;
-#endif
 
 /* parallelize over C-blocks */
 /* computing first logical thread */
@@ -190,10 +209,8 @@ if (C == 2048 && K == 1024) {
   BF = 2;
 }
 
-#if 0
 CB_BLOCKS = cBlocks/BF;
 KB_BLOCKS = kBlocks/BF;
-#endif
 
 /* Upfront reformatting of W and R */
 /* reformat W */
@@ -244,9 +261,9 @@ if (ltid == 0) {
 #endif
 
 if (use_fused_implementation) {
-/*#include "libxsmm_dnn_rnncell_st_lstm_fwd_nc_kcck_fused_bf16.tpl.c"*/
+#include "libxsmm_dnn_rnncell_st_lstm_fwd_nc_kcck_fused_bf16.tpl.c"
 } else {
-/*#include "libxsmm_dnn_rnncell_st_lstm_fwd_nc_kcck_diffused_bf16.tpl.c"*/
+#include "libxsmm_dnn_rnncell_st_lstm_fwd_nc_kcck_diffused_bf16.tpl.c"
 }
 
 #ifdef PROFILE
