@@ -1,30 +1,10 @@
 /******************************************************************************
-** Copyright (c) 2016-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** a PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Nadathur Satish (Intel Corp.)
 ******************************************************************************/
@@ -58,22 +38,22 @@ else {
   int nrows = ((mb + 1)*handle->bm > handle->m)?(handle->m - (mb)*handle->bm):handle->bm;
   int ncols = ((kb + 1)*handle->bk > handle->k)?(handle->k - (kb)*handle->bk):handle->bk;
   /*printf("nrows: %d, ncols: %d\n", nrows, ncols);*/
-  int ncols_aligned = ncols / (4*SIMD_WIDTH_FP32)*(4*SIMD_WIDTH_FP32);
   const uint16_t * input_ptr = a + block_offset;
   uint16_t * rowidx_ptr = slice.rowidx;
   uint16_t * colidx_ptr = slice.colidx;
   float * values_ptr = (float *)(slice.values);
-#if SIMD_WIDTH_FP32 > 1
-  SIMDTYPE_INT32 vzero = _MM_SET1_INT32(0);
-#endif
-  SIMDTYPE_FP32 vzerof = _MM_SET1_FP32(0.0);
   uint16_t cnt = 0;
-#if (1 == SIMD_WIDTH_FP32)
-  ncols_aligned = 0;
+#if SIMD_WIDTH_FP32 > 1
+  const SIMDTYPE_INT32 vzero = _MM_SETZERO_INT32();
+  const SIMDTYPE_FP32 vzerof = _MM_SETZERO_FP32();
+  const int ncols_aligned = ncols / (4*SIMD_WIDTH_FP32)*(4*SIMD_WIDTH_FP32);
+#else
+  const int ncols_aligned = 0;
 #endif
   for (i = 0; i < nrows; i++) {
     rowidx_ptr[i] = cnt;
     if ('T' == transa || 't' == transa) {
+#if SIMD_WIDTH_FP32 > 1
       for (k = 0; k < ncols_aligned; k += 4*SIMD_WIDTH_FP32) {
         int vals[32];
         int kk;
@@ -95,7 +75,7 @@ else {
           COMPRESS_FP32(v4, k + 3*SIMD_WIDTH_FP32, m4, cnt);
         }
       }
-
+#endif
       for (k = ncols_aligned; k < ncols; k++) {
         uint16_t v1tmp = input_ptr[k*handle->m + i];
         union {int i; float f; } v1tmp_int;
@@ -108,6 +88,7 @@ else {
       }
     }
     else {
+#if SIMD_WIDTH_FP32 > 1
       for (k = 0; k < ncols_aligned; k += 4*SIMD_WIDTH_FP32) {
         SIMDTYPE_INT32 v1tmp, v2tmp;
         SIMDTYPE_FP32 v1, v2, v3, v4;
@@ -127,6 +108,7 @@ else {
         COMPRESS_FP32(v3, k + 2*SIMD_WIDTH_FP32, m3, cnt);
         COMPRESS_FP32(v4, k + 3*SIMD_WIDTH_FP32, m4, cnt);
       }
+#endif
       for (k = ncols_aligned; k < ncols; k++) {
         uint16_t v1tmp = input_ptr[i*handle->k + k];
         union {int i; float f; } v1tmp_int;
